\set{capture}{hidden}
\begin{code}

module C.Constants (
	meaningOfFloatingConstant,
	meaningOfIntegerConstant,
	meaningOfCharacterConstant,
	meaningOfStringLiteral,
	meaningOfConstant,
) where

import Control.Monad.Fix
import Additional
import C.Variables
import C.Syntax
import C.Scopes
import C.Types
import {-# SOURCE #-} C.Implementation
import Lambda

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Constants}
\set{title}		{Constants}
\begin{section}

In the concrete syntax of the C programming language, most predetermined numeric values are represented by lexical tokens known as «constants». Their collective structure is defined by the following trivial data type definitions:

\IndexType		{ConstantT}					{constant (\tstar)}

\begin{syntax}
\InsertHaskell		{ConstantT}
\end{syntax}

Since, in this work, we do not concern ourselves with details of a textual representation of C programs, the “"FloatingConstantT"”, “"IntegerConstantT"” and “"CharacterConstantT"” constructs, as well as a similar syntactic entity “"StringLiteralT"” described later in Section \ref{C/Expressions}, are treated as abstract data types with an unspecified implementation. Their precise meanings are derived from the entity's lexical syntax and, in this work, are represented by the following four unspecified Haskell functions:

\IndexType		{FloatingConstantT}				{floating_constant (\tstar)}
\IndexType		{IntegerConstantT}				{integer_constant (\tstar)}
\IndexType		{CharacterConstantT}				{character_constant (\tstar)}
\IndexType		{StringLiteralT}				{string_literal (\tstar)}

\BeginHaskellToken	{meaningOfFloatingConstant}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{fc}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{FloatingConstantT}
  \HaskellResult							{_M{--}(TypeT, Rational)}
\EndHaskellToken

\BeginHaskellToken	{meaningOfIntegerConstant}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ic}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{IntegerConstantT}
  \HaskellResult							{_M{--}(TypeT, Integer)}
\EndHaskellToken

\BeginHaskellToken	{meaningOfCharacterConstant}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{cc}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{CharacterConstantT}
  \HaskellResult							{_M{--}(TypeT, Integer)}
\EndHaskellToken

\BeginHaskellToken	{meaningOfStringLiteral}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{sc}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{StringLiteralT}
  \HaskellResult							{_M{--}(TypeT, DataT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfFloatingConstant}			{D_FC floating_constant (M type rational) ((monad_fix M))}
\IndexTerm		{meaningOfIntegerConstant}			{D_IC integer_constant (M type integer) ((monad_fix M))}
\IndexTerm		{meaningOfCharacterConstant}			{D_CC character_constant (M type integer) ((monad_fix M))}
\IndexTerm		{meaningOfStringLiteral}			{D_SC string_literal (M type data) ((monad_fix M))}

\begin{code}
meaningOfFloatingConstant	{-[*]-} :: (MonadFix _M, Monad _M) => FloatingConstantT 	-> _M{--}(TypeT, Rational)
meaningOfIntegerConstant 	{-[*]-} :: (MonadFix _M, Monad _M) => IntegerConstantT 		-> _M{--}(TypeT, Integer)
meaningOfCharacterConstant 	{-[*]-} :: (MonadFix _M, Monad _M) => CharacterConstantT 	-> _M{--}(TypeT, Integer)
meaningOfStringLiteral	 	{-[*]-} :: (MonadFix _M, Monad _M) => StringLiteralT		-> _M{--}(TypeT, DataT VarT)
\end{code}

\set{capture}		{hidden}
\begin{code}
meaningOfFloatingConstant	{-sem-}(FloatingValue 	(t) (x)) = return $ (t, x)
meaningOfIntegerConstant 	{-sem-}(IntegerValue 	(t) (x)) = return $ (t, x)
meaningOfCharacterConstant 	{-sem-}(CharacterValue 	(t) (x)) = return $ (t, x)
meaningOfStringLiteral	 	{-sem-}(StringValue 	(t) (x)) = return $ (t, x)
\end{code}

observing that every such entity denotes a C type paired with the entity's actual value, as represented by a rational number for floating constants, an integer quantity for integer and character constants and an Etude object data specification for string literals. In all cases, the entity's denotation is derived within some unspecified exception monad "_M", in order to provision for any error diagnostics as stipulated earlier in Section \ref{C/Notation}.

On the other hand, every «enumeration constant» is represented simply by an identifier with a “"ConstantLinkage"” linkage, so that its concrete syntax is defined in Haskell as follows:

\IndexType		{EnumerationConstantT}				{enumeration_constant (\tstar)}
\IndexCons		{EnumerationConstantT'}				{ identifier (enumeration_constant)}

\begin{syntax}
\InsertHaskell		{EnumerationConstantT}
\end{syntax}

Using these definitions, the type and rational value of every C constant can be derived in the context of its current scope "_S" by the following construction:

\BeginHaskellToken	{meaningOfConstant}			{Open}	{\ZapfChancery{D}\sub{\SmallCaps{c}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S {->-}, ConstantT)}
  \HaskellResult							{_M{--}(TypeT, Rational)}
\EndHaskellToken

\IndexTerm		{meaningOfConstant}				{D_C constant S (M type rational) ((monad_fix M))}

\begin{haskell}
meaningOfConstant {-[*]-} :: (MonadFix _M, Monad _M) => (S {->-}, ConstantT) -> _M{--}(TypeT, Rational)

meaningOfConstant {-sem-}(_S {->-}, FloatingConstant floating_constant)			= meaningOfFloatingConstant	{--}(floating_constant)
meaningOfConstant {-sem-}(_S {->-}, IntegerConstant integer_constant) 			= meaningOfIntegerConstant	{--}(integer_constant)
meaningOfConstant {-sem-}(_S {->-}, CharacterConstant character_constant) 		= meaningOfCharacterConstant	{--}(character_constant)

meaningOfConstant {-sem-}(_S {->-}, EnumerationConstant enumeration_constant) 		= do
	require $ (enumeration_constant @@ dom{--}(_S) && linkageOfDesignator{--}(d) == {-sem-}(ConstantLinkage {-sem-}(integerOfLinkage{--}(linkageOfDesignator{--}(d)))))
	return $ (typeOfDesignator{--}(d), toRational (integerOfLinkage{--}(linkageOfDesignator{--}(d))))

 where d = _S#(enumeration_constant)
\end{haskell}

\set{capture}		{hidden}
\begin{code}
meaningOfConstant {-[*]-} :: (MonadFix _M, Monad _M) => (S {->-}, ConstantT) -> _M{--}(TypeT, Rational)

meaningOfConstant {-sem-}(_S {->-}, FloatingConstant floating_constant)			= do	(t, x) <- meaningOfFloatingConstant	{--}(floating_constant)
											 	return $ (t, x)
meaningOfConstant {-sem-}(_S {->-}, IntegerConstant integer_constant) 			= do	(t, x) <- meaningOfIntegerConstant	{--}(integer_constant)
												return $ (t, toRational x)
meaningOfConstant {-sem-}(_S {->-}, CharacterConstant character_constant) 		= do	(t, x) <- meaningOfCharacterConstant	{--}(character_constant)
												return $ (t, toRational x)
meaningOfConstant {-sem-}(_S {->-}, EnumerationConstant enumeration_constant) 		= do
	require $ (enumeration_constant @@ dom{--}(_S) && linkageOfDesignator{--}(d) == {-sem-}(ConstantLinkage {-sem-}(integerOfLinkage{--}(linkageOfDesignator{--}(d)))))
	return $ (typeOfDesignator{--}(d), toRational (integerOfLinkage{--}(linkageOfDesignator{--}(d))))

 where d = _S#(enumeration_constant)
\end{code}

so that every well-formed enumeration constant must always represent a variable identifier that is bound in the current variable scope "_S" to a designator with a constant linkage.

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%