\set{capture}{hidden}
\begin{code}

{-# LANGUAGE RecursiveDo #-}

module C.Declarations (
	meaningOfDeclarationListOpt,
	meaningOfDeclarationSpecifiersOpt,
	meaningOfStorageClassSpecifiers,
	meaningOfTypeSpecifiers,
	meaningOfStructOrUnionSpecifier,
	isValidMember,
	zeroMember,
	meaningOfStructDeclarationList,
	meaningOfStructDeclaratorList,
	meaningOfEnumSpecifier,
	meaningOfEnumeratorList,
	meaningOfTagSpecifiers,
	meaningOfTypeQualifiers,
	meaningOfDeclarator,
	meaningOfArrayLengthOpt,
	meaningOfPointer,
	listOfIdentifiersOpt,
	meaningOfParameterTypeListOpt,
	meaningOfParameterList,
	meaningOfTypeName,
	meaningOfAbstractDeclaratorOpt,
	meaningOfTypedefName,
	meaningOfInitDeclaratorList,
	defaultDataOfType,
	meaningOfVariableDefinition,
	meaningOfStaticInitialiser,
	meaningOfStaticInitialisers,
	listOfInitialisers,
	membersOfAggregateType,
	shiftData,
	packData,
	meaningOfDynamicInitialiser,
	scopeConstructor,
	scopeDestructor,
) where

import Control.Monad.Fix
import Additional
import C.Syntax
import C.Variables
import C.Types
import C.Scopes
import C.Constants
import C.Expressions
import {-# SOURCE #-} C.Implementation
import Lambda

infixl 6 `shiftData`

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations}
\set{title}		{Declarations}
\begin{section}

In C, the «declaration» syntax provides programs with a concrete means of introducing new entities into a translation union, be it variables, C types or type tags. Further, in Section \ref{C/Declarations/TypeNames} a subset of this syntax doubles as a source code representation of C types, in the form of entities known as «type names». Due to their vastly diverse nature, the structure and semantics of C declarations follow a pattern more irregular than that of any other syntactic entity in the language, which makes for an interesting reading but does not help to simplify their formalisation. In the concrete syntax of the language, most declarations begin with a group of «declaration specifiers» that are usually followed by one or more «initialised declarators». In particular, the canonical form of a C declaration generally assumes the following structure:

\IndexType		{DeclarationListT}				{declaration_list (\tstar)}
\IndexCons		{D0}						{ declaration (declaration_list)}
\IndexCons		{Dn}						{ declaration_list declaration (declaration_list)}

\IndexType		{DeclarationT}					{declaration (\tstar)}
\IndexCons		{Declaration}					{; declaration_specifiers init_declarator_list_opt (declaration)}

\begin{syntax}
\InsertHaskell		{DeclarationT, \par, DeclarationListT}
\end{syntax}

Under the semantic framework adopted in this work, every such construct is always interpreted in the context of its current variable and tag scopes "_S" and "_T", a set of item definitions "_D" and the current scope index "_I". Similarly to C expressions, its denotation includes any required updates to "_S", "_T" and "_D", together with a set of local variables "_V" and an initialiser term "tau", which, intuitively, prepares the initial content of the corresponding memory-resident object for declarations with automatic and register linkage forms and, in all other cases, defaults to the trivial Etude term “"RET []"”. Formally, these denotations are derived by the following Haskell function:

\BeginHaskellToken	{meaningOfDeclarationListOpt}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{d}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, Maybe DeclarationListT)}
  \HaskellResult							{_M{--}(S, S, D, V, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfDeclarationListOpt}			{D_D declaration_list_opt S S D I (M S S D V term) ((monad_fix M))}

\begin{code}
meaningOfDeclarationListOpt {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, Maybe DeclarationListT) -> _M{--}(S, S, D, V, TermT VarT)
\end{code}

In particular, in a single declaration with at least one declarator provided by its "idl_opt" component, the list of declaration specifiers is split into three components "sc_set", "tq_set" and "ts_set" as described in Section \ref{C/Declarations/Specifiers} below. The later is then promptly converted into a single C type "t_b", known as the declaration's «base type», using a simple algorithm defined later in Section \ref{C/Declarations/TypeSpecifiers}, merged with the associated list of type qualifiers "tq_set" and applied to all of the supplied declarators in "idl", eventually producing a collective meaning of the entire construct as follows:

\begin{code}
meaningOfDeclarationListOpt {-sem-}(_S, _T, _D, _I {->-}, Just (D0 (Declaration (ds) (Just idl) {-;-}))) = do
	(sc_set, tq_set, ts_set) <- meaningOfDeclarationSpecifiersOpt{--}(Just ds)
	(_S1, _T1, _D1, t_b) <- meaningOfTypeSpecifiers{--}(_S, _T, _D, _I {->-}, setToList ts_set)
	(_S2, _T2, _D2, _V, tau) <- meaningOfInitDeclaratorList{--}(_S1, _T1, _D1, _I, setToList sc_set, setToList tq_set `qualifiedType` t_b {->-}, idl)
	return $ (_S2, _T2, _D2, _V, tau)
\end{code}

If, however, no "init_declarator_list" is included in the syntax, then the construct represents a rather different kind of entity known as a «tag declaration». Its declaration specifiers are scrutinised by a dedicated function "meaningOfTagSpecifiers" that is defined separately in \hbox{Section \ref{C/Declarations/Tags}.} In Haskell:

\begin{code}
meaningOfDeclarationListOpt {-sem-}(_S, _T, _D, _I {->-}, Just (D0 (Declaration (ds) ({-hide-}Nothing) {-;-}))) = do
	(_S', _T', _D', t) <- meaningOfTagSpecifiers{--}(_S, _T, _D, _I {->-}, ds)
	return $ (_S', _T', _D', emptyMap, {-sem-}(RET {-hv-}[]))
\end{code}

Last but not least, the denotations of two or more declarations can be naturally combined by obtaining a union of their local variable sets "_V_k" and, further, composing their respective initialiser terms "tau1, tau2 .. tau_n" into a single Etude construct of the form \hbox{“"LET [{-hv-}[] =: tau1] LET [{-hv-}[] =: tau2] .. tau_n"\!”,} so that initialisation of all objects declared earlier in the program is always completed before proceeding with the translation of any following declarations. Under this interpretation, an omitted list of declarations denotes an empty set of local variables "emptyMap" and a singular initialiser term of the form “"RET []"”, as captured by the following pair of translation rules:

\set{continued}		{yes}
\begin{code}
meaningOfDeclarationListOpt {-sem-}(_S, _T, _D, _I {->-}, Just (dl `Dn` d) {-,-}) ---
  =	do	(_S1, _T1, _D1, _V1, tau1) <- meaningOfDeclarationListOpt{--}(_S,   _T,   _D,   _I {->-}, Just dl)
		(_S2, _T2, _D2, _V2, tau2) <- meaningOfDeclarationListOpt{--}(_S1,  _T1,  _D1,  _I {->-}, Just (D0 d))
		return $ (_S2, _T2, _D2, _V1 `mapUnion` _V2, {-sem-}(LET [{-hv-}[] =: tau1] (tau2)))
meaningOfDeclarationListOpt {-sem-}(_S, _T, _D, _I {->-}, Nothing) ---
  =	do	return $ (_S, _T, _D, emptyMap, {-sem-}(RET []))
\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/Specifiers}
\set{title}		{Declaration Specifiers}
\begin{subsection}

In every C declaration of the form “"Declaration (ds) (idl_opt) {-;-}"\!”, the list "ds" represents a sequence of one or more «declaration specifier» entities, constructed from a mixture of three kinds of constructs, known as «storage class specifiers», «type specifiers» and «type qualifiers», respectively. The concrete syntax of such lists is captured naturally by the following Haskell type:

\IndexType		{DeclarationSpecifiersT}			{declaration_specifiers (\tstar)}
\IndexCons		{DeclarationStorageClass}			{ storage_class_specifier declaration_specifiers_opt (declaration_specifiers)}
\IndexCons		{DeclarationSpecifier}				{ type_specifier declaration_specifiers_opt (declaration_specifiers)}
\IndexCons		{DeclarationQualifier}				{ type_qualifier declaration_specifiers_opt (declaration_specifiers)}

\begin{syntax}
\InsertHaskell		{DeclarationSpecifiersT}
\end{syntax}

The three kinds of declaration specifiers are discussed individually in Sections \ref{C/Declarations/StorageClassSpecifiers}, \ref{C/Declarations/TypeSpecifiers} and \ref{C/Declarations/TypeQualifiers} later in this chapter. However, before their meaning can be analysed by the compiler, different kinds of specifies must be segregated from each other, as described by the following simple Haskell construction:

\BeginHaskellToken	{meaningOfDeclarationSpecifiersOpt}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ds}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{Maybe DeclarationSpecifiersT}
  \HaskellResult							{_M{--}(SetT StorageClassSpecifierT, SetT TypeQualifierT, SetT TypeSpecifierT)}
\EndHaskellToken

\IndexTerm		{meaningOfDeclarationSpecifiersOpt}		{D_DS declaration_specifiers_opt (M storage_class_specifiers type_qualifiers type_specifiers)}

\begin{code}
meaningOfDeclarationSpecifiersOpt {-[*]-} :: (MonadFix _M, Monad _M) =>
	(Maybe DeclarationSpecifiersT) ->
	_M{--}(SetT StorageClassSpecifierT, SetT TypeQualifierT, SetT TypeSpecifierT)
\end{code}

In particular, a well-formed C declaration may only ever include at most one of each particular storage class specifier, type qualifier and type specifier entity form. Further, the precise order in which these entities appear in the program's syntax is irrelevant to the declaration's semantics, so that "meaningOfDeclarationSpecifiersOpt" dissects an optional list of declaration specifiers into three independent sets "sc_set", "tq_set" and "ts_set" as follows:

\set{continued}		{yes}
\begin{code}
meaningOfDeclarationSpecifiersOpt {-sem-}(Just (DeclarationStorageClass (storage_class_specifier) (ds_opt))) = do
	(sc_set, tq_set, ts_set) <- meaningOfDeclarationSpecifiersOpt{--}(ds_opt)
	require $ (storage_class_specifier /@ sc_set)
	return $ (sc_set `union` listToSet [storage_class_specifier], tq_set, ts_set)
	
meaningOfDeclarationSpecifiersOpt {-sem-}(Just (DeclarationSpecifier (type_specifier) (ds_opt))) = do
	(sc_set, tq_set, ts_set) <- meaningOfDeclarationSpecifiersOpt{--}(ds_opt)
	require $ (type_specifier /@ ts_set)
	return $ (sc_set, tq_set, ts_set `union` listToSet [type_specifier])
	
meaningOfDeclarationSpecifiersOpt {-sem-}(Just (DeclarationQualifier (type_qualifier) (ds_opt))) = do
	(sc_set, tq_set, ts_set) <- meaningOfDeclarationSpecifiersOpt{--}(ds_opt)
	require $ (type_qualifier /@ tq_set)
	return $ (sc_set, tq_set `union` listToSet [type_qualifier], ts_set)

meaningOfDeclarationSpecifiersOpt {-sem-}({-hide-}Nothing) = return $ (emptySet, emptySet, emptySet)
\end{code}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/StorageClassSpecifiers}
\set{title}		{Storage Class Specifiers}
\begin{subsection}

In the concrete syntax of the language, the five keywords “"TypedefSpecifier"”, “"ExternSpecifier"”, “"StaticSpecifier"”, “"AutoSpecifier"” and “"RegisterSpecifier"” are known as «storage class specifiers»:

\IndexType		{StorageClassSpecifierT}			{storage_class_specifier (\tstar)}
\IndexCons		{TypedefSpecifier}				{typedef (storage_class_specifier)}
\IndexCons		{ExternSpecifier}				{extern (storage_class_specifier)}
\IndexCons		{StaticSpecifier}				{static (storage_class_specifier)}
\IndexCons		{AutoSpecifier}					{auto (storage_class_specifier)}
\IndexCons		{RegisterSpecifier}				{register (storage_class_specifier)}

\begin{syntax}
\InsertHaskell		{StorageClassSpecifierT}
\end{syntax}

Intuitively, such entities describe the linkage of a designator associated with some variable "x" of the C type "t", which is introduced by the surrounding declaration as follows:

\begin{list}

•	All declarations with the “"TypedefSpecifier"” storage class specifier always have the “"TypeLinkage"” linkage form.

•	Otherwise, if a declaration includes the “"ExternSpecifier"” storage class specifier, then "x" has the same linkage as any visible declaration of that identifier with the file scope, i.e., the linkage of an existing designator in "_S" with a scope index of 0. If no such declaration can be found, then "x" is assigned the external linkage form \hbox{“"ExternLinkage (EVar x)"”.}

•	If a declaration in the file scope includes the “"StaticSpecifier"” storage class specifier, then the identifier has the internal linkage form “"InternLinkage (EVar x)"”.

•	However, if such a declaration appears in a nested scope with a non-zero scope index, then the declared identifier is assigned the private linkage “"PrivateLinkage nu"”, in which the Etude variable "nu" is given a globally-unique name "uvar{--}(_D)".

•	Likewise, if a declaration in a nested scope includes the “"AutoSpecifier"” or “"RegisterSpecifier"” storage class specifier, then "x" is assigned, respectively the automatic or register linkage form “"AutoLinkage nu"” or “"RegisterLinkage nu"”, using a similar globally-unique Etude variable "uvar{--}(_D)".

•	Further, such automatic linkage is also assigned to all declarations without any storage class specifiers, provided that the construct appears in a nested scope and that it does not assign a function type for the identifier.

•	On the other hand, a declaration without any storage class specifiers behaves exactly as if the construction included the “"ExternSpecifier"” storage class specifier whenever the entity appears within the file scope or if it designates a C function. In all other cases, "x" is assigned the external linkage form “"ExternLinkage (EVar x)"”.

\end{list}

No other combination of storage class specifiers may ever appear in a well-formed C declaration. In particular, as a direct consequence of this definition, any given C declaration can include at most one storage class specifier entity and no declaration that appears in the file scope may contain the “"AutoSpecifier"” or “"ExternSpecifier"” storage class specifier forms. In Haskell, the above rules can be formalised as follows:

\BeginHaskellToken	{meaningOfStorageClassSpecifiers}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{sc}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, [StorageClassSpecifierT])}
  \HaskellResult							{_M{--}(LinkageT)}
\EndHaskellToken

\IndexTerm		{meaningOfStorageClassSpecifiers}		{D_SC storage_class_specifiers}

\set{continued}		{yes}
\begin{code}
meaningOfStorageClassSpecifiers {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, TypeT, IdentifierT {->-}, [StorageClassSpecifierT]) -> _M{--}(LinkageT)

meaningOfStorageClassSpecifiers {-sem-}(_S, _T, _D, _I, t, x {->-}, sc_set)

  | sc_set == {-sem-}[TypedefSpecifier] ---
  	 = return {-sem-}(TypeLinkage)
  | sc_set == {-sem-}[ExternSpecifier] 		&& (x @@ dom{--}(_S) && indexOfDesignator{--}(_S#(x)) == 0) ---
  	 = return $ linkageOfDesignator{--}(_S#(x))
  | sc_set == {-sem-}[ExternSpecifier] 		&& (x /@ dom{--}(_S) || indexOfDesignator{--}(_S#(x)) /= 0) ---
  	 = return {-sem-}(ExternLinkage (EVar x))
  | sc_set == {-sem-}[StaticSpecifier] 		&& (_I == 0) ---
  	 = return {-sem-}(InternLinkage (EVar x))
  | sc_set == {-sem-}[StaticSpecifier] 		&& (_I /= 0) ---
  	 = return {-sem-}(PrivateLinkage {-sem-}(uvar{--}(_D)))
  | sc_set == {-sem-}[AutoSpecifier] 		&& (_I /= 0) ---
  	 = return {-sem-}(AutoLinkage {-sem-}(uvar{--}(_D)))
  | sc_set == {-sem-}[RegisterSpecifier] 	&& (_I /= 0) ---
  	 = return {-sem-}(RegisterLinkage {-sem-}(uvar{--}(_D)))
  | sc_set == {-sem-}[] 			&& not (isFunctionType{--}(t)) && (_I /= 0) ---
  	 = return {-sem-}(AutoLinkage {-sem-}(uvar{--}(_D)))
  | sc_set == {-sem-}[] 			&& not (isFunctionType{--}(t)) && (_I == 0) ---
  	 = return {-sem-}(ExternLinkage (EVar x))
  | sc_set == {-sem-}[] 			&& isFunctionType{--}(t) ---
  	 = meaningOfStorageClassSpecifiers {-sem-}(_S, _T, _D, _I, t, x {->-}, {-raw-}[ExternSpecifier])
  | otherwise ---
  	 = fail ("Semantic error")
\end{code}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/TypeSpecifiers}
\set{title}		{Type Specifiers}
\begin{subsection}

In C, most simple types are represented by sets of entities known as «type specifiers». The C Standard defines nine type specifier keywords and three composite type specifier structures, whose concrete syntax is captured collectively by the following grammar:

\IndexType		{TypeSpecifierT}				{type_specifier (\tstar)}
\IndexCons		{VoidSpecifier}					{void (type_specifier)}
\IndexCons		{CharSpecifier}					{char (type_specifier)}
\IndexCons		{ShortSpecifier}				{short (type_specifier)}
\IndexCons		{IntSpecifier}					{int (type_specifier)}
\IndexCons		{LongSpecifier}					{long (type_specifier)}
\IndexCons		{FloatSpecifier}				{float (type_specifier)}
\IndexCons		{DoubleSpecifier}				{double (type_specifier)}
\IndexCons		{SignedSpecifier}				{signed (type_specifier)}
\IndexCons		{UnsignedSpecifier}				{unsigned (type_specifier)}
\IndexCons		{StructOrUnionSpecifier}			{ struct_or_union_specifier (type_specifier)}
\IndexCons		{EnumSpecifier}					{ enum_specifier (type_specifier)}
\IndexCons		{TypedefNameSpecifier}				{ typedef_name (type_specifier)}

\begin{syntax}
\InsertHaskell		{TypeSpecifierT}
\end{syntax}

The precise syntax and semantics of the “"enum_specifier"”, “"struct_or_union_specifier"” and “"typedef_name"” type specifier forms is defined separately in Sections \ref{C/Declarations/StructAndUnionSpecifiers}, \ref{C/Declarations/EnumerationSpecifiers} and \ref{C/Declarations/TypeDefinitions} later in this chapter. Otherwise, every valid sorted list of such entities denotes a predetermined C type from Section \ref{C/Types}, such that, intuitively, each meaningful list of type specifiers is mapped to an abstract C type with a lexically-identical name, except that all arithmetic types other than “"PlainCharType"” and “"PlainIntType"” are explicitly marked with the “"Signed"\,” or “"Unsigned"\,” keyword and that “"PlainIntType"” is generally excluded from the names of “"ShortType"” and “"LongType"” type forms. Formally, this correspondence is captured by the following Haskell definition:

\BeginHaskellToken	{meaningOfTypeSpecifiers}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ts}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, [TypeSpecifierT])}
  \HaskellResult							{_M{--}(S, S, D, TypeT)}
\EndHaskellToken

\IndexTerm		{meaningOfTypeSpecifiers}			{D_TS type_specifiers}

\set{continued}		{yes}
\begin{code}
meaningOfTypeSpecifiers {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, [TypeSpecifierT]) -> _M{--}(S, S, D, TypeT)

meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[VoidSpecifier])						= return $ (_S, _T, _D, {-sem-}(VoidType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[CharSpecifier])						= return $ (_S, _T, _D, {-sem-}(PlainCharType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[SignedSpecifier {-hide-}, CharSpecifier])			= return $ (_S, _T, _D, {-sem-}(Signed CharType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[UnsignedSpecifier {-hide-}, CharSpecifier])		= return $ (_S, _T, _D, {-sem-}(Unsigned CharType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[ShortSpecifier])						= return $ (_S, _T, _D, {-sem-}(Signed ShortType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[ShortSpecifier {-hide-}, IntSpecifier])			= return $ (_S, _T, _D, {-sem-}(Signed ShortType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[SignedSpecifier {-hide-}, ShortSpecifier])		= return $ (_S, _T, _D, {-sem-}(Signed ShortType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[SignedSpecifier {-hide-}, ShortSpecifier {-hide-}, IntSpecifier])	---
															= return $ (_S, _T, _D, {-sem-}(Signed ShortType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[UnsignedSpecifier {-hide-}, ShortSpecifier])		= return $ (_S, _T, _D, {-sem-}(Unsigned ShortType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[UnsignedSpecifier {-hide-}, ShortSpecifier {-hide-}, IntSpecifier]) ---
															= return $ (_S, _T, _D, {-sem-}(Unsigned ShortType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[{- Nothing -}])						= return $ (_S, _T, _D, {-sem-}(PlainIntType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[IntSpecifier])						= return $ (_S, _T, _D, {-sem-}(PlainIntType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[SignedSpecifier])						= return $ (_S, _T, _D, {-sem-}(Signed IntType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[SignedSpecifier {-hide-}, IntSpecifier])			= return $ (_S, _T, _D, {-sem-}(Signed IntType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[UnsignedSpecifier])					= return $ (_S, _T, _D, {-sem-}(Unsigned IntType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[UnsignedSpecifier {-hide-}, IntSpecifier])		= return $ (_S, _T, _D, {-sem-}(Unsigned IntType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[LongSpecifier])						= return $ (_S, _T, _D, {-sem-}(Signed LongType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[LongSpecifier {-hide-}, IntSpecifier])			= return $ (_S, _T, _D, {-sem-}(Signed LongType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[SignedSpecifier {-hide-}, LongSpecifier])			= return $ (_S, _T, _D, {-sem-}(Signed LongType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[SignedSpecifier {-hide-}, LongSpecifier {-hide-}, IntSpecifier]) ---
															= return $ (_S, _T, _D, {-sem-}(Signed LongType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[UnsignedSpecifier {-hide-}, LongSpecifier])		= return $ (_S, _T, _D, {-sem-}(Unsigned LongType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[UnsignedSpecifier {-hide-}, LongSpecifier {-hide-}, IntSpecifier]) ---
															= return $ (_S, _T, _D, {-sem-}(Unsigned LongType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[FloatSpecifier])						= return $ (_S, _T, _D, {-sem-}(FloatType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[DoubleSpecifier])						= return $ (_S, _T, _D, {-sem-}(DoubleType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[LongSpecifier {-hide-}, DoubleSpecifier])			= return $ (_S, _T, _D, {-sem-}(LongDoubleType))
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[StructOrUnionSpecifier struct_or_union_specifier])	--
	= meaningOfStructOrUnionSpecifier	{--}(_S, _T, _D, _I {->-}, struct_or_union_specifier)
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[EnumSpecifier enum_specifier])				--
	= meaningOfEnumSpecifier		{--}(_S, _T, _D, _I {->-}, enum_specifier)
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}[TypedefNameSpecifier typedef_name])			--
	= meaningOfTypedefName			{--}(_S, _T, _D, _I {->-}, typedef_name)
meaningOfTypeSpecifiers {-sem-}(_S, _T, _D, _I {->-}, {-raw-}(other))							= fail ("Semantic error")
\end{code}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/StructAndUnionSpecifiers}
\set{title}		{Structure and Union Specifiers}
\begin{subsection}

\BeginHaskellToken	{isValidMember}				{Open}	{WF}
  \HaskellArgument							{MemberT}
  \HaskellResult							{Bool}
\EndHaskellToken

\set{capture}		{isValidMember/Signature}
\begin{code}
isValidMember {-[*]-} :: MemberT -> Bool
\end{code}

\set{capture}		{isValidMember/Definition}
\begin{code}
isValidMember {-sem-}(m) ---
	=	(	(nameOfMember{--}(m) /= Nothing && --
		 	 (isObjectType{--}(typeOfMember{--}(m)) || --
		 	  (isBitFieldType{--}(typeOfMember{--}(m)) && widthOfType{--}(typeOfMember{--}(m)) > 0))) ||
		 	(nameOfMember{--}(m) == Nothing && isBitFieldType{--}(typeOfMember{--}(m)) && widthOfType{--}(typeOfMember{--}(m)) == 0)) &&
		inRange (0) (<=) (toRational (offsetOfMember{--}(m))) (<=) (lubOfType{--}(ptrDiffType) - toRational (sizeOfType{--}(typeOfMember{--}(m))))
\end{code}

\BeginHaskellToken	{zeroMember}				{Open}	{\ZapfChancery{Z}}
  \HaskellArgument							{MemberT}
  \HaskellResult							{MemberT}
\EndHaskellToken

\set{capture}		{zeroMember}
\begin{code}
zeroMember {-[*]-} :: MemberT -> MemberT 

zeroMember {-sem-}(Member (t) (x) {-@-} (n)) ---
  | isBitFieldType{--}(t)	= {-sem-}(Member ({-sem-}(setToList (tqOfType{--}(t)) `qualifiedType` --
  							(BitFieldType ({-sem-}(baseOfType{--}(t))) {-:-} ({-sem-}(widthOfType{--}(t))) {-@-} (0)))) (x) {-@-} (0))
  | otherwise			= {-sem-}(Member (t) (x) {-@-} (0))
\end{code}

A «structure or union specifier» consists of the keyword “"Struct"” or “"Union"”, followed by an optional identifier and an optional list of «structure declarations» enclosed in braces. At least one of the "identifier" and "struct_declaration_list" components must be present, as described by the following set of Haskell data type definitions:

\IndexType		{StructOrUnionSpecifierT}			{struct_or_union_specifier (\tstar)}
\IndexCons		{StructOrUnion}					{{} struct_or_union identifier_opt struct_declaration_list (struct_or_union_specifier)}
\IndexCons		{StructOrUnionTag}				{ struct_or_union identifier (struct_or_union_specifier)}

\IndexType		{StructOrUnionT}				{struct_or_union (\tstar)}
\IndexCons		{Struct}					{struct (struct_or_union)}
\IndexCons		{Union}						{union (struct_or_union)}

\begin{syntax}
\InsertHaskell		{StructOrUnionSpecifierT, \par, StructOrUnionT}
\end{syntax}

Every such structure or union specifier denotes a C type derived from its syntax and the surrounding translation context using the following Haskell construction:

\BeginHaskellToken	{meaningOfStructOrUnionSpecifier}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{sus}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, StructOrUnionSpecifierT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT)}
\EndHaskellToken

\IndexTerm		{meaningOfStructOrUnionSpecifier}		{D_SUS struct_or_union_specifier}

\begin{code}
meaningOfStructOrUnionSpecifier {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, StructOrUnionSpecifierT) -> _M{--}(S, S, D, TypeT)
\end{code}

In particular, if the entity includes no identifier, then a new globally-unique type tag “"utag{--}(_D)"” is introduced into the program, with the entire construct denoting a structure or union type “"CompoundType (struct_or_union) ({-sem-}(utag{--}(_D))) ({-sem-}(m_set))"”, in which the member list "m_set" is derived from the specifier's list of structure declarations "sdl", after annotating every member "m_k @@ m_set" with an appropriate offset value as follows:

\begin{code}
meaningOfStructOrUnionSpecifier {-sem-}(_S, _T, _D, _I {->-}, StructOrUnion (struct_or_union) ({-hide-}Nothing) (sdl)) = do
	(_S', _T', _D', m_set) <- meaningOfStructDeclarationList{--}(_S, _T, _D `mapUnion` listToMap [ {-raw-}(utag{--}(_D) {-:-}, Nothing) ], _I {->-}, sdl)
	require $ and [ isValidMember{--}(m_k) | m_k <- layoutOfMembers{--}(struct_or_union {->-}, m_set) ]
	return $ (_S', _T', _D', {-sem-}(CompoundType (struct_or_union) ({-sem-}(utag{--}(_D))) ({-sem-}(layoutOfMembers{--}(struct_or_union {->-}, m_set)))))
\end{code}

In particular, the notation “"layoutOfMembers{--}(struct_or_union {->-}, m_set)"” represents an application of the following unspecified Haskell function "layoutOfMembers", which, intuitively, annotates every member "m_k @@ m_set" with an appropriate offset value within the newly-introduced structure \hbox{or union type:}

\ReindexTerm		{layoutOfMembers}

\begin{haskell}
layoutOfMembers {-[*]-} :: (StructOrUnionT {->-}, MembersT) -> MembersT
\end{haskell}

Although, in the process, "layoutOfMembers" can insert new anonymous members into the list "m_set", the relative order of all named members and their C types will be always preserved by the construction, with the possible exception of any bit field offset values appearing in the original configuration of the list:

\NewHaskellToken	{layoutOfMembersJ}			{Ord}	{\TKeyword{layout}}
\NewHaskellDescription	{layoutOfMembersJ}				{Layout of structure and union members}
\IndexTheorem		{layoutOfMembersJ}				{LAYOUT Layout of structure and union members}

\begin{haskell}
layoutOfMembersJ1 :: forall su, m_set =>
	{-sem-}[ zeroMember{--}(m_k) | m_k <- layoutOfMembers{--}(su {->-}, m_set), nameOfMember{--}(m_k) /= Nothing ] --
			= {-sem-}[ zeroMember{--}(m_k) | m_k <- m_set, nameOfMember{--}(m_k) /= Nothing ]
\end{haskell}

where "zeroMember{--}(m)" discards all offset information from a given member "m" as follows:

\IndexTerm		{zeroMember}					{Z member (member)}
\InsertHaskell		{zeroMember}

Further, for every member found in the resulting list "m_set" and a well-formed Etude object atom "alpha" whose format describes the entire structure or union type in question, the sum of the member's offset with an appropriately-converted value of "alpha" is also guaranteed to represent a meaningful object atom:

\begin{haskell}
layoutOfMembersJ2 :: forall su, nu, m_set, phi =>
	{-sem-}(phi == formatOfType{-sem-}(CompoundType (su) (nu) ({-sem-}(layoutOfMembers{--}(su {->-}, m_set))))) ->
	(forall n, m => WfJ{-sem-}(IMM (n) (phi)) -> {-sem-}(m @@ layoutOfMembers{--}(su {->-}, m_set)) ->
	{-quad-}WfJ{-sem-}(OP2 (OP1 (CNV {-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) (phi) {--} {-hv-}(IMM (n) (phi))) --
				(ADD) ({-sem-}(objectFormat{--}(formatOfType{--}(typeOfMember{--}(m))))) --
				(IMM ({-sem-}(offsetOfMember{--}(m))) (ZG`FMT`standardEncoding))))
\end{haskell}

Finally, in the entire member list of every well-formed structure type, no two members will be ever allocated to overlapping regions in the structure's memory image. Specifically, let "_A", "_B" and "_C" represent three sets, such that "_A" consists of all the distinct byte offsets allocated to some named member of "m_set", "_B" specifies all distinct member offsets allocated to named bit field members and "_C" includes every tuple "(n, i {-,-})" in which "n" represents the offset of some bit field member from "m_set" and "i" represents one of the bit indices associated with that bit field. Further, let "_S" and "_W" represent the total size of all named non-bit field members and the total width of all bit field members found in the structure, respectively. Then, in every member set "m_set" constructed by \hbox{"layoutOfMembers{-sem-}(Struct {->-}, m_set)",} the cardinality of "_A" must be equal to \hbox{"_S + cardinality (_B) * sizeOfFormat{--}(bitFieldFormat)"} and the cardinality of "_C" must be equal to "_W". Formally:

\begin{haskell}
layoutOfMembersJ3 :: forall m_set, _A, _B, _S =>
	{-sem-}(_A == listToSet [ offsetOfMember{--}(m_k) + i --
					| m_k <- layoutOfMembers{-sem-}(Struct {->-}, m_set), --
					  i <- [ 0 .. sizeOfType{--}(typeOfMember{--}(m_k)) - 1 ], --
					  nameOfMember{--}(m_k) /= Nothing ])
	{-sem-}(_B == listToSet [ offsetOfMember{--}(m_k) --
					| m_k <- layoutOfMembers{-sem-}(Struct {->-}, m_set), --
					  nameOfMember{--}(m_k) /= Nothing && isBitFieldType{--}(typeOfMember{--}(m_k)) ]) ->
	{-sem-}(_S == sum [ sizeOfType{--}(typeOfMember{--}(m_k)) | m_k <- m_set, nameOfMember{--}(m_k) /= Nothing && not (isBitFieldType{--}(typeOfMember{--}(m_k))) ]) ->
	{-sem-}(cardinality (_A) == _S + cardinality (_B) * sizeOfFormat{--}(bitFieldFormat))

layoutOfMembersJ4 :: forall m_set, _C, _W =>
	{-sem-}(_C == listToSet [ (offsetOfMember{--}(m_k), offsetOfType{--}(typeOfMember{--}(m_k)) + j)
	{-quad-}{-quad-}		| m_k <- layoutOfMembers{-sem-}(Struct {->-}, m_set), --
					  j <- [ 0 .. widthOfType{--}(typeOfMember{--}(m_k)) - 1 ], --
					  isBitFieldType{--}(typeOfMember{--}(m_k)) ]) ->
	{-sem-}(_W == sum [ widthOfType{--}(typeOfMember{--}(m_k)) | m_k <- m_set, isBitFieldType{--}(typeOfMember{--}(m_k)) ]) ->
	{-sem-}(cardinality (_C) == _W)
\end{haskell}

While the precise definition of "layoutOfMembers" may, in principle, vary between individual compiler implementations, Appendix \ref{C/Implementation} includes an almost-universally accepted design of this function, in which individual members are always allocated to successive byte offsets within the structure and adjacent bit field containers are merged together whenever possible, in order to reduce the overall memory footprint of the entire construction.

In a well-formed C program, all elements of the resulting member list are subject to a further semantic scrutiny by the following Haskell construction:

\IndexTerm		{isValidMember}					{WF member (bool)}
\InsertHaskell		{isValidMember/Signature}

In particular, every well-formed member must be associated with a non-negative offset value and a well-formed object or bit field type. Further, each bit field member must be either unnamed, or else it must specify a type with a non-zero width, since all bit field entities of the form “"BitFieldType (t {-!-}) {-:-} (0 {-!-}) {-@-} ({-!-} n)"” are reserved for a rather specialised purpose discussed later in this section. Finally, in all members, the sum of the member's offset with the size of its type must be no greater than the largest value representable under the implementation-defined C type “"ptrDiffType"”. Formally:

\InsertHaskell		{isValidMember/Definition}

Otherwise, the meaning of a structure specifier whose syntax includes both an identifier component "x" and a list of structure declarations "sdl" depends on the present binding of "x" in the current tag scope "_T". If no tag binding for this identifier is visible in "_T", or if the existing binding has been introduced in some outer scope of the program, then the specifier behaves similarly to the above anonymous version of the construct, except that, during processing of "sdl", the tag name "x" is also bound to a designator with the type linkage and an incomplete structure or union type tagged with the newly-introduced unique tag variable “"utag{--}(_D)"”. At the end of the entire construct, this type is then completed in the current scope with the list of members derived from the provided list of structure declarations and returned as part of the entity's denotation. If, on the other hand, an existing binding of "x" is already present in the innermost current scope "_T" and if that binding is associated with an incomplete structure or union type of the same kind as that specified by the entity's syntax, then the existing tag variable of "x" is completed with the list of structure or union members derived from "sdl". In all cases, the resulting member list is laid out and validated using the "layoutOfMembers" and "isValidMember" functions \hbox{described earlier:}

\begin{code}
meaningOfStructOrUnionSpecifier {-sem-}(_S, _T, _D, _I {->-}, StructOrUnion (struct_or_union) (Just x) (sdl))

	=	mdo	require $ (x /@ dom{--}(_T) || indexOfDesignator{--}(_T#(x)) /= _I)
  			(_S', _T', _D', m_set)  <- meaningOfStructDeclarationList ---
  				{--}(_S, _T // listToMap [ {-raw-}(x {-:-}, {-sem-}(Designator (TypeLinkage) --
  										(CompoundType (struct_or_union) ({-sem-}(utag{--}(_D))) {-hide-}[]) {-@-} _I)) ],
  				{-quad-}_D `mapUnion` listToMap [ {-raw-}(utag{--}(_D) {-:-}, Nothing) ], _I {->-}, sdl)
			(t) <- {-hide-}return $ {-sem-}(CompoundType (struct_or_union) ({-sem-}(utag{--}(_D))) --
								({-sem-}(completeMembers{--}(t {->-}, layoutOfMembers{--}(struct_or_union {->-}, m_set)))))
			require $ (isIncompleteType{--}(typeOfDesignator{--}(_T'#(x)))) && and [ isValidMember{--}(m_k) | m_k <- layoutOfMembers{--}(struct_or_union {->-}, m_set) ]
			return $ (completeScope{-sem-}(t {->-}, _S'), completeScope{-sem-}(t {->-}, _T'), _D', t)

	<|>	mdo	require $	(x @@ dom{--}(_T) && indexOfDesignator{--}(_T#(x)) == _I) &&
  					(isCompoundType{--}(typeOfDesignator{--}(_T#(x))) && isIncompleteType{--}(typeOfDesignator{--}(_T#(x)))) && --
  					(suOfType{--}(typeOfDesignator{--}(_T#(x))) == struct_or_union)
	  		(_S', _T', _D', m_set) <- meaningOfStructDeclarationList{--}(_S, _T, _D, _I {->-}, sdl)
			(t)  <- {-hide-}return $ {-sem-}(CompoundType (struct_or_union) ({-sem-}(tagOfType{--}(typeOfDesignator{--}(_T#(x))))) --
								({-sem-}(completeMembers{--}(t {->-}, layoutOfMembers{--}(struct_or_union {->-}, m_set)))))
			require $ (isIncompleteType{--}(typeOfDesignator{--}(_T'#(x)))) && and [ isValidMember{--}(m_k) | m_k <- layoutOfMembers{--}(struct_or_union {->-}, m_set) ]
			return $ (completeScope{-sem-}(t {->-}, _S'), completeScope{-sem-}(t {->-}, _T'), _D', t)
\end{code}

Observe that, since the list of structure declarations is processed directly in the scope of the surrounding specifier, a programmer may be tempted to complete it before the end of the entire list. Since the C Standard permits only a single complete definition of a given type tag within a particular scope, any such attempts are foiled in the above formalisation of structure specifiers by ensuring that the supplied tag name "x" remains bound to an incomplete type at the end of its list of structure declarations.

Finally, a structure or union specifier without a declaration list refers simply to the existing meaning of the designated tag name. If no such binding is visible in the current scope, then the construct introduces a new incomplete structure or union type, tagged with the unique variable “"utag{--}(_D)"”. Otherwise, it simply denotes the existing type of "x" in "_T", provided that this type represents a structure or union of the same kind as that requested in the entity's syntax. Formally:

\begin{code}	
meaningOfStructOrUnionSpecifier {-sem-}(_S, _T, _D, _I {->-}, StructOrUnionTag (struct_or_union) (x) {-,-})

	=	do	require $ (x /@ dom{--}(_T))
  			return $	(_S, --
  					 _T `mapUnion` listToMap [ {-raw-}(x {-:-}, {-sem-}(Designator (TypeLinkage) --
  											(CompoundType (struct_or_union) ({-sem-}(utag{--}(_D))) {-hide-}[]) {-@-} _I)) ],
  		  			{-quad-} _D `mapUnion` listToMap [ {-raw-}(utag{--}(_D) {-:-}, Nothing) ], --
  		  			 {-sem-}(CompoundType (struct_or_union) ({-sem-}(utag{--}(_D))) {-hide-}[]))

	<|>	do	require $ (x @@ dom{--}(_T) && isCompoundType{--}(typeOfDesignator{--}(_T#(x))) && suOfType{--}(typeOfDesignator{--}(_T#(x))) == struct_or_union)
	  		return $	(_S, _T, _D, typeOfDesignator{--}(_T#(x)))

\end{code}

In the concrete syntax of the language, a «structure declaration» is always represented by a non-empty sequence of type specifiers and qualifiers, that are followed by a list of one or more «structure declarators» and concluded by the delimiter “"StructDeclaration () () {-;-}"\!”. Formally, such constructs denote a list of structure or union members derived from its individual declarators in the order of their appearance within the program, ensuring that no two named members of a given structure or union are assigned the same identifier. \hbox{In Haskell:}

\IndexType		{StructDeclarationListT}			{struct_declaration_list (\tstar)}
\IndexCons		{SD0}						{ struct_declaration (struct_declaration_list)}
\IndexCons		{SDn}						{ struct_declaration_list struct_declaration (struct_declaration_list)}

\IndexType		{StructDeclarationT}				{struct_declaration (\tstar)}
\IndexCons		{StructDeclaration}				{; specifier_qualifier_list struct_declarator_list (struct_declaration)}

\begin{syntax}
\InsertHaskell		{StructDeclarationListT, \par, StructDeclarationT}
\end{syntax}

Similarly to the ordinary C declarations from Section \ref{C/Declarations}, any type qualifiers and specifiers associated with a particular element of the sequence are split into two homogenous sets "tq_set" and "ts_set", with the later used to derive a C type "t_b", using a mapping identical to that applied in Section \ref{C/Declarations/TypeSpecifiers} to ordinary C declaration forms. In combination with the qualifier set "tq_set", "t_b" serves as the base type in a derivation of the actual member entities from the following list of structure declarators:

\BeginHaskellToken	{meaningOfStructDeclarationList}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{sd}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, StructDeclarationListT)}
  \HaskellResult							{_M{--}(S, S, D, MembersT)}
\EndHaskellToken

\IndexTerm		{meaningOfStructDeclarationList}		{D_SD struct_declaration_list}

\begin{code}
meaningOfStructDeclarationList {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, StructDeclarationListT) -> _M{--}(S, S, D, MembersT)

meaningOfStructDeclarationList {-sem-}(_S, _T, _D, _I {->-}, SD0 (StructDeclaration (sql) (sdl) {-;-})) = do
	(tq_set, ts_set) <- meaningOfSpecifierQualifierListOpt{--}(Just sql)
	(_S1, _T1, _D1, t_b	) <- meaningOfTypeSpecifiers		{--}(_S,   _T,   _D,   _I {->-}, setToList ts_set)
	(_S2, _T2, _D2, m_set	) <- meaningOfStructDeclaratorList	{--}(_S1,  _T1,  _D1,  _I, setToList tq_set `qualifiedType` t_b {->-}, sdl)
	return $ (_S2, _T2, _D2, m_set)

meaningOfStructDeclarationList {-sem-}(_S, _T, _D, _I {->-}, sdl `SDn` sd) = do
	(_S1, _T1, _D1, m_set1	) <- meaningOfStructDeclarationList	{--}(_S,   _T,   _D,   _I {->-}, sdl)
	(_S2, _T2, _D2, m_set2	) <- meaningOfStructDeclarationList	{--}(_S1,  _T1,  _D1,  _I {->-}, SD0 sd)
	require $ (dom{--}(setOfMembers m_set1) `intersection` dom{--}(setOfMembers m_set2) == emptySet)
	return $ (_S2, _T2, _D2, m_set1 ++ m_set2)

\end{code}

The actual syntax and semantics of a structure declaration's specifier-qualifier list are identical to those of ordinary declaration specifiers described earlier in Section \ref{C/Declarations/Specifiers}, except that no storage class specifiers can ever appear in these constructs:

\IndexType		{SpecifierQualifierListT}			{specifier_qualifier_list (\tstar)}
\IndexCons		{Specifier}					{ type_specifier specifier_qualifier_list_opt (specifier_qualifier_list)}
\IndexCons		{Qualifier}					{ type_qualifier specifier_qualifier_list_opt (specifier_qualifier_list)}

\begin{syntax}
\InsertHaskell		{SpecifierQualifierListT}
\end{syntax}

In particular, such lists are always split into type specifier and qualifier sets by the following definition analogous to the earlier formalisation of "meaningOfDeclarationSpecifiersOpt":

\BeginHaskellToken	{meaningOfSpecifierQualifierListOpt}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{sql}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{Maybe SpecifierQualifierListT}
  \HaskellResult							{_M{--}(SetT TypeQualifierT, SetT TypeSpecifierT)}
\EndHaskellToken

\IndexTerm		{meaningOfSpecifierQualifierListOpt}		{D_SQL specifier_qualifier_list_opt}

\begin{code}
meaningOfSpecifierQualifierListOpt {-[*]-} :: (MonadFix _M, Monad _M) =>
	(Maybe SpecifierQualifierListT) -> _M{--}(SetT TypeQualifierT, SetT TypeSpecifierT)
	
meaningOfSpecifierQualifierListOpt {-sem-}(Just (Specifier (type_specifier) (sql_opt))) = do
	(tq_set, ts_set) <- meaningOfSpecifierQualifierListOpt{--}(sql_opt)
	require $ (type_specifier /@ ts_set)
	return $ (tq_set, ts_set `union` listToSet [type_specifier])
	
meaningOfSpecifierQualifierListOpt {-sem-}(Just (Qualifier (type_qualifier) (sql_opt))) = do
	(tq_set, ts_set) <- meaningOfSpecifierQualifierListOpt{--}(sql_opt)
	require $ (type_qualifier /@ tq_set)
	return $ (tq_set `union` listToSet [type_qualifier], ts_set)
	
meaningOfSpecifierQualifierListOpt {-sem-}({-hide-}Nothing) = return $ (emptySet, emptySet)
\end{code}

Every structure declarator, in turn, consists of an optional «declarator» with the syntax defined later in Sections \ref{C/Declarations/Declarators} and an optional constant expression, at least one of which must always be present. Formally:

\IndexType		{StructDeclaratorListT}				{struct_declarator_list (\tstar)}
\IndexCons		{SR0}						{ struct_declarator (struct_declarator_list)}
\IndexCons		{SRn}						{, struct_declarator_list struct_declarator (struct_declarator_list)}

\IndexType		{StructDeclaratorT}				{struct_declarator (\tstar)}
\IndexCons		{StructDeclarator}				{ declarator (struct_declarator)}
\IndexCons		{BitFieldDeclarator}				{: declarator_opt constant_expression (struct_declarator)}

\begin{syntax}
\InsertHaskell		{StructDeclaratorListT, \par, StructDeclaratorT}
\end{syntax}

In this work, the meaning of all such structure declarators is captured by the following Haskell definition:

\BeginHaskellToken	{meaningOfStructDeclaratorList}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{sr}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, TypeT {->-}, StructDeclaratorListT)}
  \HaskellResult							{_M{--}(S, S, D, MembersT)}
\EndHaskellToken

\IndexTerm		{meaningOfStructDeclaratorList}			{D_SR struct_declarator_list}

\begin{code}
meaningOfStructDeclaratorList {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, TypeT {->-}, StructDeclaratorListT) -> _M{--}(S, S, D, MembersT)
\end{code}

In particular, a structure declarator without the “\!"`BitFieldDeclarator`{--}e"” suffix denotes a simple member entity, whose name and type is derived from the declarator's syntax by an algorithm "meaningOfDeclarator" defined later in Section \ref{C/Declarations/Declarators}. In all cases, the result must represent an object type:

\begin{code}
meaningOfStructDeclaratorList {-sem-}(_S, _T, _D, _I, t_b {->-}, SR0 (StructDeclarator (r))) = do
	(_S', _T', _D', t, x) <- meaningOfDeclarator{--}(_S, _T, _D, _I, t_b {->-}, r)
	require $ (isObjectType{--}(t))
	return $ (_S', _T', _D', {-sem-}[Member (t) (Just x) {-@-} (undefined)])
\end{code}

On the other hand, a structure declarator of the form “"r{--}`BitFieldDeclarator`{--}e"” denotes a bit field member, whose name and base type "t" is derived from the declarator "r" and whose width is specified by the integral constant expression "e". In all well-formed C programs, this base type must always represent some qualified or unqualified version of the “"PlainIntType"”, \hbox{“"Signed IntType"”} or “"Unsigned IntType"” type, with any type qualifiers stripped from "t" and applied directly to the member's own C type. Further, "e" must always specify a positive integer no greater than the width of "t" in magnitude. Formally:

\begin{code}
meaningOfStructDeclaratorList {-sem-}(_S, _T, _D, _I, t_b {->-}, SR0 (Just r `BitFieldDeclarator` e)) = do
	(_S1, _T1, _D1, t,    x  ) <- meaningOfDeclarator{--}(_S, _T, _D, _I, t_b {->-}, r)
	(_S2, _T2, _D2, t_e,  n  ) <- meaningOfIntegralConstantExpression{--}(_S1,  _T1,  _D1,  _I {->-}, E1 (E2 e))
	require $ (unqualifiedType{--}(t) @@ listToSet [ {-sem-}(PlainIntType), {-sem-}(Signed IntType), {-sem-}(Unsigned IntType) ] && inRange (1) (<=) (n) (<=) (widthOfType{--}(t)))
	return $ (_S2, _T2, _D2, --
		  {-sem-}[Member ({-sem-}(setToList (tqOfType{--}(t)) `qualifiedType` {-sem-}(BitFieldType ({-sem-}(unqualifiedType{--}(t))) {-:-} (n) {-@-} (undefined)))) --
  					(Just x) {-@-} (undefined)])
\end{code}

In addition, a degenerate bit field declarator of the form “\!"`BitFieldDeclarator`{--}e"” denotes an anonymous bit field member with a C type constructed similarly to the above derivation, except that the present variant of such entities also admits bit fields of a zero width:

\begin{code}
meaningOfStructDeclaratorList {-sem-}(_S, _T, _D, _I, t {->-}, SR0 ({-hide-}Nothing `BitFieldDeclarator` e)) = do
	(_S', _T', _D', t, n) <- meaningOfIntegralConstantExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 e))
	require $ (unqualifiedType{--}(t) @@ listToSet [ {-sem-}(PlainIntType), {-sem-}(Signed IntType), {-sem-}(Unsigned IntType) ] && inRange (0) (<=) (n) (<=) (widthOfType{--}(t)))
	return $ (_S', _T', _D', --
		  {-sem-}[Member ({-sem-}(setToList (tqOfType{--}(t)) `qualifiedType` {-sem-}(BitFieldType ({-sem-}(unqualifiedType{--}(t))) {-:-} (n) {-@-} (undefined)))) --
  					({-hide-}Nothing) {-@-} (undefined)])
\end{code}

The reader should observe that, initially, the byte offset of every member and the bit offset of every bit field type appearing in a structure declaration list is left unspecified by the above definition. These parameters are only assigned proper integer values by the "layoutOfMembers" function described earlier in this section, once the complete list of all members for the entity has been extracted from the enclosing structure or union specifier.

Finally, a comma-separated list of two or more structure declarators denotes a list of members, derived in the order of their appearance within the program as follows:

\set{continued}		{yes}
\begin{code}
meaningOfStructDeclaratorList {-sem-}(_S, _T, _D, _I, t {->-}, srl `SRn` sr) = do
	(_S1, _T1, _D1, m_set1)	<- meaningOfStructDeclaratorList{--}(_S,   _T,   _D,   _I, t {->-}, srl)
	(_S2, _T2, _D2, m_set2)	<- meaningOfStructDeclaratorList{--}(_S1,  _T1,  _D1,  _I, t {->-}, SR0 sr)
	require $ (dom{--}(setOfMembers m_set1) `intersection` dom{--}(setOfMembers m_set2) == emptySet)
	return $ (_S2, _T2, _D2, m_set1 ++ m_set2)
\end{code}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/EnumerationSpecifiers}
\set{title}		{Enumeration Specifiers}
\begin{subsection}

In a concrete syntax of the language, an «enumeration specifier» is represented by the keyword “\CKeyword{enum}”, followed by an optional identifier and an optional list of «enumerators» enclosed in braces. At least one of the "identifier" and "enumerator_list" components must be present in every such construct, as described by the following BNF grammar:

\IndexType		{EnumSpecifierT}				{enum_specifier (\tstar)}
\IndexCons		{Enumeration}					{enum{} identifier_opt enumerator_list (enum_specifier)}
\IndexCons		{EnumerationTag}				{enum identifier (enum_specifier)}

\begin{syntax}
\InsertHaskell		{EnumSpecifierT}
\end{syntax}

Similarly to structure and union specifiers, all such enumeration specifiers denote a C type. Their denotations are derived by the following Haskell function:

\BeginHaskellToken	{meaningOfEnumSpecifier}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ens}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, EnumSpecifierT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT)}
\EndHaskellToken

\IndexTerm		{meaningOfEnumSpecifier}			{D_ENS enum_specifier}

\begin{code}
meaningOfEnumSpecifier {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, EnumSpecifierT) -> _M{--}(S, S, D, TypeT)
\end{code}

In particular, if the specifier's syntax includes a list of enumerators "enl", then that entity is used to derive a list of integer values "n_set", using an algorithm "meaningOfEnumeratorList" described later in this section. The entire construct denotes an enumeration type of the form “"EnumType (t) (nu)"”, in which "nu" is given a unique value “"utag{--}(_D')"” and "t" represents some implementation-defined integral type derived from "n_set". For named specifiers of the form “"Enumeration (Just x) (enl)"”, the identifier "x" is also bound in the current tag scope "_T" to a type designator with that enumeration type, provided that no previous declaration of the corresponding tag name "x" is visible in the innermost current scope of the entity:

\begin{code}
meaningOfEnumSpecifier {-sem-}(_S, _T, _D, _I {->-}, Enumeration ({-hide-}Nothing) (enl)) = do
	(_S', _T', _D', n_set) <- meaningOfEnumeratorList{--}(_S, _T, _D, _I {->-}, enl)
	return $ 	(_S', _T', _D' `mapUnion` listToMap [ {-raw-}(utag{--}(_D') {-:-}, Nothing) ], --
	  		 {-sem-}(EnumType ({-sem-}(typeOfEnumeratorList{--}(n_set))) ({-sem-}(utag{--}(_D')))))
  			 
meaningOfEnumSpecifier {-sem-}(_S, _T, _D, _I {->-}, Enumeration (Just x) (enl)) = do
	(_S', _T', _D', n_set) <- meaningOfEnumeratorList{--}(_S, _T, _D, _I {->-}, enl)
	require $ (x /@ dom{--}(_T) || indexOfDesignator{--}(_T#(x)) /= _I)
	return $	(_S', --
  			 _T' // listToMap [ {-raw-}(x {-:-}, {-sem-}(Designator (TypeLinkage) --
  			 					({-sem-}(EnumType ({-sem-}(typeOfEnumeratorList{--}(n_set))) ({-sem-}(utag{--}(_D'))))) {-@-} (_I))) ], --
  			 _D' `mapUnion` listToMap [ {-raw-}(utag{--}(_D') {-:-}, Nothing) ],
  			{-quad-}{-sem-}(EnumType ({-sem-}(typeOfEnumeratorList{--}(n_set))) ({-sem-}(utag{--}(_D')))))
\end{code}

The C Standard does not impose any particular restriction on the base type of the resulting enumeration. In the present work, this type is derived from the list of integer values assigned to the construct's individual enumerators by the following implementation-defined Haskell function:

\ReindexTerm		{typeOfEnumeratorList}

\begin{haskell}
typeOfEnumeratorList {-[*]-} :: [Integer] -> TypeT
\end{haskell}

A careful reading of the C Standard reveals that "typeOfEnumeratorList{--}(n_set {-,-})" must always represent a basic integral type convertible into “"Signed IntType"” under integral promotion:

\NewHaskellToken	{typeOfEnumeratorListJ}			{Ord}	{\TKeyword{enum}}
\NewHaskellDescription	{typeOfEnumeratorListJ}				{Corresponding enumerator types}
\IndexTheorem		{typeOfEnumeratorListJ}				{ENUM Corresponding enumerator types}

\begin{haskell}
typeOfEnumeratorListJ :: forall n_set => WfJ{--}(n_set) -> --
	{-sem-}(typeOfEnumeratorList{--}(n_set) @@ basicIntegralTypes && integralPromotedType{--}(typeOfEnumeratorList{--}(n_set {-,-})) == {-sem-}(Signed IntType))
\end{haskell}

On the other hand, enumeration specifiers without an explicit enumerator list must always refer to an existing tag name, that, in the current scope "_T", is bound to a designator with an enumeration type. Such entities denote the C type of the associated designator from "_T" without affecting the surrounding translation context:

\begin{code}
meaningOfEnumSpecifier {-sem-}(_S, _T, _D, _I {->-}, EnumerationTag (x)) = do
	require	$ (x @@ dom{--}(_T) && isEnumType{--}(typeOfDesignator{--}(_T#(x))))
	return $ (_S, _T, _D, typeOfDesignator{--}(_T#(x)))
\end{code}

When present, the list of enumerators found in an enumeration specifier must consist of one or more comma-separated «enumerator» entities, every one of which is formed from an identifier and an optional constant expression known as the enumerator's «initialiser», as represented by the following Haskell rendition of its concrete syntax:

\IndexType		{EnumeratorListT}				{enumerator_list (\tstar)}
\IndexCons		{N0}						{ enumerator (enumerator_list)}
\IndexCons		{Nn}						{, enumerator_list enumerator (enumerator_list)}

\IndexType		{EnumeratorT}					{enumerator (\tstar)}
\IndexCons		{DefaultEnumerator}				{ identifier (enumerator)}
\IndexCons		{InitEnumerator}				{= identifier constant_expression (enumerator)}

\begin{syntax}
\InsertHaskell		{EnumeratorListT, \par, EnumeratorT}
\end{syntax}

Every such enumerator introduces into the current scope "_S" a new binding of the specified variable name to a designator with an appropriate constant linkage and the plain “"PlainIntType"” type, provided that no other binding of that identifier exists in "_S", or that the previous binding has been introduced in some outer scope and that the integer value of the identifier's new linkage is representable under the “"PlainIntType"” type. Further, the denotation of an enumerator list includes a sequence of all integer values associated with the linkages of any new constants introduced into the program by its translation, so that the meaning of all C enumerators can be modelled by the following Haskell function:

\BeginHaskellToken	{meaningOfEnumeratorList}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{en}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, EnumeratorListT)}
  \HaskellResult							{_M{--}(S, S, D, [Integer])}
\EndHaskellToken

\IndexTerm		{meaningOfEnumeratorList}			{D_EN enumerator_list}

\begin{code}
meaningOfEnumeratorList {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, EnumeratorListT) -> _M{--}(S, S, D, [Integer])
\end{code}

In particular, for enumerators of the form “"x `InitEnumerator` e"”, "e" must represent an integral constant expression, whose numeric value is used to form a linkage of the resulting designator:

\begin{code}
meaningOfEnumeratorList {-sem-}(_S, _T, _D, _I {->-}, N0 (x `InitEnumerator` e)) = do
	(_S', _T', _D', t, n) <- meaningOfIntegralConstantExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 e))
	require $ (x /@ dom{--}(_S') || indexOfDesignator{--}(_S'#(x)) /= _I) && (inRange (glbOfType{-sem-}(PlainIntType)) (<=) (toRational n) (<=) (lubOfType{-sem-}(PlainIntType)))
	return $ (_S' // listToMap [ {-raw-}(x {-:-}, {-sem-}(Designator (ConstantLinkage n) (PlainIntType) {-@-} (_I))) ], _T, _D, [n])

meaningOfEnumeratorList {-sem-}(_S, _T, _D, _I {->-}, enl `Nn` {-raw-}(x `InitEnumerator` e)) = do
	(_S1, _T1, _D1, n_set) <- meaningOfEnumeratorList{--}(_S, _T, _D, _I {->-}, enl)
	(_S2, _T2, _D2, t, n) <- meaningOfIntegralConstantExpression{--}(_S1, _T1, _D1, _I {->-}, E1 (E2 e))
	require $ (x /@ dom{--}(_S2) || indexOfDesignator{--}(_S2#(x)) /= _I) && (inRange (glbOfType{-sem-}(PlainIntType)) (<=) (toRational n) (<=) (lubOfType{-sem-}(PlainIntType)))
	return $ (_S2 // listToMap [ {-raw-}(x {-:-}, {-sem-}(Designator (ConstantLinkage n) (PlainIntType) {-@-} (_I))) ], _T2, _D2, n_set ++ [n])
\end{code}

Otherwise, if the enumerator consists entirely of an identifier without an explicit expression initialiser, then its linkage is assigned a value one greater than that of the last enumerator preceding the current one in the specifier's syntax, or 0 if such an entity appears at the very beginning of the list. Formally:

\set{continued}		{yes}
\begin{code}
meaningOfEnumeratorList {-sem-}(_S, _T, _D, _I {->-}, N0 (DefaultEnumerator (x))) = do
	require $ (x /@ dom{--}(_S) || indexOfDesignator{--}(_S#(x)) /= _I)
  	return $ (_S // listToMap [ {-raw-}(x {-:-}, {-sem-}(Designator (ConstantLinkage 0) (PlainIntType) {-@-} (_I))) ], _T, _D, [0])

meaningOfEnumeratorList {-sem-}(_S, _T, _D, _I {->-}, enl `Nn` DefaultEnumerator (x)) = do
	(_S', _T', _D', n_set) <- meaningOfEnumeratorList{--}(_S, _T, _D, _I {->-}, enl)
	require $ (x /@ dom{--}(_S') || indexOfDesignator{--}(_S'#(x)) /= _I) && --
		  (inRange (glbOfType{-sem-}(PlainIntType)) (<=) (toRational (last{--}(n_set) + 1)) (<=) (lubOfType{-sem-}(PlainIntType)))
  	return $ (_S' // listToMap [ {-raw-}(x {-:-}, {-sem-}(Designator (ConstantLinkage {-sem-}(last{--}(n_set) + 1)) (PlainIntType) {-@-} (_I))) ], --
  		  _T', _D', n_set ++ [last{--}(n_set) + 1])
\end{code}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/Tags}
\set{title}		{Tag Declarations}
\begin{subsection}

As mentioned at the beginning of Section \ref{C/Declarations}, a C declaration whose syntax does not include any declarator entities is known as a «tag declaration». The list of specifiers in every such construct must consist entirely of a single structure, union or enumeration specifier entity. Formally, their meaning is derived by the following Haskell function:

\BeginHaskellToken	{meaningOfTagSpecifiers}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{tag}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, DeclarationSpecifiersT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT)}
\EndHaskellToken

\IndexTerm		{meaningOfTagSpecifiers}			{D_TAG declaration_specifiers}

\begin{code}
meaningOfTagSpecifiers {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, DeclarationSpecifiersT) -> _M{--}(S, S, D, TypeT)
\end{code}

In particular, if the declaration assumes the form of “"StructOrUnionTag (struct_or_union) (x)"”, in which "x" is either undeclared in the current scope "_T", or else introduced only in some outer scope of the program, then the construct always extend "_T" with a new binding of "x" to a unique incomplete structure or union type as follows:

\begin{code}
meaningOfTagSpecifiers {-sem-}(_S, _T, _D, _I {->-}, DeclarationSpecifier (StructOrUnionSpecifier (StructOrUnionTag (struct_or_union) (x))) ({-hide-}Nothing)) = do
	require $ (x /@ dom{--}(_T) || indexOfDesignator{--}(_T#(x)) /= _I)
	return $	(_S, --
  			 _T // listToMap [ {-raw-}(x {-:-}, {-sem-}(Designator (TypeLinkage) (CompoundType (struct_or_union) ({-sem-}(utag{--}(_D))) {-hide-}[]) {-@-} _I)) ], --
			 _D `mapUnion` listToMap [ {-raw-}(utag{--}(_D) {-:-}, Nothing) ],
			{-quad-}{-sem-}(CompoundType (struct_or_union) ({-sem-}(utag{--}(_D))) {-hide-}[]))
\end{code}

Otherwise, a tag declaration must consist entirely of a single named structure or union specifier whose syntax includes a list of structure declarations, or else an enumeration specifier with a non-empty enumerator list, behaving exactly like any ordinary occurrence of such an entity described earlier in Sections \ref{C/Declarations/StructAndUnionSpecifiers} and \ref{C/Declarations/EnumerationSpecifiers}:

\begin{code}
meaningOfTagSpecifiers {-sem-}(_S, _T, _D, _I {->-}, DeclarationSpecifier (StructOrUnionSpecifier (StructOrUnion (struct_or_union) (Just x) (sdl))) ({-hide-}Nothing)) ---
  =	meaningOfStructOrUnionSpecifier		{-sem-}(_S, _T, _D, _I {->-}, StructOrUnion (struct_or_union) (Just x) (sdl))
meaningOfTagSpecifiers {-sem-}(_S, _T, _D, _I {->-}, DeclarationSpecifier (EnumSpecifier (Enumeration (x_opt) (enl))) ({-hide-}Nothing)) ---
  =	meaningOfEnumSpecifier			{-sem-}(_S, _T, _D, _I {->-}, Enumeration (x_opt) (enl))
meaningOfTagSpecifiers {-sem-}(_S, _T, _D, _I {->-}, other) ---
  =	fail ("Semantic error")
\end{code}

Intuitively, tag declarations allow C programs to hide an existing declaration of a structure or union type, in order to replace them with a new complete or incomplete binding of the same tag name. The reader should observe that the special behaviour of these declaration forms is only applicable to structure specifiers, since, in C, enumeration types are always completed upon their initial appearance within the program.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/TypeQualifiers}
\set{title}		{Type Qualifiers}
\begin{subsection}

In a concrete syntax of the language, the qualification of all C types described earlier in Section \ref{C/Types} is represented by a pair of «type qualifier» keywords featured in the following Haskell definition:

\IndexType		{TypeQualifierT}				{type_qualifier (\tstar)}
\IndexCons		{ConstQualifier}				{const (type_qualifier)}
\IndexCons		{VolatileQualifier}				{volatile (type_qualifier)}

\IndexType		{TypeQualifierListT}				{type_qualifier_list (\tstar)}
\IndexCons		{Q0}						{ type_qualifier (type_qualifier_list)}
\IndexCons		{Qn}						{ type_qualifier_list type_qualifier (type_qualifier_list)}

\begin{syntax}
\InsertHaskell		{TypeQualifierT, \par, TypeQualifierListT}
\end{syntax}

The “"TypeQualifierListT"” syntax corresponds naturally to Haskell lists of type qualifiers, which, formally, can be derived from such entities using the following function:

\BeginHaskellToken	{meaningOfTypeQualifiers}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{tq}}}
  \HaskellArgument							{Maybe TypeQualifierListT}
  \HaskellResult							{[TypeQualifierT]}
\EndHaskellToken

\IndexTerm		{meaningOfTypeQualifiers}			{D_TQ type_qualifier_list_opt}

\begin{code}
meaningOfTypeQualifiers {-[*]-} :: Maybe TypeQualifierListT -> [TypeQualifierT]

meaningOfTypeQualifiers {-sem-}({-hide-}Nothing)					= {-empty-}[]
meaningOfTypeQualifiers {-sem-}(Just (Q0 type_qualifier))				= [type_qualifier]
meaningOfTypeQualifiers {-sem-}(Just (type_qualifier_list `Qn` type_qualifier))		= meaningOfTypeQualifiers{--}(Just type_qualifier_list) ++ [type_qualifier]
\end{code}

The resulting list can be applied to an almost-arbitrary C type using one of the algorithms described earlier in Section \ref{C/Types/Qualifiers}. However, as discussed in Section \ref{C/Types/Properties}, the reader should keep in mind that a single C type may be only qualified with at most one of the four syntactic forms “"ConstQualifier"”, “"VolatileQualifier"”, \hbox{“"ConstQualifier VolatileQualifier"”} or \hbox{“"VolatileQualifier ConstQualifier"”.} In particular, redundant qualifications such as \hbox{“"ConstQualifier ConstQualifier"”} are explicitly forbidden by the C Standard. However, an actual enforcement of these rules is encapsulated within the type classification predicates from Section \ref{C/Types} and generally disregarded during the actual semantic analysis of C declarations.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/Declarators}
\set{title}		{Declarators}
\begin{subsection}

\BeginHaskellToken	{meaningOfPointer}			{Open}	{\ZapfChancery{D}\sub{\SmallCaps{p}}}
  \HaskellArgument							{(TypeT {->-}, PointerT)}
  \HaskellResult							{TypeT}
\EndHaskellToken

\set{capture}		{meaningOfPointer}
\begin{code}
meaningOfPointer {-[*]-} :: (TypeT {->-}, PointerT) -> TypeT

meaningOfPointer {-sem-}(t {->-}, Pointers (tql_opt) (pointer)) ---
  = meaningOfPointer{--}(meaningOfTypeQualifiers{--}(tql_opt) `qualifiedType` {-sem-}(PointerType (t) {-*-}) {->-}, pointer)
meaningOfPointer {-sem-}(t {->-}, Pointer (tql_opt)) ---
  = meaningOfTypeQualifiers{--}(tql_opt) `qualifiedType` {-sem-}(PointerType (t) {-*-})
\end{code}

\BeginHaskellToken	{meaningOfArrayLengthOpt}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ale}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, Maybe ConstantExpressionT)}
  \HaskellResult							{_M{--}(S, S, D, Maybe Integer)}
\EndHaskellToken

\set{capture}		{meaningOfArrayLengthOpt}
\begin{code}
meaningOfArrayLengthOpt {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, Maybe ConstantExpressionT) -> _M{--}(S, S, D, Maybe Integer)

meaningOfArrayLengthOpt {-sem-}(_S, _T, _D, _I {->-}, Just e) = do
	(_S', _T', _D', t, n) <- meaningOfIntegralConstantExpression{--}(_S, _T, _D, _I {->-}, E1 (E2 e))
	return $ (_S', _T', _D', Just n)

meaningOfArrayLengthOpt {-sem-}(_S, _T, _D, _I {->-}, Nothing) = return $ (_S, _T, _D, Nothing)
\end{code}

In C, «declarators» are used to introduce new object identifiers into the current variable scope "_S". Formally, their syntax is depicted by the following set of three Haskell data type definitions:

\IndexType		{DeclaratorT}					{declarator (\tstar)}
\IndexCons		{Declarator}					{ pointer_opt direct_declarator (declarator)}

\IndexType		{DirectDeclaratorT}				{direct_declarator (\tstar)}
\IndexCons		{IdentifierDeclarator}				{ identifier (direct_declarator)}
\IndexCons		{NestedDeclarator}				{() declarator (direct_declarator)}
\IndexCons		{ArrayDeclarator}				{[] direct_declarator constant_expression_opt (direct_declarator)}
\IndexCons		{PrototypeDeclarator}				{() direct_declarator parameter_type_list (direct_declarator)}
\IndexCons		{FunctionDeclarator}				{() direct_declarator identifier_list_opt (direct_declarator)}

\IndexType		{PointerT}					{pointer (\tstar)}
\IndexCons		{Pointer}					{* type_qualifier_list_opt (pointer)}
\IndexCons		{Pointers}					{* type_qualifier_list_opt pointer (pointer)}

\begin{syntax}
\InsertHaskell		{DeclaratorT, \par, DirectDeclaratorT, \par, PointerT}
\end{syntax}

In all cases, every occurrence of a given declarator "r" in a C program must be associated with a list of zero or more type specifiers by the surrounding declaration. As already mentioned in Section \ref{C/Declarations}, these specifiers are used to construct the «base type» for the declarator, which the entity refines systematically into its eventual denotation. Specifically, the denotation of every C declarator "r" consists of a C type and an identifier found at the heart of its concrete syntax. Formally, these denotations are obtained using the following translation function:

\BeginHaskellToken	{meaningOfDeclarator}			{Open}	{\ZapfChancery{D}\sub{\SmallCaps{r}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, TypeT {->-}, DeclaratorT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT, IdentifierT)}
\EndHaskellToken

\IndexTerm		{meaningOfDeclarator}				{D_R declarator}

\begin{code}
meaningOfDeclarator {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I, TypeT {->-}, DeclaratorT) -> _M{--}(S, S, D, TypeT, IdentifierT)
\end{code}

In particular, a simple declarator whose syntax is formed entirely from some identifier "x" always denotes that identifier, assigning to it the derivation's base type "t_b" itself. Further, a parenthesised declarator of the form “\!"NestedDeclarator (r)"\!” has the same denotation as its direct declarator component "r", while a «pointer declarator» of the form “"Declarator (pointer) (r)"” denotes the same identifier and type as that depicted by "r", in the context of a new base type derived from its "pointer" entity as follows:

\begin{code}
meaningOfDeclarator {-sem-}(_S, _T, _D, _I, t_b {->-}, Declarator ({-hide-}Nothing) (IdentifierDeclarator (x))) ---
  =	return $ (_S, _T, _D, t_b, x)
meaningOfDeclarator {-sem-}(_S, _T, _D, _I, t_b {->-}, Declarator ({-hide-}Nothing) (NestedDeclarator (r))) ---
  =	meaningOfDeclarator{--}(_S, _T, _D, _I, t_b {->-}, r)
meaningOfDeclarator {-sem-}(_S, _T, _D, _I, t_b {->-}, Declarator (Just pointer) (r)) ---
  =	meaningOfDeclarator{--}(_S, _T, _D, _I, meaningOfPointer{--}(t_b {->-}, pointer) {->-}, Declarator ({-hide-}Nothing) (r))
\end{code}

Specifically, the notation “"meaningOfPointer{--}(t_b {->-}, Pointer (tql_opt))"” represents a "tql"-qualified pointer to the base type "t_b", while “"meaningOfPointer{--}(t_b {->-}, Pointers (tql_opt) (p))"” further refines that pointer in accordance with the syntax of "p", as captured by the following recursive algorithm:

\IndexTerm		{meaningOfPointer}				{D_P pointer}
\InsertHaskell		{meaningOfPointer}

Intuitively, these rules imply that a C declaration of the form “"Declaration (PlainIntType) (Declarator (Pointer) {--} x) {-;-}"\!” denotes the plain “"PlainIntType"” directly, while “"Declaration (ds) (Declarator (Pointer {--} tql) (x)) {-;-}"\!” denotes a "tql"-qualified pointer to the C type derived from the list of declaration specifiers "ds". On the other hand, an «array declarator» of the form “"r `ArrayDeclarator` (e_opt)"\!” denotes an array of its element type "t_b", with a length described by the optional integral constant expression "e_opt":

\begin{code}
meaningOfDeclarator {-sem-}(_S, _T, _D, _I, t_b {->-}, Declarator ({-hide-}Nothing) (r `ArrayDeclarator` (e_opt))) = mdo
	(_S_r,  _T_r,  _D_r,  t, x   ) <- meaningOfDeclarator{--}(_S, _T, _D, _I, {-sem-}({-,-}t_b `ArrayType` (n_opt)) {->-}, Declarator ({-hide-}Nothing) (r))
	(_S_e,  _T_e,  _D_e,  n_opt  ) <- meaningOfArrayLengthOpt{--}(_S_r, _T_r, _D_r, _I {->-}, e_opt)
	return $ (_S_e, _T_e, _D_e, t, x)
\end{code}

A a result, a declaration of the form “"Declaration (ds) (Declarator (Pointer {--} tql) (x `ArrayDeclarator` ({-!-})) {-!-}) {-;-}"\!” represents an array of "tql"-qualified pointers to the type derived from "ds", while “"Declaration (ds) (Declarator () (NestedDeclarator (Declarator (Pointer {--} tql) (x)) `ArrayDeclarator` ({-!-})) {-!-}) {-;-}"\!” represents a "tql"-qualified pointer to such an array. If no expression is included in the declarator's syntax, then the construct describes an incomplete array type of an unknown length. Otherwise, the type has a length equal to the expression's integer value, as captured by the following Haskell derivation:

\IndexTerm		{meaningOfArrayLengthOpt}			{D_ALE constant_expression_opt}
\InsertHaskell		{meaningOfArrayLengthOpt}

Last but not least, all declarators of the form “"r `PrototypeDeclarator` (ptl {-,-})"\!” and “"r `FunctionDeclarator` ()"\!” describe function types that return the declarator's base type "t_b". If the entity includes a parameter type list, then this list is used to construct the function's prototype. Otherwise, the result is a function without a prototype and the identifier list featured in the declarator's syntax must be empty, since function declarators with non-empty identifier lists are valid only in the context of a «function definition» considered later in Section \ref{C/Definitions/Functions}:

\begin{code}
meaningOfDeclarator {-sem-}(_S, _T, _D, _I, t_b {->-}, Declarator ({-hide-}Nothing) (r `PrototypeDeclarator` (ptl))) = mdo
	(_S_r, _T_r, _D_r, t, x) <- meaningOfDeclarator{--}(_S, _T, _D, _I, {-sem-}({-,-}t_b `FunctionType` (p_opt)) {->-}, Declarator ({-hide-}Nothing) (r))
	(_D_p, p_opt) <- meaningOfParameterTypeListOpt{--}(_S_r, _T_r, _D_r, _I {->-}, Just ptl)
	return $ (_S_r, _T_r, _D_p, t, x)

meaningOfDeclarator {-sem-}(_S, _T, _D, _I, t_b {->-}, Declarator ({-hide-}Nothing) (r `FunctionDeclarator` (xl_opt))) = do
	(_S_r, _T_r, _D_r, t, x) <- meaningOfDeclarator{--}(_S, _T, _D, _I, {-sem-}({-,-}t_b `FunctionType` ({-hide-}Nothing)) {->-}, Declarator ({-hide-}Nothing) (r))
	require $ (listOfIdentifiersOpt{--}(xl_opt) == {-empty-}[])
	return $ (_S_r, _T_r, _D_r, t, x)
\end{code}

In the concrete syntax of the language, actual function prototypes are described by entities known as «parameter type lists». Intuitively, a parameter type list consists of a comma-separated sequence of «parameter declarations», every one of which is formed from a list of declaration specifiers followed by a single declarator or an «abstract declarator» entity described later in Section \ref{C/Declarations/TypeNames}. The entire construct can be also concluded with an optional “\!"{-...-}{--}"\!” symbol. Alternatively, when no prototype information is to be provided for a given function, the names of its arguments may be described simply by a comma-separated list of identifiers, although, as already mentioned, non-empty identifier lists are only ever permitted in the syntax of «function definitions» scrutinised separately in Section \ref{C/Definitions/Functions}. Formally, both kinds of function annotations are depicted by the following set of Haskell type definitions:

\IndexType		{ParameterTypeListT}				{parameter_type_list (\tstar)}
\IndexCons		{FixedParameters}				{ parameter_list (parameter_type_list)}
\IndexCons		{VariableParameters}				{,... parameter_list (parameter_type_list)}

\IndexType		{ParameterListT}				{parameter_list (\tstar)}
\IndexCons		{P0}						{ parameter_declaration (parameter_list)}
\IndexCons		{Pn}						{, parameter_list parameter_declaration (parameter_list)}

\IndexType		{ParameterDeclarationT}				{parameter_declaration (\tstar)}
\IndexCons		{NamedParameter}				{ declaration_specifiers declarator (parameter_declaration)}
\IndexCons		{AbstractParameter}				{ declaration_specifiers abstract_declarator_opt (parameter_declaration)}

\IndexType		{IdentifierListT}				{identifier_list (\tstar)}
\IndexCons		{X0}						{ identifier (identifier_list)}
\IndexCons		{Xn}						{, identifier_list identifier (identifier_list)}

\begin{syntax}
\InsertHaskell		{ParameterTypeListT, \par, ParameterListT, \par, ParameterDeclarationT, \par, IdentifierListT}
\end{syntax}

where, for convenience, every "identifier_list" entity is generally treated as a Haskell list of identifiers, derived from the entity's syntax by the following recursive algorithm:

\BeginHaskellToken	{listOfIdentifiersOpt}			{Open}	{list}
  \HaskellArgument							{Maybe IdentifierListT}
  \HaskellResult							{[IdentifierT]}
\EndHaskellToken

\IndexTerm		{listOfIdentifiersOpt}				{list identifier_list_opt (identifiers)}

\begin{code}
listOfIdentifiersOpt {-[*]-} :: Maybe IdentifierListT -> [IdentifierT]

listOfIdentifiersOpt {-sem-}(Just (xl `Xn` x))		= listOfIdentifiersOpt{--}(Just xl) ++ [x]
listOfIdentifiersOpt {-sem-}(Just (X0 x))		= [x]
listOfIdentifiersOpt {-sem-}({-hide-}Nothing)		= {-empty-}[]
\end{code}

When present in a function declarator, a parameter type list always introduces a new variable and tag scope into the program. Intuitively, this scope spans all of its parameter declarations and is promptly discarded at the end of the list. Accordingly, the denotations of such constructs consist entirely of the intended prototype "p" and an updated set of item definitions "_D'", excluding the usual scope components "_S'" and "_T'". In particular, every parameter type list "pl" and its suffixed form “"VariableParameters (pl) {-,-} {--} {-...-}"\!” denotes a prototype whose type list is derived from the list of parameters "pl" and whose abstract syntax includes the “\!"{-...-}{--}"\!” suffix if and only if such suffix is also present in the actual source code of the program. In Haskell, this derivation is represented by the following simple translation:

\BeginHaskellToken	{meaningOfParameterTypeListOpt}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ptl}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, Maybe ParameterTypeListT)}
  \HaskellResult							{_M{--}(D, Maybe PrototypeT)}
\EndHaskellToken

\IndexTerm		{meaningOfParameterTypeListOpt}			{D_PTL parameter_type_list_opt}

\begin{code}
meaningOfParameterTypeListOpt {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, Maybe ParameterTypeListT) -> _M{--}(D, Maybe PrototypeT)

meaningOfParameterTypeListOpt {-sem-}(_S, _T, _D, _I {->-}, Just (FixedParameters (pl))) = do
	(_S', _T', _D', t_set) <- meaningOfParameterList{--}(_S, _T, _D, _I + 1 {->-}, pl)
	return $ (_D', Just {-sem-}(Types t_set))

meaningOfParameterTypeListOpt {-sem-}(_S, _T, _D, _I {->-}, Just (VariableParameters (pl) {-,-} {--} {-...-})) = do
	(_S', _T', _D', t_set) <- meaningOfParameterList{--}(_S, _T, _D, _I + 1 {->-}, pl)
	return $ (_D', Just {-sem-}(Ellipsis (t_set) {-...-}))

meaningOfParameterTypeListOpt {-sem-}(_S, _T, _D, _I {->-}, Nothing) = return $	(_D, {-sem-}({-hide-}Nothing))
\end{code}

Every parameter declaration contributes the C type of a single parameter to the function prototype under construction. Further, the identifier of every non-abstract parameter declarator is bound in the current scope "_S" to an appropriately-typed designator for a variable object, whose linkage is obtained by the algorithm described earlier in Section \ref{C/Declarations/StorageClassSpecifiers}.  In all cases, the parameter must have a well-formed object, function or incomplete type, must not include any storage class specifies other than “"RegisterSpecifier"” and its name must be either unbound in the current scope or else bound only in some outer scope of the program. Formally:

\BeginHaskellToken	{meaningOfParameterList}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{pl}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ParameterListT)}
  \HaskellResult							{_M{--}(S, S, D, TypesT)}
\EndHaskellToken

\IndexTerm		{meaningOfParameterList}			{D_PL parameter_list}

\set{continued}		{yes}
\begin{code}
meaningOfParameterList {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, ParameterListT) -> _M{--}(S, S, D, TypesT)

meaningOfParameterList {-sem-}(_S, _T, _D, _I {->-}, P0 (NamedParameter (ds) (r))) = do
	(sc_set, tq_set, ts_set) <- meaningOfDeclarationSpecifiersOpt{--}(Just ds)
	(_S1, _T1, _D1, t_b) <-	meaningOfTypeSpecifiers{--}(_S, _T, _D, _I {->-}, setToList ts_set)
	(_S2, _T2, _D2, t, x) <- meaningOfDeclarator{--}(_S1, _T1, _D1, _I, setToList tq_set `qualifiedType` t_b {->-}, r)
	(ell) <- meaningOfStorageClassSpecifiers{--}(_S2, _T2, _D2, _I, pointerPromotedType{--}(t), x {->-}, setToList sc_set)
	require $	(sc_set `subseteq` listToSet [RegisterSpecifier] && (isObjectType{--}(t) || isFunctionType{--}(t) || isIncompleteType{--}(t))) &&
			(x /@ dom{--}(_S2) || indexOfDesignator{--}(_S2#(x)) /= _I)
	return $ (_S2 // listToMap [ {-raw-}(x {-:-}, {-sem-}(Designator (ell) (t) {-@-} (_I))) ], --
		  _T2, _D2 // listToMap [ {-raw-}(variableOfLinkage{--}(ell) {-:-}, Nothing) ], [t])

meaningOfParameterList {-sem-}(_S, _T, _D, _I {->-}, P0 (AbstractParameter (ds) (ar_opt))) = do
	(sc_set, tq_set, ts_set) <- meaningOfDeclarationSpecifiersOpt{--}(Just ds)
	(_S1, _T1, _D1, t_b	) <- meaningOfTypeSpecifiers{--}(_S, _T, _D, _I {->-}, setToList ts_set)
  	(_S2, _T2, _D2, t	) <- meaningOfAbstractDeclaratorOpt{--}(_S1, _T1, _D1, _I, setToList tq_set `qualifiedType` t_b {->-}, ar_opt)
	require $	(sc_set `subseteq` listToSet [RegisterSpecifier] && (isObjectType{--}(t) || isFunctionType{--}(t) || isIncompleteType{--}(t)))
	return $ (_S2, _T2, _D2, [t])

meaningOfParameterList {-sem-}(_S, _T, _D, _I {->-}, pl `Pn` pd) = do
	(_S1, _T1, _D1, t_set1) <- meaningOfParameterList{--}(_S,   _T,   _D,   _I {->-}, pl)
	(_S2, _T2, _D2, t_set2) <- meaningOfParameterList{--}(_S1,  _T1,  _D1,  _I {->-}, P0 pd)
	return $ (_S2, _T2, _D2, t_set1 ++ t_set2)
\end{code}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/TypeNames}
\set{title}		{Type Names}
\begin{subsection}

\BeginHaskellToken	{toAbstractDeclaratorOpt}		{Void}	{}
  \HaskellArgument							{Maybe DirectAbstractDeclaratorT}
  \HaskellResult							{Maybe AbstractDeclaratorT}
\EndHaskellToken

\set{capture}		{hidden}
\begin{code}
toAbstractDeclaratorOpt {-[*]-} :: Maybe DirectAbstractDeclaratorT -> Maybe AbstractDeclaratorT

toAbstractDeclaratorOpt {-sem-}(Just ar)		= {-sem-}(Just (AbstractDeclarator ({-hide-}Nothing) (ar)))
toAbstractDeclaratorOpt {-sem-}({-hide-}Nothing)	= {-sem-}({-hide-}Nothing)
\end{code}

A «type name» has a structure similar to a declaration in which the declarator's identifier has been omitted from the concrete syntax, except that only one such declarator, said to be «abstract», may be present in a given type name entity and that no storage class specifiers may appear within its declaration specifier list. Further, in a well-formed C program, the abstract declarator is often omitted entirely from the syntax of such constructs. Formally, these structures are depicted in Haskell as follows:

\IndexType		{TypeNameT}					{type_name (\tstar)}
\IndexCons		{TypeName}					{ specifier_qualifier_list abstract_declarator_opt (type_name)}

\IndexType		{AbstractDeclaratorT}				{abstract_declarator (\tstar)}
\IndexCons		{AbstractPointer}				{ pointer (abstract_declarator)}
\IndexCons		{AbstractDeclarator}				{ pointer_opt direct_abstract_declarator (abstract_declarator)}

\IndexType		{DirectAbstractDeclaratorT}			{direct_abstract_declarator (\tstar)}
\IndexCons		{NestedAbstractDeclarator}			{() abstract_declarator (direct_abstract_declarator)}
\IndexCons		{AbstractArrayDeclarator}			{[] direct_abstract_declarator_opt constant_expression_opt (direct_abstract_declarator)}
\IndexCons		{AbstractFunctionDeclarator}			{() direct_abstract_declarator_opt parameter_type_list_opt (direct_abstract_declarator)}

\begin{syntax}
\InsertHaskell		{TypeNameT, \par, AbstractDeclaratorT, \par, DirectAbstractDeclaratorT}
\end{syntax}

Conceptually, the meaning of all type names is derived by an algorithm similar to that described in Section \ref{C/Declarations/Declarators} for ordinary C declarators, except that their denotations consist entirely of a C type, without an associated identifier entity:

\BeginHaskellToken	{meaningOfTypeName}			{Open}	{\ZapfChancery{D}\sub{\SmallCaps{tn}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, TypeNameT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT)}
\EndHaskellToken

\IndexTerm		{meaningOfTypeName}				{D_TN type_name}

\begin{code}
meaningOfTypeName {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, TypeNameT) -> _M{--}(S, S, D, TypeT)

meaningOfTypeName {-sem-}(_S, _T, _D, _I {->-}, TypeName (sql) (ar_opt)) = do
	(tq_set, ts_set) <- meaningOfSpecifierQualifierListOpt{--}(Just sql)
	(_S1, _T1, _D1, t_b  ) <- meaningOfTypeSpecifiers		{--}(_S,   _T,   _D,   _I {->-}, setToList ts_set)
	(_S2, _T2, _D2, t    ) <- meaningOfAbstractDeclaratorOpt	{--}(_S1,  _T1,  _D1,  _I, setToList tq_set `qualifiedType` t_b {->-}, ar_opt)
	require $ (isObjectType{--}(t) || isFunctionType{--}(t) || isIncompleteType{--}(t))
	return $ (_S2, _T2, _D2, t)
\end{code}

In particular, a type name without an abstract declarator denotes directly the base type of its derivation, while an abstract declarator consisting entirely of a "pointer" entity describes an appropriate pointer to that base type. If such a "pointer" is also followed by some other abstract declarator "ar", then the entire construct has the denotation of "ar", obtained in the context of an adjusted base type derived from that "pointer". Finally, a parenthesised abstract declarator of the form “\!"NestedAbstractDeclarator (ar)"\!” has the same denotation \hbox{as "ar" itself:}

\BeginHaskellToken	{meaningOfAbstractDeclaratorOpt}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ar}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, TypeT {->-}, Maybe AbstractDeclaratorT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT)}
\EndHaskellToken

\IndexTerm		{meaningOfAbstractDeclaratorOpt}		{D_AR abstract_declarator_opt}

\begin{code}
meaningOfAbstractDeclaratorOpt {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I, TypeT {->-}, Maybe AbstractDeclaratorT) -> _M{--}(S, S, D, TypeT)

meaningOfAbstractDeclaratorOpt {-sem-}(_S, _T, _D, _I, t_b {->-}, Nothing) ---
  = return $ (_S, _T, _D, t_b)
meaningOfAbstractDeclaratorOpt {-sem-}(_S, _T, _D, _I, t_b {->-}, Just (AbstractPointer (pointer))) ---
  = return $ (_S, _T, _D, meaningOfPointer{--}(t_b {->-}, pointer))
meaningOfAbstractDeclaratorOpt {-sem-}(_S, _T, _D, _I, t_b {->-}, Just (AbstractDeclarator (Just pointer) (ar))) ---
  = meaningOfAbstractDeclaratorOpt{--}(_S, _T, _D, _I, meaningOfPointer{--}(t_b {->-}, pointer) {->-}, Just (AbstractDeclarator ({-hide-}Nothing) (ar)))
meaningOfAbstractDeclaratorOpt {-sem-}(_S, _T, _D, _I, t_b {->-}, Just (AbstractDeclarator ({-hide-}Nothing) (NestedAbstractDeclarator (ar)))) ---
  = meaningOfAbstractDeclaratorOpt{--}(_S, _T, _D, _I, t_b {->-}, Just ar)
\end{code}

Further, array and function types can be described by abstract declarators of the form “"ar_opt `AbstractArrayDeclarator` (e_opt)"\!” and “"ar_opt `AbstractFunctionDeclarator` (ptl_opt)"\!”, observing that no identifier list is ever admitted into the later syntax. A derivation of their meanings proceeds in a manner analogous to that of earlier non-abstract declarators with similar structures:

\set{continued}		{yes}
\begin{code}
meaningOfAbstractDeclaratorOpt {-sem-}(_S, _T, _D, _I, t_b {->-}, Just (AbstractDeclarator ({-hide-}Nothing) (ar_opt `AbstractArrayDeclarator` (e_opt)))) = mdo
	(_S1, _T1, _D1, t) <- meaningOfAbstractDeclaratorOpt{--}(_S, _T, _D, _I, {-sem-}(ArrayType (t) (n_opt)) {->-}, toAbstractDeclaratorOpt ar_opt)
	(_S2, _T2, _D2, n_opt) <- meaningOfArrayLengthOpt{--}(_S1, _T1, _D1, _I {->-}, e_opt)
	return $ (_S2, _T2, _D2, t)

meaningOfAbstractDeclaratorOpt {-sem-}(_S, _T, _D, _I, t_b {->-}, Just (AbstractDeclarator ({-hide-}Nothing) (ar_opt `AbstractFunctionDeclarator` (ptl_opt)))) = mdo
	(_S1, _T1, _D1, t) <- meaningOfAbstractDeclaratorOpt{--}(_S, _T, _D, _I, {-sem-}(FunctionType (t_b) (p_opt)) {->-}, toAbstractDeclaratorOpt ar_opt)  
	(_D2, p_opt) <- meaningOfParameterTypeListOpt{--}(_S1, _T1, _D1, _I {->-}, ptl_opt)
	return $ (_S1, _T1, _D2, t)
\end{code}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/TypeDefinitions}
\set{title}		{Type Definitions}
\begin{subsection}

As already mentioned in Section \ref{C/Scopes}, an identifier with a type linkage is known as a «typedef name». In the concrete C grammar, such entities are depicted by the following trivial syntax:

\IndexType		{TypedefNameT}					{typedef_name (\tstar)}
\IndexCons		{TypedefNameT'}					{ identifier (typedef_name)}

\begin{syntax}
\InsertHaskell		{TypedefNameT}
\end{syntax}

A given typedef name "x" is considered to be well-formed only in the context of a variable scope "_S" which binds that identifier to a designator with the “"TypeLinkage"” linkage form. Such constructs always denote the type of that designator:

\BeginHaskellToken	{meaningOfTypedefName}			{Open}	{\ZapfChancery{D}\sub{\SmallCaps{tdn}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, TypedefNameT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT)}
\EndHaskellToken

\IndexTerm		{meaningOfTypedefName}				{D_TDN typedef_name}

\begin{code}
meaningOfTypedefName {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I {->-}, TypedefNameT) -> _M{--}(S, S, D, TypeT)

meaningOfTypedefName {-sem-}(_S, _T, _D, _I {->-}, x) = do
	require $ (x @@ dom{--}(_S) && linkageOfDesignator{--}(_S#(x)) == {-sem-}(TypeLinkage))
	return $ (_S, _T, _D, typeOfDesignator{--}(_S#(x)))
\end{code}

Since typedef names share a single name space with ordinary C variables, yet generally appear in vastly different context within the syntax of the language, they must be distinguished from variable names by propagating certain scope information throughout the lexical analysis of C programs. A complete formalisation of this well-known technique is beyond the scope of the present work and an interested reader is referred to the abundant literature on compiler construction for a further discussion of the topic.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/InitDeclarators}
\set{title}		{Initialised Declarators}
\begin{subsection}

\BeginHaskellToken	{defaultDataOfType}			{Open}	{«\superimpose{\,\,\raise.45ex\hbox{\,\macron{}}}{\delta}»}
  \HaskellArgument							{TypeT}
  \HaskellResult							{DataT VarT}
\EndHaskellToken

\set{capture}		{defaultDataOfType}
\begin{code}
defaultDataOfType {-[*]-} :: TypeT -> DataT VarT

defaultDataOfType {-sem-}(t) ---
  = listToSet [ {-sem-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, IMM (0) (phi_k)) | {-sem-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k) <- setToList (envelopeOfType{--}(t)) ]
\end{code}

We are now ready to define the precise structure of «initialised declarator lists» that appear in the syntax of every C declaration described at the beginning of this section. Syntactically, every such list is represented by a comma-separated sequence of one or more declarators. Each declarator in the sequence may be also followed by an optional symbol “\CKeyword{=}” and an entity known as an «initialiser». Formally:

\IndexType		{InitDeclaratorListT}				{init_declarator_list (\tstar)}
\IndexCons		{IR0}						{init_declarator (init_declarator_list)}
\IndexCons		{IRn}						{, init_declarator_list init_declarator (init_declarator_list)}

\IndexType		{InitDeclaratorT}				{init_declarator (\tstar)}
\IndexCons		{DefaultDeclarator}				{ declarator (init_declarator)}
\IndexCons		{InitDeclarator}				{= declarator initialiser (init_declarator)}

\begin{syntax}
\InsertHaskell		{InitDeclaratorListT, \par, InitDeclaratorT}
\end{syntax}

As described earlier in Section \ref{C/Declarations}, the meaning of all such entities is modelled by an initialiser term "tau", a set of local variables "_V" and the usual updated context components "_S'", "_T'" and "_D'". In particular, every initialised declarator extends the current scope "_S" with a binding of its identifier "x" to a designator with the declarator's type and a linkage derived from any storage class specifiers appearing in its syntax. Formally, a construction of this designator is modelled by the following algorithm "meaningOfVariableDefinition":

\BeginHaskellToken	{meaningOfVariableDefinition}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{x}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, [StorageClassSpecifierT], TypeT {->-}, IdentifierT)}
  \HaskellResult							{_M{--}(DesignatorT)}
\EndHaskellToken

\IndexTerm		{meaningOfVariableDefinition}			{D_X identifier}

\begin{code}
meaningOfVariableDefinition {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, [StorageClassSpecifierT], TypeT {->-}, IdentifierT) -> _M{--}(DesignatorT)

meaningOfVariableDefinition {-sem-}(_S, _T, _D, _I, sc_set, t {->-}, x)

	= 	 do	(ell) <- meaningOfStorageClassSpecifiers{--}(_S, _T, _D, _I, t, x {->-}, sc_set)
  			require	$ (x /@ dom{--}(_S) || indexOfDesignator{--}(_S#(x)) /= _I)
  			return $ {-sem-}(Designator (ell) (t) {-@-} (_I))

	<|>	 do	(ell) <- meaningOfStorageClassSpecifiers{--}(_S, _T, _D, _I, t, x {->-}, sc_set)
  			require	$ (x @@ dom{--}(_S) && indexOfDesignator{--}(_S#(x)) == _I && --
  				   typeOfDesignator{--}(_S#(x)) `areCompatibleTypes` t && linkageOfDesignator{--}(_S#(x)) == ell && isGlobalLinkage{--}(ell))
	  		return $ {-sem-}(Designator (ell) ({-sem-}(typeOfDesignator{--}(_S#(x)) `compositeType` t)) {-@-} (_I))
\end{code}

Intuitively, the above Haskell definition implies that, whenever a given variable name "x" is introduced into the current scope "_S", it is generally assigned its declarator's type "t" and a linkage derived from that type in combination with any storage class specifiers appearing in the declaration's syntax, using the algorithm "meaningOfStorageClassSpecifiers" defined earlier in Section \ref{C/Declarations/StorageClassSpecifiers}. However, if the current scope "_S" already includes a binding of that variable to a designator with the innermost scope index "_I", then the new declaration of "x" must have a C type compatible with that of its existing designator and an identical external or internal linkage form. In this case, the new binding introduced by "meaningOfVariableDefinition" has a C type obtained by a composition of "t" with the type of the existing designator "_S{--}(x)", as described earlier in Section \ref{C/Types/Composition}. Accordingly, the denotation of all well-formed initialised C declarators can be modelled by the following translation:

\BeginHaskellToken	{meaningOfInitDeclaratorList}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{ir}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, [StorageClassSpecifierT], TypeT {->-}, InitDeclaratorListT)}
  \HaskellResult							{_M{--}(S, S, D, V, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfInitDeclaratorList}			{D_IR init_declarator_list}
                                                                      
\begin{code}
meaningOfInitDeclaratorList {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, [StorageClassSpecifierT], TypeT {->-}, InitDeclaratorListT) ->
	_M{--}(S, S, D, V, TermT VarT)
\end{code}

The precise meaning of an initialised C declarator for some identifier "x" without an explicit initialiser is determined by the designator "d" derived from the C type and storage class of the surrounding declaration as follows:

\begin{list}

•	If "d" has a type linkage, then the declarator may specify an arbitrary well-formed object, function or incomplete type for its identifier. Such declarations never introduce new variable bindings into the program's set of item definitions "_D".

•	Otherwise, if "d" has a global linkage form and a function type, or else if the declaration does not include an explicit “"ExternSpecifier"” storage class specifier and its designator describes a globally-linked entity of a well-formed object or incomplete type, then its associated Etude variable "variableOfLinkage{--}(linkageOfDesignator{--}(d))" is bound in the resulting set of item definitions "_D" to an imported Etude item of the form “"IMPI x"”, unless a previous binding of that variable already exists in the program.

•	Further, if the entity's syntax does not include an explicit “"ExternSpecifier"” storage class specifier and if the designator "d" has a global linkage with an object type, then the declaration constitutes a «tentative definition» for "x", represented in the resulting set of item definitions by a binding of its Etude variable to the tentative item “"Nothing"” and, once again, leaving any existing bindings of that variable unchanged. Observe that such declarations cannot specify an incomplete type for the identifier.

•	More so, if the declaration prescribes a private linkage for the identifier, then, in the resulting set of item definitions, its Etude variable is bound to a «default object specification», in which every member is given the initial value of “"integerConstantZero"”. As for tentative C definitions, all such privately-linked variables must have object types.

•	Finally, if the variable's linkage has an automatic or register form “"AutoLinkage nu"” or “"RegisterLinkage nu"”, then "x" must also assume a complete object type by the end of the entire declaration. Its Etude variable "nu" is mapped in "_D" to the tentative linkage form “"Nothing"” and, in the resulting set of local variables "_V", to the envelope of its C type.

\end{list}

In all cases, the declaration's current scope "_S" is extended with a binding of "x" to the constructed designator "d" and the associated initialiser term "tau" is given the trivial Etude form “"RET []"”. The set of local variables "_V" is left empty if the declaration specifies a static or type linkage form for the identifier. Formally:

\begin{code}
meaningOfInitDeclaratorList {-sem-}(_S, _T, _D, _I, sc_set, t_b {->-}, IR0 (DefaultDeclarator r))

	= 	do	(_S', _T', _D', t, x) <- meaningOfDeclarator{--}(_S, _T, _D, _I, t_b {->-}, r)
			(d) <- meaningOfVariableDefinition{--}(_S', _T', _D', _I, sc_set, t {->-}, x)
			require $ (linkageOfDesignator{--}(d) == {-sem-}(TypeLinkage) && --
				  (isObjectType{--}(typeOfDesignator{--}(d)) || --
				   isFunctionType{--}(typeOfDesignator{--}(d)) || --
				   isIncompleteType{--}(typeOfDesignator{--}(d))))
			return $ (_S' // listToMap [ {-raw-}(x {-:-}, d) ], _T', _D', emptyMap, {-sem-}(RET []))
		
	<|>	do	(_S', _T', _D', t, x) <- meaningOfDeclarator{--}(_S, _T, _D, _I, t_b {->-}, r)
			(d) <- meaningOfVariableDefinition{--}(_S', _T', _D', _I, sc_set, t {->-}, x)
			require $	(isGlobalLinkage{--}(linkageOfDesignator{--}(d)) &&
					{-quad-}(isFunctionType{--}(typeOfDesignator{--}(d)) || --
					  	 (sc_set == {-sem-}[ExternSpecifier] && (isObjectType{--}(typeOfDesignator{--}(d)) || isIncompleteType{--}(typeOfDesignator{--}(d))))))
			return $ (_S' // listToMap [ {-raw-}(x {-:-}, d) ], _T', --
				  listToMap [ {-raw-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)) {-:-}, Just {-sem-}(IMPI x)) ] // _D', emptyMap, {-sem-}(RET []))

	<|>	do	(_S', _T', _D', t, x) <- meaningOfDeclarator{--}(_S, _T, _D, _I, t_b {->-}, r)
			(d) <- meaningOfVariableDefinition{--}(_S', _T', _D', _I, sc_set, t {->-}, x)
			require $ (isGlobalLinkage{--}(linkageOfDesignator{--}(d)) && --
				   isObjectType{--}(typeOfDesignator{--}(d)) && --
				   sc_set /= {-sem-}[ExternSpecifier])
			return $ (_S' // listToMap [ {-raw-}(x {-:-}, d) ], _T', --
				  listToMap [ {-raw-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)) {-:-}, Nothing) ] // _D', emptyMap, {-sem-}(RET []))

	<|>	do	(_S', _T', _D', t, x) <- meaningOfDeclarator{--}(_S, _T, _D, _I, t_b {->-}, r)
			(d) <- meaningOfVariableDefinition{--}(_S', _T', _D', _I, sc_set, t {->-}, x)
			require $ (linkageOfDesignator{--}(d) == {-sem-}(PrivateLinkage {-hide-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)))) && --
				   isObjectType{--}(typeOfDesignator{--}(d)))
			return $  (_S' // listToMap [ {-raw-}(x {-:-}, d) ], _T',
				  {-quad-}_D' // listToMap [ {-raw-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)) {-:-}, --
					 				Just {-sem-}(OBJI ({-sem-}(defaultDataOfType{--}(typeOfDesignator{--}(d)))) {-OF-} --
					 							({-sem-}(envelopeOfType{--}(typeOfDesignator{--}(d)))))) ], --
				   emptyMap, {-sem-}(RET []))

	<|>	do	(_S', _T', _D', t, x) <- meaningOfDeclarator{--}(_S, _T, _D, _I, t_b {->-}, r)
			(d) <- meaningOfVariableDefinition{--}(_S', _T', _D', _I, sc_set, t {->-}, x)
			require $ (linkageOfDesignator{--}(d) @@ listToSet [ {-sem-}(AutoLinkage {-hide-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)))), --
				   {-sem-}(RegisterLinkage {-hide-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)))) ] && --
					 isObjectType{--}(typeOfDesignator{--}(d)))
			return $ (_S' // listToMap [ {-raw-}(x {-:-}, d) ], _T', --
				  _D' // listToMap [ {-raw-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)) {-:-}, Nothing) ], --
				  listToMap [ {-raw-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)) {-:-}, envelopeOfType{--}(typeOfDesignator{--}(d))) ], {-sem-}(RET []))
\end{code}

in which "defaultDataOfType{--}(t)" represents the default data specification for an Etude object of the given C type "t", constructed by converting every element "{-hv-}(n, phi, mu_set)" from "t"'s envelope into a datum of the form "{-hv-}(n, phi, mu_set, IMM (0) (phi))":

\IndexTerm		{defaultDataOfType}				{\delta- type (data)}
\InsertHaskell		{defaultDataOfType}

On the other hand, an initialised C declaration of the form “"r `InitDeclarator` i"\,” has a rather less versatile semantics, since, in all such constructs, the newly-introduced identifier "x" must be assigned a static, automatic or register linkage form and a C type that is complete by the end of the initialiser "i". If the declaration specifies a static linkage for the identifier, then "i" must represent a «static initialiser» for the type of "x", of the form scrutinised later in Section \ref{C/Declarations/Initialisers/Static}. Any existing binding of the corresponding Etude variable in "_D" must have the tentative form “"Nothing"” or an imported form “"IMPI x"”. Further, if the declaration specifies a global linkage form for its identifier, then the entire construct must appear in the file scope with the index of 0. The denotation of such an initialiser represents an Etude object specification "delta_set" which, in the resulting set of item definitions "_D", always replaces any previous binding of the Etude variable "x".

Otherwise, if the declaration specifies an automatic or register linkage for the newly-introduced identifier, then "i" must assume the form of a «dynamic initialiser» described in Section \ref{C/Declarations/Initialisers/Dynamic}, whose meaning is represented by an Etude term that, intuitively, populates the corresponding memory-resident object upon its original introduction into the program's address space. The associated Etude variable is always bound to a tentative item “"Nothing"” in the resulting set of item definitions "_D" and, in the associated set of local variables "_V", it is described by an Etude envelope derived from the variable's C type. Formally:

\begin{code}
meaningOfInitDeclaratorList {-sem-}(_S, _T, _D, _I, sc_set, t_b {->-}, IR0 (r `InitDeclarator` i) {-,-})

	= 	do	(_S_r,  _T_r,  _D_r,  t_r,  x			) <- meaningOfDeclarator		{--}(_S,    _T,    _D,    _I,  t_b  {->-}, r  )
  			(_S_i,  _T_i,  _D_i,  t_i,  delta_set {-,,-}	) <- meaningOfStaticInitialiser		{--}(_S_r,  _T_r,  _D_r,  _I,  t_r  {->-}, i  )
			(d) <- meaningOfVariableDefinition{--}(_S_i, _T_i, _D_i, _I, sc_set, t_i {->-}, x)
			require $	(isStaticLinkage{--}(linkageOfDesignator{--}(d)) && --
					 isObjectType{--}(typeOfDesignator{--}(d)) && --
					 (_I == 0 || not (isGlobalLinkage{--}(linkageOfDesignator{--}(d)))) &&
					(variableOfLinkage{--}(linkageOfDesignator{--}(d)) /@ dom{--}(_D_i) || --
					  _D_i#(variableOfLinkage{--}(linkageOfDesignator{--}(d))) @@ listToSet [ Nothing, Just {-sem-}(IMPI x) ]))
			return $ (_S_i // listToMap [ {-raw-}(x {-:-}, d) ], _T_i, --
					 _D_i // listToMap [ {-raw-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)) {-:-}, --
					 				Just {-sem-}(OBJI (delta_set) {-OF-} ({-sem-}(envelopeOfType{--}(typeOfDesignator{--}(d)))))) ], --
					 emptyMap, {-sem-}(RET []))

	<|> 	mdo	(_S_r,  _T_r,  _D_r,  t_r,  x			) <- meaningOfDeclarator		{--}(_S,    _T,    _D,    _I,  t_b {->-}, r)
  			(_S_i,  _T_i,  _D_i,  t_i,  tau			) <- meaningOfDynamicInitialiser	{--}(_S_r,  _T_r,  _D_r,  _I,  t_r, --
  													VAR (variableOfLinkage{--}(linkageOfDesignator{--}(d)))  {->-}, i {-,-})
			(d) <- meaningOfVariableDefinition{--}(_S_i, _T_i, _D_i, _I, sc_set, t_i {->-}, x)
			require $ (linkageOfDesignator{--}(d) @@ listToSet [ {-sem-}(AutoLinkage {-hide-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)))), --
						{-sem-}(RegisterLinkage {-hide-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)))) ] && --
					 isObjectType{--}(typeOfDesignator{--}(d)))
			return $ (_S_i // listToMap [ {-raw-}(x {-:-}, d) ], _T_i, --
					 _D_i // listToMap [ {-raw-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)) {-:-}, Nothing) ], --
					 listToMap [ {-raw-}(variableOfLinkage{--}(linkageOfDesignator{--}(d)) {-:-}, envelopeOfType{--}(typeOfDesignator{--}(d))) ], tau)
\end{code}

Finally, when a single C declaration includes two or more initialised declarator entities, then the meanings of all such declarators are derived from a common base type described by the preceding list of declaration specifiers. In the resulting unified denotation, the local variable set "_V" represents a union of all local variables introduced by any automatic and register declarators in the list and the term "tau" evaluates these declarators' initialisers in the order of their appearance within the program, as captured by the following Haskell construction:

\set{continued}		{yes}
\begin{code}
meaningOfInitDeclaratorList {-sem-}(_S, _T, _D, _I, sc_set, t_b {->-}, irl `IRn` ir {-,-}) = do
	(_S1, _T1, _D1, _V1, tau1) <- meaningOfInitDeclaratorList{--}(_S,   _T,   _D,   _I, sc_set, t_b {->-}, irl {-,-})
	(_S2, _T2, _D2, _V2, tau2) <- meaningOfInitDeclaratorList{--}(_S1,  _T1,  _D1,  _I, sc_set, t_b {->-}, IR0 ir)
	return $ (_S2, _T2, _D2, _V1 `mapUnion` _V2, {-sem-}(LET [{-hv-}[] =: tau1] (tau2)))
\end{code}
\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/Initialisers}
\set{title}		{Initialisation}
\begin{subsection}

Every «initialiser» appearing in a list of initialised declarators described above may represent either a single assignment expression, or else a comma-separated list of such expressions enclosed in braces. In the C Standard, a concrete syntax of these entities is captured by the following BNF grammar:

\IndexType		{InitialiserT}					{initialiser (\tstar)}
\IndexCons		{ExpressionInitialiser}				{ assignment_expression (initialiser)}
\IndexCons		{CompoundInitialiser}				{{} initialiser_list (initialiser)}
\IndexCons		{CompoundInitialiser'}				{{,} initialiser_list (initialiser)}

\IndexType		{InitialiserListT}				{initialiser_list (\tstar)}
\IndexCons		{I0}						{ initialiser (initialiser)}
\IndexCons		{In}						{, initialiser_list initialiser (initialiser)}

\begin{syntax}
\InsertHaskell		{InitialiserT, \par, InitialiserListT}
\end{syntax}

Intuitively, an initialiser describes the original value of the corresponding memory-resident Etude object upon its introduction into the program's address space by the associated C declarator. Before dwelling into the somewhat elaborate semantics of these entities, it is, however, useful to begin their formal treatment with a few simple auxiliary definitions.

First of all, an initialiser whose syntax consists entirely of a single expression, optionally enclosed in braces “"CompoundInitialiser {}"”, is known as an «expression initialiser», while all initialisers with a concrete syntax of the form “\!"CompoundInitialiser (il {-,-})"\!” or “\!"CompoundInitialiser (il {-!-} `Comma` {-!-}{-!-})"\!” are said to be «aggregate». Further, an expression initialiser constructed from a single string literal is also distinguished as a «string initialiser form». Formally, these three kinds of C initialisers are identified by the following Haskell predicate functions:

\BeginHaskellToken	{isExpressionInitialiser}		{Open}	{EXPR}
  \HaskellArgument							{InitialiserT}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{isAggregateInitialiser}		{Open}	{AGGR}
  \HaskellArgument							{InitialiserT}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{isStringInitialiser}			{Open}	{STR}
  \HaskellArgument							{InitialiserT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isExpressionInitialiser}			{EXPR initialiser (bool)}
\IndexTerm		{isAggregateInitialiser}			{AGGR initialiser (bool)}
\IndexTerm		{isStringInitialiser}				{STR initialiser (bool)}

\begin{code}
isExpressionInitialiser		{-[*]-}, isStringInitialiser {-[*]-}, isAggregateInitialiser {-[*]-} :: InitialiserT -> Bool

isExpressionInitialiser		{-sem-}(ExpressionInitialiser (assignment_expression))					= True
isExpressionInitialiser		{-sem-}(CompoundInitialiser (I0 (ExpressionInitialiser (assignment_expression))))	= True
isExpressionInitialiser		{-sem-}(CompoundInitialiser' (I0 (ExpressionInitialiser (assignment_expression))))	= True
isExpressionInitialiser		{-sem-}(other)										= False

isStringInitialiser		{-sem-}(ExpressionInitialiser (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (StringLiteral string_literal))))))))))))))))) ---
															= True
isStringInitialiser		{-sem-}(CompoundInitialiser (I0 (ExpressionInitialiser --
					(E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (StringLiteral string_literal))))))))))))))))))) ---
															= True
isStringInitialiser		{-sem-}(CompoundInitialiser' (I0 (ExpressionInitialiser --
					(E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (StringLiteral string_literal))))))))))))))))))) ---
															= True
isStringInitialiser		{-sem-}(other)										= False

isAggregateInitialiser		{-sem-}(CompoundInitialiser (initialiser_list))						= True
isAggregateInitialiser		{-sem-}(CompoundInitialiser' (initialiser_list))					= True
isAggregateInitialiser		{-sem-}(other)										= False
\end{code}

In the following discussion, expression initialisers are generally converted into plain C expressions with the help of the following Haskell coercion function:

\BeginHaskellToken	{fromExpressionInitialiser}		{Void}	{}
  \HaskellArgument							{InitialiserT}
  \HaskellResult							{ExpressionT}
\EndHaskellToken

\BeginHaskellToken	{fromStringInitialiser}			{Void}	{}
  \HaskellArgument							{InitialiserT}
  \HaskellResult							{StringLiteralT}
\EndHaskellToken

\IndexTerm		{fromExpressionInitialiser}			{ initialiser (expression)}
\IndexTerm		{fromStringInitialiser}				{ initialiser (string_literal)}

\begin{code}
fromExpressionInitialiser	{-[*]-} :: InitialiserT -> ExpressionT

fromExpressionInitialiser	{-sem-}(ExpressionInitialiser (assignment_expression))					= E1 assignment_expression
fromExpressionInitialiser	{-sem-}(CompoundInitialiser (I0 (ExpressionInitialiser (assignment_expression))))	= E1 assignment_expression
fromExpressionInitialiser	{-sem-}(CompoundInitialiser' (I0 (ExpressionInitialiser (assignment_expression))))	= E1 assignment_expression

{-{ Similarly, a string initialiser forms can be converted into a plain string literal entity using the following auxiliary definition: }-}

fromStringInitialiser		{-[*]-} :: InitialiserT -> StringLiteralT

fromStringInitialiser		{-sem-}(ExpressionInitialiser (E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (StringLiteral string_literal))))))))))))))))) ---
															= string_literal
fromStringInitialiser		{-sem-}(CompoundInitialiser (I0 (ExpressionInitialiser --
					(E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (StringLiteral string_literal))))))))))))))))))) ---
															= string_literal
fromStringInitialiser		{-sem-}(CompoundInitialiser' (I0 (ExpressionInitialiser --
					(E2 (E3 (E4 (E5 (E6 (E7 (E8 (E9 (E10 (E11 (E12 (E13 (E14 (E15 (E16 (StringLiteral string_literal))))))))))))))))))) ---
															= string_literal
\end{code}

Further, to simplify the following semantic analysis of aggregate initialisers, it is usually convenient to convert their initialiser sequences into plain Haskell lists, as facilitated by the following simple recursive construction:

\BeginHaskellToken	{listOfInitialisers}			{Open}	{list}
  \HaskellArgument							{InitialiserT}
  \HaskellResult							{[InitialiserT]}
\EndHaskellToken

\IndexTerm		{listOfInitialisers}				{list initialiser (initialisers)}

\begin{code}
listOfInitialisers {-[*]-} :: InitialiserT -> [InitialiserT]

listOfInitialisers {-sem-}(CompoundInitialiser' (il))		= listOfInitialisers{-sem-}(CompoundInitialiser (il))
listOfInitialisers {-sem-}(CompoundInitialiser (il `In` i))	= listOfInitialisers{-sem-}(CompoundInitialiser (il)) ++ [i {-,-}]
listOfInitialisers {-sem-}(CompoundInitialiser (I0 i))		= [i {-,-}]
\end{code}

If such a list is intended as a description of the initial value for a structure, union or array object, its successive elements are assigned to the corresponding «initialisable members» of that object. Formally, this member list is derived from the object's C type "t" by the construction "membersOfAggregateType{--}(t)", as described by the following simple algorithm:

\begin{list}

•	If "t" represents a structure type, then "membersOfAggregateType{--}(t)" consists of all named members found in the type's member list "membersOfType{--}(t)", with every such member further requalified with any additional type qualifiers of "t" itself.

•	Otherwise, if "t" represents a union type, then the list includes only the very first named member from "membersOfType{--}(t)", observing that every complete union must, by construction, include at least one such member.

•	If, on the other hand, "t" represents an array type of some length "n", then "membersOfAggregateType{--}(t)" represents a sequence of "n" anonymous members, or else an infinite sequence if "t" is an incomplete type. All of these members are assigned the same element type of "t" and successive offset values equal to the respective multiples of that type's size, so that the "k"th element of such an array is always placed at the byte offset \hbox{"k * sizeOfType{--}(baseOfType{--}(t))"} from the beginning of the entire object.

\end{list}

Formally:

\BeginHaskellToken	{membersOfAggregateType}		{Open}	{«\macron m\sub{\SmallCaps{i}}»}
  \HaskellArgument							{TypeT}
  \HaskellResult							{MembersT}
\EndHaskellToken

\IndexTerm		{membersOfAggregateType}			{m-I type (members)}

\begin{code}
membersOfAggregateType {-[*]-} :: TypeT -> MembersT

membersOfAggregateType {-sem-}(t) ---
  | isCompoundType{--}(t) && suOfType{--}(t) == {-sem-}(Struct) ---
  	= [ m_k | m_k <- tqOfType{--}(t) `requalifiedMembers` membersOfType{--}(t), nameOfMember{--}(m_k) /= Nothing ]
  | isCompoundType{--}(t) && suOfType{--}(t) == {-sem-}(Union) ---
  	= takeFromList{--}(1, [ m_k | m_k <- tqOfType{--}(t) `requalifiedMembers` membersOfType{--}(t), nameOfMember{--}(m_k) /= Nothing ])
  | isArrayType{--}(t) && lengthOfType{--}(t) /= Nothing ---
	= [ {-sem-}(Member ({-sem-}(baseOfType{--}(t))) ({-hide-}Nothing) {-@-} ({-sem-}(k * sizeOfType{--}(baseOfType{--}(t))))) --
				| k <- [0 .. unJust (lengthOfType{--}(t)) - 1] ] {-!-} {-!-} {-!-} {-!-} {-!-} {-!-}
  | isArrayType{--}(t) && lengthOfType{--}(t) == Nothing ---
	= [ {-sem-}(Member ({-sem-}(baseOfType{--}(t))) ({-hide-}Nothing) {-@-} ({-sem-}(k * sizeOfType{--}(baseOfType{--}(t))))) | k <- [0 .. ] ]
\end{code}

Using these seven auxiliary definitions, the meanings of all C initialiser forms can be given a concise semantic definition presented in Sections \ref{C/Declarations/Initialisers/Static} and \ref{C/Declarations/Initialisers/Dynamic} for the static and dynamic variants of these entities, respectively.

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/Initialisers/Static}
\set{title}		{Static Initialisers}
\begin{subsubsection}

As suggested earlier in Section \ref{C/Declarations/InitDeclarators}, the meaning of every static C initialiser is represented by an Etude data specification set, which determines the initial value of a corresponding statically-linked object at the beginning of the program's execution. Further, under certain conditions discussed shortly, such initialisers may adjust the C types of their associated declarators, which, accordingly, are also included in their complete denotations. In particular, the meaning of every static initialiser form is determined from its structure and the C type "t" of the associated designator "d" in the following manner:

\begin{list}

•	If "t" has a bit field type, then the initialiser must assume an expression form that is derived from a single static initialiser expression "e" of an arithmetic type, as described earlier in Section \ref{C/Expressions/Constants/Static}. The expression's atomic denotation is converted into the type "t" and packed using the familiar bit field constructor "packBitField" into a single datum "{-hv-}(0 {-hv-}, bitFieldFormat {-hv-}, mu_set {-hv-}, IMM (0) (bitFieldFormat))", in which "bitFieldFormat" is the standard bit field format and "mu_set" represents the set of access attributes appropriate for the declarator's type.

•	Similarly, if "t" has any other arithmetic or pointer type, then the initialiser must assume a static initialiser expression form, subject to the same typing constraints as those described earlier in Section \ref{C/Expressions/FunctionArguments} for function argument assignments. Its denotation consists of a single datum "{-hv-}(0 {-hv-}, phi {-hv-}, mu_set {-hv-}, OP1 (CNV phi') (phi) {--} {-hv-}(alpha))", in which "phi'" and "mu_set" represent the format and access attributes of "t", while "alpha" and "phi" are determined by the atomic denotation of the initialiser's expression and the Etude format of its unconverted type, respectively.

•	Otherwise, a string initialiser can be used to prepare the initial content of a complete or incomplete array type, provided that the array's length, if known, is greater or equal to one less then the length of the string literal's type, and that both "t" and the string literal itself represent arrays of character types, or else both depict arrays with elements that are compatible with a qualified or unqualified version of the implementation-defined type “"wideCharType"”, so that all of the literal's characters with the possible exception of its implicit NUL terminator can be accommodated by the targeted object. In both cases, the resulting denotation consists of the data specification set derived from the string literal's syntax, except that, if the targeted array object is longer than its initialiser, then any remaining elements of the array are implicitly assigned the default data specification described earlier in Section \ref{C/Declarations/InitDeclarators}. If the declarator has an incomplete type, then the initialiser also completes that array with the length of the specified string literal.

•	In all other cases, if "t" represents a non-scalar object type or an array of an unknown length, then its initialiser must always have an aggregate form, whose elements "i_set" are assigned to the successive initialisable members of "t", forming a set of data specifications "delta_set" using an algorithm "meaningOfStaticInitialisers" described later in this section.  If "t" represents an incomplete array type prior to initialisation, then, in the resulting denotation, that type is completed with the length "n" equal to the number of "t"'s members initialised by "i_set". Otherwise, "i_set" must not specify more elements than those required for a complete initialisation of "t". If it specifies fewer elements than required, then, in "delta_set", any remaining members are assigned their default data specifications.

\end{list}

In Haskell, the above algorithm can be implemented concisely as follows:

\BeginHaskellToken	{meaningOfStaticInitialiser}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{si}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, TypeT {->-}, InitialiserT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT, DataT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfStaticInitialiser}			{D_SI initialiser}

\begin{code}
meaningOfStaticInitialiser {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D, I, TypeT {->-}, InitialiserT) -> _M{--}(S, S, D, TypeT, DataT VarT)

meaningOfStaticInitialiser {-sem-}(_S, _T, _D, _I, t {->-}, i {-,-}) 

	=	do	require $ (isBitFieldType{--}(t) && isExpressionInitialiser{--}(i {-,-}))
			(_S', _T', _D', t', alpha) <- meaningOfStaticInitialiserExpression{--}(_S, _T, _D, _I {->-}, fromExpressionInitialiser i {-,-})
  			require $ (isArithmeticType{--}(t'))
			return $ (_S', _T', _D', t, --
					 listToSet {-sem-}[ {-hv-}(0 {-hv-}, bitFieldFormat {-hv-}, {-sem-}(attrOfType{--}(t)) {-hv-}, --
				 				{-sem-}(packBitField{-sem-}(t {->-}, --
				 							IMM (0) (bitFieldFormat), --
				 							OP1 (CNV {-sem-}(formatOfType{--}(t))) --
													({-sem-}(formatOfType{--}(t'))) {--} {-hv-}(alpha)))) ])

	<|>	do	require $ (isArithmeticType{--}(t) && not (isBitFieldType{--}(t)) || (isPointerType{--}(t))) && (isExpressionInitialiser{--}(i {-,-}))
			(_S', _T', _D', t', alpha) <- meaningOfStaticInitialiserExpression{--}(_S, _T, _D, _I {->-}, fromExpressionInitialiser i {-,-})
  			require $  (isArithmeticType{--}(t) && isArithmeticType{--}(t')) ||
  				   (isPointerType{--}(t) && isPointerType{--}(t') && ( ---
  						(unqualifiedType{--}(baseOfType{--}(t)) `areCompatibleTypes` unqualifiedType{--}(baseOfType{--}(t'))) ||
  						(isVoidType{--}(baseOfType{--}(t'  )) && (isObjectType{--}(baseOfType{--}(t   )) || isIncompleteType{--}(baseOfType{--}(t   )))) ||
  					 	(isVoidType{--}(baseOfType{--}(t   )) && (isObjectType{--}(baseOfType{--}(t'  )) || isIncompleteType{--}(baseOfType{--}(t'  ))))) &&
					(tqOfType{--}(baseOfType{--}(t')) `subseteq` tqOfType{--}(baseOfType{--}(t)))) || 
  				   (isPointerType{--}(t) && isNullPointerConstant{--}(_S, _T, _D, _I {->-}, fromExpressionInitialiser i {-,-}))
			return $ (_S', _T', _D', t, --
					 listToSet {-sem-}[ {-hv-}(0 {-hv-}, formatOfType{--}(t) {-hv-}, attrOfType{--}(t) {-hv-}, --
				 			OP1 (CNV {-sem-}(formatOfType{--}(t))) ({-sem-}(formatOfType{--}(t'))) {--} {-hv-}(alpha)) ])

	<|>	do	require $ (isStringInitialiser{--}(i {-,-}) && isArrayType{--}(t))
			(t_i, delta_set {-,,-}) <- meaningOfStringLiteral{--}(fromStringInitialiser i {-,-})
  			require $  (  (isCharType{--}(baseOfType{--}(t)) && isCharType{--}(baseOfType{--}(t_i))) || --
  				      (inRange (unqualifiedType{--}(baseOfType{--}(t))) (areCompatibleTypes {- () {-approx-} () -}) --
  				 		 (baseOfType{--}(t_i)) (areCompatibleTypes {- () {-approx-} () -}) --
  				 		 ({-sem-}(wideCharType)))) &&
				   (lengthOfType{--}(t) == Nothing || unJust (lengthOfType{--}(t)) >= unJust (lengthOfType{--}(t_i)) - 1)
			return $  (_S, _T, _D, completeArrayType{--}(t, unJust (lengthOfType{--}(t_i))),
				  {-quad-}delta_set `union` unions{--}[ defaultDataOfType{--}(typeOfMember{--}(m_k)) `shiftData` offsetOfMember{--}(m_k) --
									  | m_k <- dropFromList{--}(cardinality(delta_set), --
									    membersOfAggregateType{--}(completeArrayType{--}(t, unJust (lengthOfType{--}(t_i))))) ])

	<|>	do	require $ ((not (isScalarType{--}(t)) && isObjectType{--}(t) || isArrayType{--}(t)) && isAggregateInitialiser{--}(i {-,-}))
			(_S', _T', _D', delta_set, n, i_set') <- meaningOfStaticInitialisers{--}(_S, _T, _D, _I, membersOfAggregateType{--}(t) {->-}, listOfInitialisers{--}(i {-,-}))
  			require $ (i_set' == {-empty-}[])
  			return $  (_S', _T', _D', completeArrayType{--}(t, n),
  				  {-quad-}packData{--}(delta_set `union` unions{--}[ defaultDataOfType{--}(typeOfMember{--}(m_k)) `shiftData` offsetOfMember{--}(m_k) --
											| m_k <- dropFromList{--}(n, membersOfAggregateType{--}(completeArrayType{--}(t, n))) ]))
\end{code}

The actual completion of an array's length is performed in the above definition using the following natural Haskell construction:

\BeginHaskellToken	{completeArrayType}			{Open}	{\ZapfChancery{C}\sub{\SmallCaps{a}}}
  \HaskellArgument							{(Integer {->-}, TypeT)}
  \HaskellResult							{TypeT}
\EndHaskellToken

\IndexTerm		{completeArrayType}				{C_A type integer (type)}

\begin{code}
completeArrayType {-[*]-} :: (TypeT, Integer) -> TypeT

completeArrayType {-sem-}(t, n) ---
  | (isArrayType{--}(t) && lengthOfType{--}(t) == Nothing)	= {-sem-}(ArrayType ({-sem-}(baseOfType{--}(t))) (Just n))
  | otherwise							= t
\end{code}

Further, in the above formalisation of static initialisers, all otherwise uninitialised members of a structure, union or array type are assigned their correct default values by computing a union of the set "delta_set" derived from any explicitly specified initialisers for the object with the default data specification sets "defaultDataOfType{--}(typeOfMember{--}(m_k)) `shiftData` offsetOfMember{--}(m_k)" for every remaining uninitialised but initialisable member "m_k". Intuitively, “"delta_set `shiftData` n"” simply relocates every datum "{-hv-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k) @@ delta_set" to a new offset "n_k + n" within the surrounding structure, union or array, as depicted by the following list comprehension:

\BeginHaskellToken	{shiftData}				{Void}	{}
  \HaskellArgument							{DataT VarT}
  \HaskellToken		{{-(+)-}}				{Bin}	{\oplus}
  \HaskellArgument							{Integer}
  \HaskellResult							{DataT VarT}
\EndHaskellToken

\NewHaskellToken	{{-(+)-}}				{Bin}	{\oplus}
\NewHaskellToken	{`shiftData`}				{Bin}	{\oplus}

\IndexTerm		{shiftData}					{\oplus data integer (data)}

\begin{code}
shiftData {-[*]-} 			{-(+)-} {-[*]-} :: DataT VarT -> Integer -> DataT VarT

shiftData {-sem-}(delta_set {-,,-}) 	{-(+)-} {-sem-}(n) --
	= listToSet [ {-sem-}(n_k + n {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k) | {-sem-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k) <- setToList delta_set {-,,-} ]
\end{code}

Finally, the initialisation algorithm must «compact» all initialisers for every group of bit field members that, in "delta_set", are placed by the structure layout algorithm "layoutOfMembers" from Section \ref{C/Declarations/StructAndUnionSpecifiers} at the same offset into the entire object, into a single datum entity. In the above definition of "meaningOfStaticInitialiser", this process is represented by the notation “"packData{--}(delta_set {-,,-})"”. In particular, if "delta_set" contains two or more elements with the same format "phi" and byte offset value "n", then, in the compacted set of data specification "packData{--}(delta_set {-,,-})", all such elements are combined into a single tuple "{-hv-}(n {-hv-}, phi {-hv-}, mu_set' {-hv-}, alpha')", in which "mu_set'" represents an intersection of the individual access attribute sets assigned to each of these overlapping elements and "alpha'" depicts a single Etude atom with a value derived by an implementation-defined construction "packAtoms{--}(alpha_set)". In this chapter, such atoms are introduced by the following Haskell function:

\ReindexTerm		{packAtoms}

\begin{haskell}
packAtoms {-[*]-} :: AtomsT VarT -> AtomT VarT
\end{haskell}

Although the precise semantics of this construct are left unspecified in the generic fragment of the language, every implementation of a C compiler must ensure that, for every well-formed bit field type "t_k @@ t_set" and atom "alpha_k `EqvJ` packBitField{-sem-}(t_k {->-}, IMM (0) (bitFieldFormat), alpha_k') @@ alpha_set", the atom's original value "alpha_k'" can be always retrieved from "packAtoms{--}(t_set {->-}, alpha_set)" using the standard bit field extraction operation "unpackBitField{--}(t_k {->-}, packAtoms{--}(t_set {->-}, alpha_set))", provided that all types in "t_set" specify non-overlapping bit ranges. Formally:

\NewHaskellToken	{packAtomsJ}				{Ord}	{\TKeyword{pack\sub{i}}}
\NewHaskellDescription	{packAtomsJ}					{Superimposition of bit field initialisers}
\IndexTheorem		{packAtomsJ}					{PACK_I Superimposition of bit field initialisers}

\begin{haskell}
packAtomsJ :: forall alpha_set, alpha_set' t_set =>
	WfJ{--}(alpha_set) -> WfJ{--}(alpha_set') ->
	{-sem-}(lengthOfList{--}(alpha_set) == lengthOfList{--}(alpha_set') == lengthOfList{--}(t_set)) ->
	{-sem-}(cardinality (unions{--}[ listToSet [ offsetOfType{--}(t_k) .. offsetOfType{--}(t_k) + widthOfType{--}(t_k) - 1] | t_k <- t_set ]) --
			== sum{--}[ widthOfType{--}(t_k) | t_k <- t_set ]) ->
	(	forall t_k => --
			{-sem-}(t_k @@ t_set) -> --
			{-sem-}(isBitFieldType{--}(t_k))) ->
	(	forall alpha_k, alpha_k', t_k => --
			{-sem-}((alpha_k, alpha_k', t_k) @@ alpha_set `zipList` alpha_set' `zipList` t_set {-,-}) -> --
			{-sem-}(alpha_k) `EqvJ` {-sem-}(packBitField{-sem-}(t_k {->-}, IMM (0) (bitFieldFormat), alpha_k'))) ->
	(	forall alpha_k, alpha_k', t_k => --
			{-sem-}((alpha_k, alpha_k', t_k) @@ alpha_set `zipList` alpha_set' `zipList` t_set {-,-}) -> --
			{-sem-}(alpha_k') `EqvJ` {-sem-}(unpackBitField{-sem-}(t_k {->-}, alpha_k)))
\end{haskell}

Using this auxiliary definition, compaction of a well-formed Etude data specification set can be formalised concisely as the following recursive algorithm:

\BeginHaskellToken	{packData}				{Open}	{\ZapfChancery{P}}
  \HaskellArgument							{DataT VarT}
  \HaskellResult							{DataT VarT}
\EndHaskellToken

\IndexTerm		{packData}					{P data (data)}

\begin{code}
packData {-[*]-} :: DataT VarT -> DataT VarT

packData {-sem-}(delta_set {-,,-}) ---
  | (delta_set == emptySet)	= emptySet
  | otherwise			= listToSet [ {-sem-}(n {-hv-}, phi {-hv-}, mu_set' {-hv-}, alpha') ] `union` packData{--}(delta_set' {-,-})

 where	Just {-sem-}(n {-hv-}, phi {-hv-}, mu_set {-hv-}, alpha) = Just (minimumOfSet{--}(delta_set {-,,-}))

 	mu_set'		= intersections		{--}[ mu_set_k  | {-sem-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k) <- setToList delta_set, n_k == n && phi_k == phi ]
	alpha'		= packAtoms 		{--}[ alpha_k   | {-sem-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k) <- setToList delta_set, n_k == n && phi_k == phi ]
	delta_set'	= listToSet [ {-sem-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k) --
					| {-sem-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k) <- setToList delta_set, n_k /= n || phi_k /= phi ]
\end{code}

Last but not least, the actual meaning of an initialiser list "i_set" for a given sequence of aggregate members "m_set" is represented by the following Haskell function:

\BeginHaskellToken	{meaningOfStaticInitialisers}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{sil}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, MembersT {->-}, [InitialiserT])}
  \HaskellResult							{_M{--}(S, S, D, DataT VarT, Integer, [InitialiserT])}
\EndHaskellToken

\IndexTerm		{meaningOfStaticInitialisers}			{D_SIL initialisers}

\begin{code}
meaningOfStaticInitialisers {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, MembersT {->-}, [InitialiserT]) -> _M{--}(S, S, D, DataT VarT, Integer, [InitialiserT])

meaningOfStaticInitialisers {-sem-}(_S, _T, _D, _I, m_set {->-}, i_set)

	=	do	require $ (m_set == {-empty-}[] || i_set == {-empty-}[])
  			return $ (_S, _T, _D, emptySet, 0, i_set)

	<|>	do	require $ (isObjectType{--}(typeOfMember{--}(m0)) && not (isScalarType{--}(typeOfMember{--}(m0))) && not (isAggregateInitialiser{--}(head{--}(i_set))))
  			(_S1, _T1, _D1, delta_set1, n1, i_set1) <- meaningOfStaticInitialisers{--}(_S,   _T,   _D,   _I, membersOfAggregateType{--}(typeOfMember{--}(m0)) {->-}, i_set)
  			(_S2, _T2, _D2, delta_set2, n2, i_set2) <- meaningOfStaticInitialisers{--}(_S1,  _T1,  _D1,  _I, tail{--}(m_set) {->-}, i_set1)
  			return $ (_S2, _T2, _D2, packData{--}(delta_set1 `shiftData` offsetOfMember{--}(m0)) `union` delta_set2, 1 + n2, i_set2)

	<|>	do	(_S1, _T1, _D1, t,  delta_set1 {-,-}) <- meaningOfStaticInitialiser{--}(_S, _T, _D, _I, typeOfMember{--}(m0) {->-}, head{--}(i_set))
  			(_S2, _T2, _D2, delta_set2, n, i_set') <- meaningOfStaticInitialisers{--}(_S1, _T1, _D1, _I, tail{--}(m_set) {->-}, tail{--}(i_set))
  			return $ (_S2, _T2, _D2, delta_set1 `shiftData` offsetOfMember{--}(m0) `union` delta_set2, 1 + n, i_set')
  		
 where m0 = head{--}(m_set)
\end{code}

Intuitively, the above inductive algorithm processes all members "m_k @@ m_set" in the order of their appearance in the list, progressively applying one or more elements from "i_set" as an initialiser for that member. In particular, if "m_k" represents a member of a non-scalar object type and the list "i_set" begins with a non-aggregate initialiser, then the construction obtains as many successive elements from "i_set" as required for all the initialisable members of "m_k", using a recursive application of that initialisable member list to "i_set". Any initialisers remaining in "i_set" at the end of this process are then applied to the following members in "m_set". Otherwise, the very first element of "i_set" must represent a complete well-formed initialiser for the type of "m_k" and that initialiser alone is applied to the member's C type using the algorithm "meaningOfStaticInitialiser" described earlier in this section. In all cases, the resulting denotation is formed from a compacted union of appropriately-relocated data specification sets for all explicitly-initialised members of "m_set", together with the actual number of these members and the list of any unused initialisers from "i_set". The algorithm terminates when either of the two lists "m_set" or "i_set" has been exhausted. If the end of "m_set" is reached first, then all members have been initialised completely and the remaining elements of "i_set" are made available for use with any following elements of the surrounding structure or array; otherwise, if "i_set" was exhausted before the end of "m_set" has been reached, then any remaining members of the structure will be eventually assigned their default data specifications as part of the earlier algorithm "meaningOfStaticInitialiser".

\end{subsubsection}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Declarations/Initialisers/Dynamic}
\set{title}		{Dynamic Initialisers}
\begin{subsubsection}

Last but not least, every well-formed «dynamic C initialiser» denotes a monadic Etude term whose evaluation is intended to populate the initial content of some memory-resident object located at a given address "alpha" within the program's memory image. If the initialiser has an expression form, then this term behaves as if that expression was assigned to a function argument with a C type of the associated declarator, subject to the same constraints as those discussed earlier in Section \ref{C/Expressions/FunctionArguments} for such assignments. Otherwise, if an aggregate initialiser is used to initialise a non-scalar l-value, or if a string initialiser is utilised in preparation of an appropriately-typed array object, then the construct represents a «dynamic aggregate initialiser» and its semantics are defined later in this section. The precise meaning of every dynamic initialiser form is determined from the entity's syntax and the type "t" of the associated C declarator by the following algorithm:

\begin{list}

•	If the surrounding C declarator has a non-bit field scalar type, then the initialiser's value is evaluated and stored in the l-value described by the parameter "alpha", using the familiar Etude constructor “"INI"”. Observe that, in a well-formed C program, "t" will never assume a bit field form.

•	Otherwise, if "t" represents a complete structure or union type and the initialiser has a non-aggregate form of an expression whose type is compatible with a qualified or unqualified variant of "t", then the initialisation is performed using the construction “"iniObject ((t {->-}, alpha)) {-to-} ((t' {->-}, alpha'))"” discussed earlier in Section \ref{C/Expressions/FunctionArguments}.

•	Finally, if "t" has a non-scalar type and the initialiser has an aggregate form, or else if "t" represents an array and the initialiser depicts a single string literal entity, then its «dynamic aggregate» meaning is derived using the algorithm "meaningOfDynamicAggregateInitialiser" defined later in this section. Every such dynamic aggregate initialiser denotes a set of Etude data specifications "delta_set" similar to that derived earlier from static initialisers of the same structure. In the resulting denotation, every datum \hbox{"{-hv-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k) @@ delta_set"} is converted into a binding of the form \hbox{“"{-hv-}[] =: INI (OP2 (OP1 (CNV {-sem-}(objectFormat{--}(phi_k))) ({-sem-}(formatOfType{--}(t))) {--} {-hv-}(alpha)) (ADD) ({-sem-}(objectFormat{--}(phi_k))) (IMM (toRational n_k) (ZG`FMT`standardEncoding)) {-hv-}, mu_set_k) (phi_k) {-TO-} (alpha_k)"”,} which, intuitively, stores the value of "alpha_k" in a memory-resident object found at an offset "n_k" into the address space region located at the address "alpha". All such bindings are then placed within a single Etude group, leaving the precise order of their evaluation unspecified.

\end{list}

In the first two cases, the initialiser may represent an arbitrary C expression, so that all temporary variables required for its evaluation must be introduced into the program's address space at the beginning of the construct and purged before proceeding with the translation of any following entities in the program. In Section \ref{C/Expressions}, all such temporary objects were represented in an expression's denotation by its local variable set "_V", so that, following the principles from Section \ref{Lambda/Terms}, the required list of object environment extensions must be modelled in the initialiser's denotation by a sequence of Etude terms “"NEW (xi_set_k)"” for every envelope "xi_set_k" bound to some variable in "_V". Formally, this term sequence is introduce by a «scope constructor» combinator "scopeConstructor{--}(_V)", which, intuitively, maps every binding "nu_k {-:-}, xi_set_k @@ _V" to a partially-applied Etude term “"LET [ nu_k =: NEW (xi_set_k)]"” and composes these terms into a single Etude construct using the list operator “"composeList"” defined in Appendix \ref{Additional}. Further, the dual «scope destructor» combinator "scopeDestructor{--}(_V)" purges all such temporary objects from the program's address space once they are no longer required for its execution, which, in Etude, is modelled by an analogous sequence of “"DEL (xi_set_k)"” term forms, applied in the reverse order of the corresponding objects' allocation, so that the minimal requirements of the address space contraction operations are always satisfied as described in Section \ref{Lambda/State}. In Haskell, these two combinators are represented by the following pair of functions:

\BeginHaskellToken	{scopeConstructor}			{Open}	{\ZapfChancery{C}}
  \HaskellArgument							{V}
  \HaskellResult							{(TermT VarT -> TermT VarT)}
\EndHaskellToken

\BeginHaskellToken	{scopeDestructor}			{Open}	{\ZapfChancery{D}}
  \HaskellArgument							{V}
  \HaskellResult							{(TermT VarT -> TermT VarT)}
\EndHaskellToken

\IndexTerm		{scopeConstructor}				{C V (term term)}
\IndexTerm		{scopeDestructor}				{D V (term term)}

\begin{code}
scopeConstructor 	{-[*]-}, scopeDestructor {-[*]-} :: V -> (TermT VarT -> TermT VarT)

scopeConstructor 	{-sem-}(_V) = composeList{--}[ {-sem-}(LET [{-raw-}[nu_k] =: NEW (xi_set_k)]) | {-raw-}(nu_k {-:-}, xi_set_k) <- mapToList _V ]
scopeDestructor		{-sem-}(_V) = composeList{--}(reverse{--}[ {-sem-}(LET [{-hv-}[] =: DEL (xi_set_k)]) | {-raw-}(nu_k {-:-}, xi_set_k) <- mapToList _V ])
\end{code}

Using this pair of auxiliary definitions, the meaning of every well-formed dynamic C initialiser can be formalised as follows:

\BeginHaskellToken	{meaningOfDynamicInitialiser}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{di}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, TypeT, AtomT VarT {->-}, InitialiserT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfDynamicInitialiser}			{D_DI initialiser}

\begin{code}
meaningOfDynamicInitialiser {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, TypeT, AtomT VarT {->-}, InitialiserT) -> _M{--}(S, S, D, TypeT, TermT VarT)

meaningOfDynamicInitialiser {-sem-}(_S, _T, _D, _I, t, alpha {->-}, i {-,-})

	=	do	require $ (isScalarType{--}(t) && not (isBitFieldType{--}(t)) && isExpressionInitialiser{--}(i {-,-}))
			(_S', _T', _D', _V, t', tau) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, fromExpressionInitialiser i {-,-})
			require $	(isArithmeticType{--}(t) && isArithmeticType{--}(t')) ||
					(isPointerType{--}(t) && isPointerType{--}(t') && ---
						     (	(unqualifiedType{--}(baseOfType{--}(t)) `areCompatibleTypes` unqualifiedType{--}(baseOfType{--}(t'))) ||
							((isObjectType{--}(baseOfType{--}(t)) || isIncompleteType{--}(baseOfType{--}(t))) && isVoidType{--}(baseOfType{--}(t'))) ||
							((isObjectType{--}(baseOfType{--}(t')) || isIncompleteType{--}(baseOfType{--}(t'))) && isVoidType{--}(baseOfType{--}(t))))) ||
					(isPointerType{--}(t) && isNullPointerConstant{--}(_S, _T, _D, _I {->-}, fromExpressionInitialiser i {-,-}))
  			return $  (_S', _T', _D', t,
	  			  {-quad-}{-sem-}( ---
						{-sem-}(scopeConstructor{--}(_V)) `composeTerms`
  						LET [{-raw-}[TVar 1] =: tau] $
  					 	LET [{-hv-}[] =: INI (alpha {-hv-}, {-sem-}(attrOfType{--}(t))) ({-sem-}(formatOfType{--}(t))) --
			  					 		{-TO-} ({-hv-}(OP1 (CNV {-sem-}(formatOfType{--}(t))) --
			  					 					({-sem-}(formatOfType{--}(t'))) {--} {-hv-}(VAR (TVar 1))))] $
			  			{-sem-}(scopeDestructor{--}(_V)) `completeTerm`
			  			RET []))

	<|>	do	require $ (isCompoundType{--}(t) && isObjectType{--}(t) && not (isAggregateInitialiser{--}(i {-,-})))
			(_S', _T', _D', _V, t', tau) <- meaningOfValueExpression{--}(_S, _T, _D, _I {->-}, fromExpressionInitialiser i {-,-})
			require $ (unqualifiedType{--}(t) `areCompatibleTypes` unqualifiedType{--}(t'))
  			return $  (_S', _T', _D', t,
	  			  {-quad-}{-sem-}( ---
						{-sem-}(scopeConstructor{--}(_V)) `composeTerms`
  						LET [{-raw-}[TVar 1] =: tau] $
  					 	LET [{-hv-}[] =: {-sem-}(iniObject ((t {->-}, alpha)) {-to-} ({-sem-}(t' {->-}, VAR (TVar 1))))] $
			  			{-sem-}(scopeDestructor{--}(_V)) `completeTerm`
			  			RET []))
			
	<|>	do	require $ (not (isScalarType{--}(t)) && isAggregateInitialiser{--}(i {-,-})) || (isArrayType{--}(t) && isStringInitialiser{--}(i {-,-}))
			(_S', _T', _D', t', delta_set {-,,-}) <- meaningOfDynamicAggregateInitialiser{--}(_S, _T, _D, _I, t {->-}, i {-,-})
			(beta_set) <- {-hide-}return $ ---
						 [ {-sem-}({-hv-}[] =: INI (OP2 (OP1 (CNV {-sem-}(objectFormat{--}(phi_k))) ({-sem-}(formatOfType{--}(t))) {--} {-hv-}(alpha)) --
										(ADD) ({-sem-}(objectFormat{--}(phi_k))) --
										(IMM (toRational n_k) (ZG`FMT`standardEncoding)) {-hv-}, mu_set_k) (phi_k) {-TO-} (alpha_k))
						 {-quad-} | {-sem-}({-hv-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k)) <- setToList delta_set {-,,-} ]
			return $ (_S', _T', _D', t', {-sem-}(LET (beta_set) (RET [])))
\end{code}

In particular, if a dynamic aggregate initialiser is used to populate an object of a scalar C type, then the construction has a meaning identical to its earlier static initialiser interpretation from Section \ref{C/Declarations/Initialisers/Static}, as epitomised by the algorithm "meaningOfStaticInitialiser". Further, in the present context, a dynamic string initialiser also behaves similarly to its earlier static variant, except that the resulting object is never padded with the implicit “"integerConstantZero"” values, even when such a string is used to populate a dynamically-allocated array object of a length greater than itself, keeping in mind that the implicit NUL terminator is still retained at the end of every string literal's denotation. In every other form of a dynamic aggregate initialiser, the list of all initialiser entities found in the construct's syntax is applied recursively to the object's initialisable members using the algorithm "meaningOfDynamicInitialisers" defined at the end of the present section. Formally:

\BeginHaskellToken	{meaningOfDynamicAggregateInitialiser}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{dai}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, TypeT {->-}, InitialiserT)}
  \HaskellResult							{_M{--}(S, S, D, TypeT, DataT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfDynamicAggregateInitialiser}		{D_DAI initialiser}

\begin{code}
meaningOfDynamicAggregateInitialiser {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, TypeT {->-}, InitialiserT) -> _M{--}(S, S, D, TypeT, DataT VarT)

meaningOfDynamicAggregateInitialiser {-sem-}(_S, _T, _D, _I, t {->-}, i {-,-}) 

	=	do	require $ (isScalarType{--}(t))
			meaningOfStaticInitialiser{--}(_S, _T, _D, _I, t {->-}, i {-,-})

	<|>	do	require $ (isStringInitialiser{--}(i {-,-}) && isArrayType{--}(t))
			(t_i, delta_set {-,,-}) <- meaningOfStringLiteral{--}(fromStringInitialiser i {-,-})
  			require $  (  (isCharType{--}(baseOfType{--}(t)) && isCharType{--}(baseOfType{--}(t_i))) || --
  				      (inRange (unqualifiedType{--}(baseOfType{--}(t))) (areCompatibleTypes {- () {-approx-} () -}) --
  				 		 (baseOfType{--}(t_i)) (areCompatibleTypes {- () {-approx-} () -}) --
  				 		 ({-sem-}(wideCharType)))) &&
				   (lengthOfType{--}(t) == Nothing || unJust (lengthOfType{--}(t)) >= unJust (lengthOfType{--}(t_i)) - 1)
			return $  (_S, _T, _D, completeArrayType{--}(t, unJust (lengthOfType{--}(t_i))), delta_set {-,,-})

	<|>	do	require $ ((not (isScalarType{--}(t)) && isObjectType{--}(t) || isArrayType{--}(t)) && isAggregateInitialiser{--}(i {-,-}))
			(_S', _T', _D', delta_set, n, i_set') --
				<- meaningOfDynamicInitialisers{--}(_S, _T, _D, _I, membersOfAggregateType{--}(t) {->-}, listOfInitialisers{--}(i {-,-}))
  			require $ (i_set' == {-empty-}[])
  			return $  (_S', _T', _D', completeArrayType{--}(t, n),
  				  {-quad-}packData{--}(delta_set `union` unions{--}[ defaultDataOfType{--}(typeOfMember{--}(m_k)) `shiftData` offsetOfMember{--}(m_k) --
											| m_k <- dropFromList{--}(n, membersOfAggregateType{--}(completeArrayType{--}(t, n))) ]))
\end{code}

In particular, any aggregate initialiser forms that appear within the syntax of a dynamic aggregate initialiser are translated identically to their static equivalents, except that their individual members are processed recursively using the above function "meaningOfDynamicAggregateInitialiser" instead of its earlier static variant "meaningOfStaticInitialiser", in order to ensure that no default padding is ever inserted into any string-initialised arrays embedded within the targeted object. Accordingly, the actual translation of all such initialiser lists is formulated in Haskell as follows:

\BeginHaskellToken	{meaningOfDynamicInitialisers}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{dil}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, MembersT {->-}, [InitialiserT])}
  \HaskellResult							{_M{--}(S, S, D, DataT VarT, Integer, [InitialiserT])}
\EndHaskellToken

\IndexTerm		{meaningOfDynamicInitialisers}			{D_DIL initialisers}

\set{continued}		{yes}
\begin{code}
meaningOfDynamicInitialisers {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, I, MembersT {->-}, [InitialiserT]) -> _M{--}(S, S, D, DataT VarT, Integer, [InitialiserT])

meaningOfDynamicInitialisers {-sem-}(_S, _T, _D, _I, m_set {->-}, i_set)

	=	do	require $ (m_set == {-empty-}[] || i_set == {-empty-}[])
  			return $ (_S, _T, _D, emptySet, 0, i_set)

	<|>	do	require $ (isObjectType{--}(typeOfMember{--}(m0)) && not (isScalarType{--}(typeOfMember{--}(m0))) && not (isAggregateInitialiser{--}(head{--}(i_set))))
  			(_S1, _T1, _D1, delta_set1, n1, i_set1) <- meaningOfDynamicInitialisers{--}(_S,   _T,   _D,   _I, membersOfAggregateType{--}(typeOfMember{--}(m0)) {->-}, i_set)
  			(_S2, _T2, _D2, delta_set2, n2, i_set2) <- meaningOfDynamicInitialisers{--}(_S1,  _T1,  _D1,  _I, tail{--}(m_set) {->-}, i_set1)
  			return $ (_S2, _T2, _D2, packData{--}(delta_set1 `shiftData` offsetOfMember{--}(m0)) `union` delta_set2, 1 + n2, i_set2)

	<|>	do	(_S1, _T1, _D1, t,  delta_set1 {-,-}) <- meaningOfDynamicAggregateInitialiser{--}(_S, _T, _D, _I, typeOfMember{--}(m0) {->-}, head{--}(i_set))
  			(_S2, _T2, _D2, delta_set2, n, i_set') <- meaningOfDynamicInitialisers{--}(_S1, _T1, _D1, _I, tail{--}(m_set) {->-}, tail{--}(i_set))
  			return $ (_S2, _T2, _D2, delta_set1 `shiftData` offsetOfMember{--}(m0) `union` delta_set2, 1 + n, i_set')
  		
 where m0 = head{--}(m_set)
\end{code}
\end{subsubsection}
\end{subsection}
\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
