\set{capture}{hidden}
\begin{code}

{-# LANGUAGE RecursiveDo #-}

module C.Definitions (
	meaningOfTranslationUnit,
	meaningOfExternalDeclarations,
	meaningOfFunctionDefinition,
) where

import Control.Monad.Fix
import Additional
import C.Syntax
import C.Variables
import C.Types
import C.Scopes
import C.Constants
import C.Expressions
import C.Declarations
import C.Statements
import {-# SOURCE #-} C.Implementation
import Lambda

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{C/Definitions}
\set{title}{External Definitions}
\begin{section}

Traditionally, large C programs are generally developed as a collection of separate «source files», every one of which corresponds, intuitively, to a single Etude module of the form described earlier in Section \ref{Lambda/Modules}. Formally, such independent units of compilation are represented in the C grammar by syntactic entities known as «translation units», which, in concrete terms, consist of one or more «external declarations», as captured by the following Haskell definition of their syntax:

\IndexType		{TranslationUnitT}				{translation_unit (\tstar)}
\IndexCons		{U0}						{ external_declaration (translation_unit)}
\IndexCons		{Un}						{ translation_unit external_declaration (translation_unit)}

\begin{syntax}
\InsertHaskell		{TranslationUnitT}
\end{syntax}

The precise syntax and semantics of the individual external declarations in a translation union are discussed separately in Section \ref{C/Definitions/External}, whereby a list of such entities, taken in the context of an initially empty current scopes and item definition set "emptySet", is used to derive the complete scopes "_S" and "_T", the set of item definitions "_D" and the initialiser term "tau" for the entire translation unit.

Once all external declarations in a given source file have been processed, any tentative definitions “"Nothing"” remaining in "_D" for Etude variables that, in the ultimate scope "_S" current at the end of the translation unit, are associated with a designator of a static linkage form, are converted into specification of an implicitly-initialised C object, i.e., into an Etude item definition of the form “"OBJI (delta_set_k) {-OF-} (xi_set_k)"”, in which "delta_set_k" and "xi_set_k" represent the default data specification and envelope derived from the variable's C type, as described earlier in the respective Sections \ref{C/Declarations/InitDeclarators} and \ref{C/Types/Storage}.

Once this adjustment has been performed for all tentative definitions in the translation unit, the collective denotation of its external declarations can be used to derive a complete Etude module of the form “"MODULE (tau) {-EXPORT-} (xi_set) {-WHERE-} (iota_set)"\,\,”, in which the module initialiser "tau" represents the composition of the individual declaration initialisers derived from all the external declarations in the unit.

Intuitively, the module's export set "xi_set" includes the bindings of all variables introduced into the translation unit with an external linkage form, other than those which are simply imported from some other source file of the entire program. In particular, for every identifier-like Etude variable "x" that, in the ultimate scope "_S" current at the end of translation, is assigned a designator with an external linkage and, in the associated set of item definitions "_D", is associated with any Etude item form other than “"IMPI x"”, the resulting set "xi_set" will contain a binding of "x" to its own name.

Further, the item set "iota_set" of the resulting module consists of all the static definitions introduced by the translation unit, including those bound to the imported item forms “"IMPI x"”, but excluding any tentative definitions for objects with a non-static linkage. Keeping in mind that, throughout this chapter, all such tentative definitions were distinguished in "_D" by bindings to the special symbol “"Nothing"”, "iota_set" consists simply of all the bindings "(nu_k {-:-}, iota_k) @@ _D" in which "iota_k /= Nothing". 

\break

\noindent In Haskell, the entire algorithm can be depicted quite naturally as follows:

\BeginHaskellToken	{meaningOfTranslationUnit}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{tu}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{TranslationUnitT}
  \HaskellResult							{_M{--}(ModuleT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfTranslationUnit}			{D_TU translation_unit}

\begin{code}
meaningOfTranslationUnit {-[*]-} :: (MonadFix _M, Monad _M) => TranslationUnitT -> _M{--}(ModuleT VarT)

meaningOfTranslationUnit {-sem-}(tu) = do
	(_S, _T, _D, _V, tau) <- meaningOfExternalDeclarations{--}(emptyMap, emptyMap, emptyMap {->-}, tu)
	({-hfil-}_D'       ) <- {-hide-}return $ _D // listToMap [ {-sem-}({-sem-}(variableOfLinkage{--}(linkageOfDesignator{--}(g_k))) --
								=: Just (OBJI ({-sem-}(defaultDataOfType{--}(typeOfDesignator{--}(g_k)))) --
										{-OF-} ({-sem-}(envelopeOfType{--}(typeOfDesignator{--}(g_k))))))
	{--}							| {-raw-}(x_k {-:-}, g_k) <- mapToList _S, --
								  isStaticLinkage{--}(linkageOfDesignator{--}(g_k)) && --
								  _D#(variableOfLinkage{--}(linkageOfDesignator{--}(g_k))) == Nothing ]
	({-hfil-}xi_set    ) <- {-hide-}return $ listToMap [ {-sem-}(x_k 	=: EVar x_k		) ---
								| {-raw-}(x_k 		{-:-}, g_k		) <- mapToList _S, ---
								  linkageOfDesignator{--}(g_k) --
								  	== {-sem-}(ExternLinkage {-hide-}(variableOfLinkage{--}(linkageOfDesignator{--}(g_k)))) && --
								  _D'#(EVar x_k) /= Just {-sem-}(IMPI x_k) ]
	({-hfil-}iota_set  ) <- {-hide-}return $ listToMap [ {-sem-}(nu_k 	=: unJust iota_k	) ---
								| {-raw-}(nu_k		{-:-}, iota_k		) <- mapToList _D, ---
								  iota_k /= Nothing ]
	require $ (_V == emptyMap && and [ _D'#(variableOfLinkage{--}(linkageOfDesignator{--}(g_k))) /= {-sem-}(Just (IMPI x_k)) --
						| (x_k {-:-}, g_k) <- mapToList _S, --
						  linkageOfDesignator{--}(g_k) --
						  	== {-sem-}(InternLinkage {-hide-}(variableOfLinkage{--}(linkageOfDesignator{--}(g_k)))) ]) {-!-}{-!-}{-!-}{-!-}{-!-}{-!-}
	return $ {-sem-}(MODULE	(tau) {-EXPORT-} (xi_set) {-WHERE-} (iota_set {-,,-}))
\end{code}

The reader should observe that the set of local variable definitions "_V" derived from the external C declarations must always remain empty and that, by the end of every source file, all internally-linked entities must include a proper object definition somewhere within their surrounding translation unit, so that no C variable "x" with an internal linkage in "_S" should be bound in "_D" to the imported item form “"IMPI x"”.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Definitions/External}
\set{title}		{External Declarations}
\begin{subsection}

In a C program, every «external declaration» constitutes either an ordinary declaration construct, whose syntax and semantics were the subject of Section \ref{C/Declarations}, or else a new kind of entity known as a «function definition», whose meaning will be described shortly in Section \ref{C/Definitions/Functions}. In Haskell, this dual syntax of external C declarations can be captured by the following BNF grammar:

\IndexType		{ExternalDeclarationT}				{external_declaration (\tstar)}
\IndexCons		{FunctionDefinition}				{ function_definition (external_declaration)}
\IndexCons		{ExternalDeclaration}				{ declaration (external_declaration)}

\begin{syntax}
\InsertHaskell		{ExternalDeclarationT}
\end{syntax}

Formally, the translation of a C source file into Etude is performed by threading the supplied current scope and item definition set through all of its external declarations, in the order of their appearance within the program, also composing the individual initialiser terms "tau_k" derived from each declaration into a monadic sequence of the form \hbox{“"LET [{-hv-}[] =: tau1] LET [{-hv-}[] =: tau2] .. LET [{-hv-}[] =: tau_n]"”.} In Haskell:

\BeginHaskellToken	{meaningOfExternalDeclarations}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{edl}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D {->-}, TranslationUnitT)}
  \HaskellResult							{_M{--}(S, S, D, V, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfExternalDeclarations}			{D_EDL translation_unit}

\begin{code}
meaningOfExternalDeclarations {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D {->-}, TranslationUnitT) -> _M{--}(S, S, D, V, TermT VarT)

meaningOfExternalDeclarations {-sem-}(_S, _T, _D {->-}, edl `Un` ed {-,-}) = do
	(_S1, _T1, _D1, _V1, tau1) <- meaningOfExternalDeclarations{--}(_S,   _T,   _D   {->-}, edl)
	(_S2, _T2, _D2, _V2, tau2) <- meaningOfExternalDeclarations{--}(_S1,  _T1,  _D1  {->-}, U0 ed)
	return $ (_S2, _T2, _D2, _V1 `mapUnion` _V2, {-sem-}(LET [{-hv-}[] =: tau1] (tau2)))
  		
meaningOfExternalDeclarations {-sem-}(_S, _T, _D {->-}, U0 (FunctionDefinition function_definition)) ---
  = meaningOfFunctionDefinition{--}(_S, _T, _D {->-}, function_definition)
meaningOfExternalDeclarations {-sem-}(_S, _T, _D {->-}, U0 (ExternalDeclaration declaration)) ---
  = meaningOfDeclarationListOpt{--}(_S, _T, _D, 0 {->-}, Just (D0 declaration))
\end{code}

recalling that all three of the finite maps "_S", "_T" and "_D" supplied in the context of an external declaration will be empty at the beginning of every translation unit. The reader should also observe that, in every standard and well-formed C program, all external declarations will, by construction, always produce the trivial initialiser term “"RET []"”, so that the resulting initialiser for an entire Etude module produced the program's translation will always remain semantically indistinguishable from a single “"RET []"” term. During a subsequent linking stage of compilation, this term will be replaced with a call to the program's designated “\,\CKeyword{main}\!” function, although all of the relevant details of that replacement remain beyond the scope of the present work. Accordingly, the declaration initialisers are retained under the above translation only in the interest of its forward compatibility with any future extensions to the C Standard.

\end{subsection}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Definitions/Functions}
\set{title}		{Function Definitions}
\begin{subsection}

A new function is always introduced into a C program by an entity known as a «function definition», whose concrete syntax is defined by the following Haskell data type:

\IndexType		{FunctionDefinitionT}				{function_definition (\tstar)}
\IndexCons		{Function}					{ declaration_specifiers_opt declarator declaration_list_opt compound_statement (function_definition)}

\begin{syntax}
\InsertHaskell		{FunctionDefinitionT}
\end{syntax}

In other words, every function definition consists of a C declaration with a single declarator and no initialisers, together with a compound statement known as the «function body». The list of declaration specifiers may be omitted from the syntax if no particular specifiers are applicable to the function being defined. The compound statement may be also preceded by an optional list of declarations, intended to refine the meanings of any function arguments in an absence of an explicit prototype in the \hbox{associated declarator.}

The derivation of a function definitions' formal meaning bootstraps the entire statement translation process presented earlier in Section \ref{C/Statements}. Accordingly, the construction "meaningOfFunctionDefinition{-sem-}(_S, _T, _D {->-}, Function (ds_opt) (r) (dl_opt) (s))" has a somewhat esoteric formulation, which is best described informally by the following algorithm:

\begin{list}

•	First of all, the construct's list of declaration specifiers is used to derive the storage class "sc_set", qualification "tq_set" and base type "t_b" for the function's designator, using the standard algorithms "meaningOfDeclarationSpecifiersOpt" and "meaningOfTypeSpecifiers" defined earlier in Sections \ref{C/Declarations/Specifiers} and \ref{C/Declarations/TypeSpecifiers}.

•	Next, a meaning of the following declarator "r" is established using the algorithm "meaningOfFunctionDeclarator" described later in this section. Besides the usual C type "t" and identifier "x" found in the denotations of ordinary C declarators, this algorithm also derives the set of argument names "x_set2" from the construct's identifier list whenever such a list appears in the syntax, as well as an additional pair of scopes "_S'" and "_T'" current at the end of any prevailing parameter type list in "r".

•	Further, the new function is introduced into its declarator's scope "_S2" with a binding of the identifier "x" to a designator "d", derived from the storage class "sc_set" and the C type of the specified declarator "r" using the algorithm "meaningOfVariableDefinition" defined earlier in Section \ref{C/Declarations/InitDeclarators}. In the associated set of item definitions "_D2", the variable "x" is also bound to an Etude function, whose parameter list and term represent, respectively, the set of all local variables "_V3'" visible to the compound statement "s" and the actual denotation "tau" of that statement, as derived is steps 5 and 6 of the present algorithm.

•	Now, the list of parameter declarations "dl_opt" is processed using the algorithm "meaningOfParameterDeclarationListOpt", in the context of the parameter scopes "_S'" and "_T'" derived from "r". The resulting denotation includes a new set of identifiers "x_set3", formed precisely from those parameters to the new function whose C types are declared by "dl_opt", together with the set of all local variables "_V3" that have been associated with these parameters.

•	Once "dl_opt" has been analysed, all identifiers from "x_set2" that are still left without a local declaration are introduced into the program as automatically-linked entities of the plain “"PlainIntType"” type. Further, the initial set of all local variables "_V3'" visible upon entry into the newly-created C function is defined as a union of all local variables "_V2" declared in the parameter type list of the declarator "r", together with the set "_V3" of those that are declared by "dl_opt" and, finally, any variables introduced by the implicit “"PlainIntType"” bindings of the otherwise undeclared parameters from "x_set2".

•	Having completed these preparatory tasks, we are now ready to scrutinise the function's body statement "s", which is processed directly using the algorithm "meaningOfStatementListOpt" defined earlier in Section \ref{C/Statements}. The statement's function scope "_L" is determined by the set of all label identifiers derived from "s" itself. Similarly, the collection of all basic block definitions "_B" available throughout the construct's translation is set to contain all of the basic blocks "_B'" introduced by the function's body, together with an additional «exit block» "uvar{--}(_D4)" that is appended to both "_B" and "_D4" with a trivial definition of “"FUNI (FUN (setToList {-sem-}(dom{--}(_V3'))) {-DOT-} (RET {-hv-}[]))"”. Intuitively, this block represents the exit point from the entire function and, during translation of "s", it is made available to the statement as a binding of its return label "nu_r". A jump to that label also serves as the statement's continuation term "tau_kappa". Finally, the function's returned type "t_r" is set to the base type of the designator "r", while the continuation and break labels "nu_c" and "nu_b" and the selection type "t_s" are left with their default values of “"Nothing"”.

•	In every well-formed function definition, the resulting designator "d" must have a static linkage, with a C type of a function returning either an object type other than an array, or else a qualified or unqualified version of the “"VoidType"\,” type. Further, at most one of the parameter type lists and parameter declaration components can be ever supplied in a single function definition, so that, if the declarator "r" specifies a non-empty function prototype for the new entity, then the associated declaration list "dl_opt" must always remain empty. The set of all label names "_J" that are actually referenced by "s" must also represent a subset of its function scope "_L", in order to ascertain that all “"GotoStatement"” statements in the program refer to valid statement labels. Last but not least, the set of case labels "_C" derived from the function's body must be empty by the end of the entire construct, so that no “"CaseStatement"” label may ever appear outside of a well-formed “"SwitchStatement"” statement.

•	Finally, in the denotation of the entire function definition, the local scope of "s" is discarded and the returned Etude term is given the trivial form “"RET []"”, in order to facilitate its incorporation into a larger list of external declarations as described earlier in Section \ref{C/Definitions/External}.

\end{list}

In Haskell, these translation semantics of C functions can be formalised as follows:

\BeginHaskellToken	{meaningOfFunctionDefinition}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{fd}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D {->-}, FunctionDefinitionT)}
  \HaskellResult							{_M{--}(S, S, D, V, TermT VarT)}
\EndHaskellToken

\IndexTerm		{meaningOfFunctionDefinition}			{D_FD function_definition}

\begin{code}
meaningOfFunctionDefinition {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D {->-}, FunctionDefinitionT) -> _M{--}(S, S, D, V, TermT VarT)

meaningOfFunctionDefinition {-sem-}(_S, _T, _D {->-}, Function (ds_opt) (r) (dl_opt) (s)) = mdo
	(sc_set, tq_set, ts_set) <- meaningOfDeclarationSpecifiersOpt{--}(ds_opt)
	(_S1, _T1, _D1, t_b) <- meaningOfTypeSpecifiers{--}(_S, _T, _D, 0 {->-}, setToList ts_set)
	(_S2, _S', _T2, _T', _D2, _V2, t, x, x_set2) <- meaningOfFunctionDeclarator{--}(_S1, _T1, _D1, setToList tq_set `qualifiedType` t_b {->-}, r)
	({-hfil-}d       	)  <- meaningOfVariableDefinition{--}(_S2, _T2, _D2, 0, setToList sc_set, t {->-}, x)
	({-hfil-}_S2'		)  <- {-hide-}return $ _S2 // listToMap [ {-raw-}(x {-:-}, d) ]
	({-hfil-}_D2'		)  <- {-hide-}return $ _D2 // listToMap [ {-raw-}(EVar x {-:-}, Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V3'))) {-DOT-} (tau)))) ]
	(_S3, _T3, _D3, _V3, x_set3) <- meaningOfParameterDeclarationListOpt{--}(_S2' // _S', _T2 // _T', _D2' {->-}, dl_opt)
	({-hfil-}_S3'		)  <- {-hide-}return $ _S3 // listToMap [ {-raw-}(x_k {-:-}, {-sem-}(Designator (AutoLinkage (nu_k)) (PlainIntType) {-@-} (1))) --
								| (x_k {-:-}, nu_k) <- setToList ((x_set2 `difference` x_set3)) `zipList` [ uvar{--}(_D3) {--} .. {-,,-} ] ]
	({-hfil-}_D3'		)  <- {-hide-}return $ _D3 `mapUnion` listToMap [ {-raw-}(variableOfLinkage{--}(linkageOfDesignator{--}(_S3'#(x_k))) {-:-}, Nothing) --
					  			| x_k <- setToList (x_set2 `difference` x_set3) ]
	({-hfil-}_V3'   	)  <- {-hide-}return $ _V2 `mapUnion` _V3 --
					  `mapUnion` listToMap [ {-raw-}(variableOfLinkage{--}(linkageOfDesignator{--}(_S3'#(x_k))) {-:-}, --
					  					envelopeOfType{--}(typeOfDesignator{--}(_S3'#(x_k)))) --
					  			| x_k <- setToList (x_set2 `difference` x_set3) ]
	(_S4, _T4, _D4, _B, _L, _C, _J, tau)
				   <- meaningOfStatementListOpt{--}( --
					_S3', --
					_T3, --
					_D3', --
					_B', --
					_L, --
					_V3', --
					1, --
					baseOfType{--}(typeOfDesignator{--}(d)), --
					Nothing, --
					uvar{--}(_D4), --
					Nothing, --
					Nothing, --
					meaningOfJump{--}(_B', _V3' {->-}, uvar{--}(_D4)) {->-}, --
					Just (S0 (S2 s)))
	({-hfil-}_D4'  		)  <- {-hide-}return $ _D4 // listToMap [ {-raw-}(uvar{--}(_D4) {-:-}, --
											Just {-sem-}(FUNI (FUN (setToList {-sem-}(dom{--}(_V3'))) {-DOT-} (RET {-hv-}[])))) ]
	({-hfil-}_B'  		)  <- {-hide-}return $ _B `mapUnion` listToMap [ {-raw-}(uvar{--}(_D4) {-:-}, _V3') ]
	require $	(isStaticLinkage{--}(linkageOfDesignator{--}(d))) &&
			(isFunctionType{--}(typeOfDesignator{--}(d)) && --
			 ((isObjectType{--}(baseOfType{--}(typeOfDesignator{--}(d))) && not (isArrayType{--}(baseOfType{--}(typeOfDesignator{--}(d))))) || --
			  isVoidType{--}(baseOfType{--}(typeOfDesignator{--}(d))))) &&
			(x_set3 `subseteq` x_set2) &&
			(_J `subseteq` dom{--}(_L)) && --
			(_C == emptyMap)
	return $ (_S2', --
  		  _T2, --
  		  _D4', --
  		  emptyMap, --
  		  {-sem-}(RET {-hv-}[]))
\end{code}

In every such construct, the function's type must be specified directly in "r", rather than through a type name in the associated list of declaration specifiers. In order to capture this requirement formally, it is convenient to introduce a notion of «simple declarators», whose concrete syntax must be formed either from a single identifier, or else from a parenthesised version of such a declarator. In Haskell, these constructs are characterised by the following derivation, and their denotations consist solely of the specified identifier itself:

\BeginHaskellToken	{meaningOfSimpleDeclarator}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{sr}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{DeclaratorT}
  \HaskellResult							{_M{--}(IdentifierT)}
\EndHaskellToken

\IndexTerm		{meaningOfSimpleDeclarator}			{D_SR declarator}

\begin{code}
meaningOfSimpleDeclarator {-[*]-} :: (MonadFix _M, Monad _M) => (DeclaratorT) -> _M{--}(IdentifierT)

meaningOfSimpleDeclarator {-sem-}(Declarator ({-hide-}Nothing) (NestedDeclarator (r))) ---
  =	meaningOfSimpleDeclarator{--}(r)
meaningOfSimpleDeclarator {-sem-}(Declarator ({-hide-}Nothing) (IdentifierDeclarator (x))) ---
  =	return $	(x)
meaningOfSimpleDeclarator {-sem-}(other) --
  =	fail		("Semantic error")
\end{code}

With the help of this auxiliary definition, the meaning of all declarators appearing within a function definition can be described by the following algorithm "meaningOfFunctionDeclarator", akin to its earlier variant "meaningOfDeclarator" that, in Section \ref{C/Declarations/Declarators} was used to scrutinise the semantics of ordinary C declarators:

\BeginHaskellToken	{meaningOfFunctionDeclarator}		{Open}	{\ZapfChancery{D}\sub{\SmallCaps{fr}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I, TypeT {->-}, DeclaratorT)}
  \HaskellResult							{_M{--}(S, S, S, S, D, V, TypeT, IdentifierT, SetT IdentifierT)}
\EndHaskellToken

\IndexTerm		{meaningOfFunctionDeclarator}			{D_FR declarator}

\begin{code}
meaningOfFunctionDeclarator {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, TypeT {->-}, DeclaratorT) -> _M{--}(S, S, S, S, D, V, TypeT, IdentifierT, SetT IdentifierT)
\end{code}

In particular, like "meaningOfDeclarator", "meaningOfFunctionDeclarator" proclaims every nested declarator “\!"NestedDeclarator (r)"\!” to represent a simple semantic equivalent of "r" itself and all declarators of the form “"Declarator (pointer) (r)"” to assume the meaning of "r" under the new pointer base type "meaningOfPointer{--}(t_b {->-}, pointer)", in which "t_b" depicts the prevailing base type of the entire construction. On the other hand, declarators consisting entirely of a single identifier are never permitted in the present context:

\begin{code}
meaningOfFunctionDeclarator {-sem-}(_S, _T, _D, t_b {->-}, Declarator ({-hide-}Nothing) (NestedDeclarator (r))) ---
  =	meaningOfFunctionDeclarator{--}(_S, _T, _D, t_b {->-}, r)
meaningOfFunctionDeclarator {-sem-}(_S, _T, _D, t_b {->-}, Declarator (Just pointer) (r {-,-})) ---
  =	meaningOfFunctionDeclarator{--}(_S, _T, _D, meaningOfPointer{--}(t_b {->-}, pointer) {->-}, Declarator ({-hide-}Nothing) (r))
meaningOfFunctionDeclarator {-sem-}(_S, _T, _D, t_b {->-}, Declarator ({-hide-}Nothing) (IdentifierDeclarator (x))) ---
  =	fail ("Semantic error")
\end{code}

Further, array declarators of the form “"r `ArrayDeclarator` (e_opt)"\!” denote the array type “"t_b `ArrayType` (n_opt)"”, in which the element type "t_b" is equal to the construct's base type, identically to the earlier ordinary applications of this syntax:

\begin{code}
meaningOfFunctionDeclarator {-sem-}(_S, _T, _D, t_b {->-}, Declarator ({-hide-}Nothing) (r `ArrayDeclarator` (e_opt))) = mdo
	(_S_r, _S', _T_r, _T', _D_r, _V, t, x, x_set) <- meaningOfFunctionDeclarator{--}(_S, _T, _D, {-sem-}({-,-}t_b `ArrayType` (n_opt)) {->-}, Declarator ({-hide-}Nothing) (r))
	(_S_e, _T_e, _D_e, n_opt) <- meaningOfArrayLengthOpt{--}(_S_r, _T_r, _D_r, 0 {->-}, e_opt)
	return $ (_S_e, _S', _T_e, _T', _D_e, _V, t, x, x_set)
\end{code}

The peculiar nature of function declarators only becomes apparent in constructs of the form “"r `PrototypeDeclarator` (ptl)"\!” and “"r `FunctionDeclarator` (xl_opt)"\!”.

In particular, if the supplied declarator "r" that accompanies a non-empty parameter type list "ptl" consists entirely of an optionally-parenthesised identifier "x" (in other words, if it depicts a simple declarator), then the construction itself determines the actual type of the C function being defined. Accordingly, its denotation consists of the declarator's identifier "x", the function type “"t_b `FunctionType` (Just p)"\!” whose prototype "p" is derived from the parameter type list "ptl" and a local variable set formed from all of the Etude variables introduced by "ptl", with an addition of a designated Etude object intended for storage of the function's return value. In "_V", this object is bound to the unique local variable "uvar{--}(_D)" with an envelope derived from the function's returned type "t_b", using the construction \hbox{"returnedEnvelope{--}(uvar{--}(_D), t_b)"} described later in the present section. The identifier set "x_set" returned by such entities is left empty, while the prototype scopes "_S'" and "_T'" consist precisely of those variables and type tags that have been introduced into the program locally by the parameter type list "ptl". In all other cases, declarators of this form are processed identically to their ordinary usage from Section \ref{C/Declarations/Declarators}, deriving the meanings of their direct declarators "r" under the adjusted base type “"t_b `FunctionType` (p_opt)"”. Formally:

\begin{code}
meaningOfFunctionDeclarator {-sem-}(_S, _T, _D, t_b {->-}, Declarator ({-hide-}Nothing) (r `PrototypeDeclarator` (ptl)))

	=	do	(x) <- meaningOfSimpleDeclarator{--}(Declarator ({-hide-}Nothing) (r))
			(_S', _T', _D', _V, p) <- meaningOfFunctionParameterTypeList {--}(_S, _T, _D `mapUnion` listToMap [ {-raw-}(uvar{--}(_D) {-:-}, Nothing) ] {->-}, ptl)
			return $ (	_S,  listToMap [ {-raw-}(x_k {-:-}, _S'  #(x_k)) | x_k <- setToList (dom{--}({-hfil-}_S'  )), indexOfDesignator{--}(_S'  #(x_k)) == 1 ],
				  	_T,  listToMap [ {-raw-}(x_k {-:-}, _T'  #(x_k)) | x_k <- setToList (dom{--}({-hfil-}_T'  )), indexOfDesignator{--}(_T'  #(x_k)) == 1 ],
					_D', _V `mapUnion` returnedEnvelope{--}(uvar{--}(_D), t_b), {-sem-}({-,-} t_b `FunctionType` (Just p)), x, emptySet)

	<|>	mdo	(_S_r, _S', _T_r, _T', _D_r, _V, t, x, x_set) --
				<- meaningOfFunctionDeclarator{--}(_S, _T, _D, {-sem-}({-,-} t_b `FunctionType` (p_opt)) {->-}, Declarator ({-hide-}Nothing) (r))
			(_D_p, p_opt) <- meaningOfParameterTypeListOpt{--}(_S_r, _T_r, _D_r, 0 {->-}, Just ptl)
			return $ (_S_r, _S', _T_r, _T', _D_p, _V, t, x, x_set)
\end{code}

Similarly, a function declarator of the form “"r `FunctionDeclarator` (xl_opt)"\!”, in which "xl_opt" represents a list of zero or more identifiers and "r" constitutes a simple declarator, denotes the list of identifiers "x_set" derived from the syntax of "xl_opt", the declarator's name "x", the type of a function “"t_b `FunctionType` ()"\!” without any prototype information and a singular set of local variables formed entirely from the result object "returnedEnvelope{--}(uvar{--}(_D), t_b)", that is bound to a globally-unique Etude variable "uvar{--}(_D)". The prototype scopes "_S'" and "_T'" are always left empty by such declarators. In all such constructs, the identifier list "xl_opt" must represent a proper set, so that any particular identifier may appear at most once in a given list of parameter names and, further, none of these identifiers may redeclare an existing typedef name that, in the current scope "_S", is bound to a designator with a type linkage. Otherwise, such entities are treated in a fashion analogous to that prescribed for their ordinary usage in Section \ref{C/Declarations/Declarators}, so that the identifier list "xl_opt" must be left empty whenever "r" does not represent a simple declarator. In Haskell:

\begin{code}
meaningOfFunctionDeclarator {-sem-}(_S, _T, _D, t_b {->-}, Declarator ({-hide-}Nothing) (r `FunctionDeclarator` (xl_opt)))

	=	do	(x) <- meaningOfSimpleDeclarator{--}(Declarator ({-hide-}Nothing) (r))
			require $ (lengthOfList{--}(listOfIdentifiersOpt{--}(xl_opt)) == cardinality (listToSet (listOfIdentifiersOpt{--}(xl_opt)))) &&
			{-~-}{-~-}{-~-}and [ x_k /@ dom{--}(_S) || (indexOfDesignator{--}(_S#(x_k)) /= 1 && linkageOfDesignator{--}(_S#(x_k)) /= {-sem-}(TypeLinkage)) --
					  	| x_k <- listOfIdentifiersOpt{--}(xl_opt) ] {-!-} {-!-} {-!-} {-!-} {-!-}
			return $ (_S, emptyMap, --
				  _T, emptyMap, --
				  _D `mapUnion` listToMap [ {-raw-}(uvar{--}(_D) {-:-}, Nothing) ], --
				  returnedEnvelope{--}(uvar{--}(_D), t_b), --
				  {-sem-}({-,-} t_b `FunctionType` ({-hide-}Nothing)), --
				  x, --
				  listToSet (listOfIdentifiersOpt{--}(xl_opt)))

	<|>	do	(_S_r, _S', _T_r, _T', _D_r, _V, t, x, x_set) --
				<- meaningOfFunctionDeclarator{--}(_S, _T, _D, {-sem-}({-,-} t_b `FunctionType` ({-hide-}Nothing)) {->-}, Declarator ({-hide-}Nothing) (r))
			require $ (listOfIdentifiersOpt{--}(xl_opt) == {-empty-}[])
			return $ (_S_r, _S', _T_r, _T', _D_r, _V, t, x, x_set)
\end{code}

The restriction on redeclaration of typedef names within identifier lists has been imposed in the C Standard in order to resolve subtle syntactic ambiguity in the published BNF grammar of the language. In the present work, this constraint serves no other useful purpose and it is included in the above translation only for the sake of completeness.

In both cases, the collection of additional objects that are introduced implicitly into the resulting set of local variables for storage of the function's return value is determined by its returned type "t", as represented by the notation “"returnedEnvelope{--}(nu, t)"”. In most cases, "returnedEnvelope" produces a singleton set, consisting of a single variable binding that associates "nu" with the Etude envelope derived from the C type "t", provided that the function returns an object type other than an array. However, if "t" represents a qualified or unqualified version of the “"VoidType"\,” type, then no returned variable is introduced into the local scope, so that "returnedEnvelope{-sem-}(nu, VoidType {-,-})" always produces the empty set "emptySet":

\BeginHaskellToken	{returnedEnvelope}			{Open}	{«\superimpose{\raise.5ex\hbox{\macron{}}}{\xi}»\sub{\SmallCaps{r}}}
  \HaskellArgument							{(VarT, TypeT)}
  \HaskellResult							{V}
\EndHaskellToken

\IndexTerm		{returnedEnvelope}				{\xi-R var type (V)}

\begin{code}
returnedEnvelope {-[*]-} :: (VarT, TypeT) -> V

returnedEnvelope {-sem-}(nu, t) ---
  | isObjectType{--}(t) && not (isArrayType{--}(t))	= listToMap [ {-raw-}(nu {-:-}, envelopeOfType{--}(t)) ]
  | isVoidType{--}(t)					= emptyMap
\end{code}

Every list of parameter types appearing within a function declarator is processed similarly to the ordinary usage of such constructs described earlier in Section \ref{C/Declarations/Declarators}, except that, in the present context, these entities also derive the set of all Etude variables associated with the individual parameters of the new function, in addition to the actual prototype information. Formally, this new interpretation of such lists is characterised by the following Haskell function:

\BeginHaskellToken	{meaningOfFunctionParameterTypeList}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{fptl}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, I {->-}, ParameterTypeListT)}
  \HaskellResult							{_M{--}(S, S, D, V, PrototypeT)}
\EndHaskellToken

\IndexTerm		{meaningOfFunctionParameterTypeList}		{D_FPTL parameter_type_list}

\begin{code}
meaningOfFunctionParameterTypeList {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D {->-}, ParameterTypeListT) -> _M{--}(S, S, D, V, PrototypeT)
\end{code}

If such a list consists entirely of a single storage class specifier “"VoidSpecifier"\,”, then the resulting set of local variables "_V" is left empty. Otherwise, the meaning of such entities is determined from their parameter lists "pl", using the algorithm "meaningOfFunctionParameterList" described shortly:

\begin{code}
meaningOfFunctionParameterTypeList {-sem-}(_S, _T, _D {->-}, FixedParameters (P0 (AbstractParameter (DeclarationSpecifier (VoidSpecifier {-,-}) ({-hide-}Nothing)) ({-hide-}Nothing)))) --
  = return $ (_S, _T, _D, emptyMap, {-sem-}(Types {-raw-}[VoidType{-,-}]))

meaningOfFunctionParameterTypeList {-sem-}(_S, _T, _D {->-}, FixedParameters (pl) {-,-}) = do
	(_S', _T', _D', _V, t_set {-,-}) <- meaningOfFunctionParameterList{--}(_S, _T, _D {->-}, pl {-,-})
	return $ (_S', _T', _D', _V, {-sem-}(Types t_set) {-,,-})

meaningOfFunctionParameterTypeList {-sem-}(_S, _T, _D {->-}, VariableParameters (pl) {-,-} {-...-}) = do
	(_S', _T', _D', _V, t_set {-,-}) <- meaningOfFunctionParameterList{--}(_S, _T, _D {->-}, pl {-,-})
	return $ (_S', _T', _D', _V, {-sem-}(Ellipsis (t_set) {-...-}))
\end{code}

In particular, every parameter declaration found in a function definition must specify a distinct identifier "x" that is unique within its scope, so that abstract declarators are not permitted in such constructs, except for the special “"VoidSpecifier"\,” case mentioned earlier. Further, the declaration's syntax must not include any storage class specifiers other than “"RegisterSpecifier"” and the semantics of its declarator must assign to "x" a C type "t" that is complete after pointer promotion. The construct extends its current scope "_S" with a binding of "x" to a designator with the unadjusted type "t" and a linkage derived from the supplied storage class "sc_set" using the usual algorithm "meaningOfStorageClassSpecifiers" defined in Section \ref{C/Declarations/StorageClassSpecifiers}. The formal denotation of such parameters always includes a set of local variables "_V" that binds individual parameter variables to Etude envelopes of their pointer-promoted types. In Haskell:

\BeginHaskellToken	{meaningOfFunctionParameterList}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{fpl}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D {->-}, ParameterListT)}
  \HaskellResult							{_M{--}(S, S, D, V, TypesT)}
\EndHaskellToken

\IndexTerm		{meaningOfFunctionParameterList}		{D_FPL parameter_list}

\begin{code}
meaningOfFunctionParameterList {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D {->-}, ParameterListT) -> _M{--}(S, S, D, V, TypesT)

meaningOfFunctionParameterList {-sem-}(_S, _T, _D {->-}, P0 (NamedParameter (ds) (r))) = do
	(sc_set, tq_set, ts_set) <- meaningOfDeclarationSpecifiersOpt{--}(Just ds)
	(_S1, _T1, _D1, t_b) <-	meaningOfTypeSpecifiers{--}(_S, _T, _D, 1 {->-}, setToList ts_set)
	(_S2, _T2, _D2, t, x) <- meaningOfDeclarator{--}(_S1, _T1, _D1, 1, setToList tq_set `qualifiedType` t_b {->-}, r)
	(ell) <- meaningOfStorageClassSpecifiers{--}(_S2, _T2, _D2, 1, pointerPromotedType{--}(t), x {->-}, setToList sc_set)
	require $	(sc_set `subseteq` listToSet [RegisterSpecifier] && isObjectType{--}(pointerPromotedType{--}(t)) && --
			(x /@ dom{--}(_S2) || indexOfDesignator{--}(_S2#(x)) /= 1))
	return $ (_S2 // listToMap [ {-raw-}(x {-:-}, {-sem-}(Designator (ell) (t) {-@-} (1))) ], --
		  _T2, --
		  _D2 // listToMap [ {-raw-}(variableOfLinkage{--}(ell) {-:-}, Nothing) ], --
		  listToMap [ {-raw-}(variableOfLinkage{--}(ell) {-:-}, envelopeOfType{--}(pointerPromotedType{--}(t))) ], --
		  [t])

meaningOfFunctionParameterList {-sem-}(_S, _T, _D {->-}, pl `Pn` pd {-,-}) = do
	(_S1, _T1, _D1, _V1, t_set1) <- meaningOfFunctionParameterList{--}(_S,   _T,   _D   {->-}, pl {-,-})
	(_S2, _T2, _D2, _V2, t_set2) <- meaningOfFunctionParameterList{--}(_S1,  _T1,  _D1  {->-}, P0 pd {-,-})
	return $ (_S2, _T2, _D2, _V1 `mapUnion` _V2, t_set1 ++ t_set2)

meaningOfFunctionParameterList {-sem-}(_S, _T, _D {->-}, P0 (AbstractParameter (ds) (ar_opt))) = fail ("Semantic error")
\end{code}

On the other hand, if the function definition's declarator includes an identifier list instead of parameter types, then the names of all function arguments are determined by those identifiers and their C types are described by a list of zero or more ordinary C declarations that follow the declarator in the concrete syntax of the entire definition. Conceptually, all of these declarations are injected into the program at the beginning of the scope associated with the function's compound statement, in a manner similar to the earlier ordinary applications of the declaration syntax scrutinised in Section \ref{C/Declarations}, except that, in the present context, no declaration may include any storage class specifiers other than “"RegisterSpecifier"” and that every element of the list must specify at least one initialised declarator, so that tag declarations are explicitly disallowed within such lists:

\BeginHaskellToken	{meaningOfParameterDeclarationListOpt}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{pdl}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D {->-}, Maybe DeclarationListT)}
  \HaskellResult							{_M{--}(S, S, D, V, SetT IdentifierT)}
\EndHaskellToken

\IndexTerm		{meaningOfParameterDeclarationListOpt}		{D_PDL declaration_list_opt}

\begin{code}
meaningOfParameterDeclarationListOpt {-[*]-} :: (MonadFix _M, Monad _M) => (S, S, D {->-}, Maybe DeclarationListT) -> _M{--}(S, S, D, V, SetT IdentifierT)

meaningOfParameterDeclarationListOpt {-sem-}(_S, _T, _D {->-}, Just (D0 (Declaration (ds) (Just irl) {-;-}))) = do
	(sc_set, tq_set, ts_set) <- meaningOfDeclarationSpecifiersOpt{--}(Just ds)
	require $ (sc_set `subseteq` listToSet [RegisterSpecifier])
	(_S1, _T1, _D1, t_b) <- meaningOfTypeSpecifiers{--}(_S,	_T, _D, 1 {->-}, setToList ts_set)
	(_S2, _T2, _D2, _V, x_set) <- meaningOfParameterDeclaratorList{--}(_S1, _T1, _D1, setToList sc_set, setToList tq_set `qualifiedType` t_b {->-}, irl)
	return $ (_S2, _T2, _D2, _V, x_set)

meaningOfParameterDeclarationListOpt {-sem-}(_S, _T, _D {->-}, Just (dl `Dn` d)) = do
	(_S1, _T1, _D1, _V1, x_set1) <- meaningOfParameterDeclarationListOpt{--}(_S,   _T,   _D   {->-}, Just dl)
	(_S2, _T2, _D2, _V2, x_set2) <- meaningOfParameterDeclarationListOpt{--}(_S1,  _T1,  _D1  {->-}, Just (D0 d))
	return $ (_S2, _T2, _D2, _V1 `mapUnion` _V2, x_set1 `union` x_set2)

meaningOfParameterDeclarationListOpt {-sem-}(_S, _T, _D {->-}, Nothing) ---
  = return $ (_S, _T, _D, emptyMap, emptySet)
meaningOfParameterDeclarationListOpt {-sem-}(_S, _T, _D {->-}, Just (D0 (Declaration (ds) ({-hide-}Nothing) {-;-}))) ---
  = fail ("Semantic error")
\end{code}

Each initialised declarator of such a construct has a meaning similar to that of an analogous parameter declaration with the same identifier and C type, except that this type is not contributed to the prototype of the newly-defined function. Explicit initialisers are always disallowed in the syntax of function definitions, so that the semantics of this final construct in the C grammar can be described by the following simple translation:

\BeginHaskellToken	{meaningOfParameterDeclaratorList}	{Open}	{\ZapfChancery{D}\sub{\SmallCaps{prl}}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellArgument							{(S, S, D, SetT IdentifierT, Maybe StorageClassSpecifierT, TypeT {->-}, InitDeclaratorListT)}
  \HaskellResult							{_M{--}(S, S, D, V, SetT IdentifierT)}
\EndHaskellToken

\IndexTerm		{meaningOfParameterDeclaratorList}		{D_PRL init_declarator_list}

\begin{code}
meaningOfParameterDeclaratorList {-[*]-} :: (MonadFix _M, Monad _M) =>
	(S, S, D, [StorageClassSpecifierT], TypeT {->-}, InitDeclaratorListT) ->
	_M{--}(S, S, D, V, SetT IdentifierT)

meaningOfParameterDeclaratorList {-sem-}(_S, _T, _D, sc_set, t_b {->-}, IR0 (DefaultDeclarator r)) = do
	(_S_r, _T_r, _D_r, t, x) <- meaningOfDeclarator{--}(_S, _T, _D, 1, t_b {->-}, r)
	(ell) <- meaningOfStorageClassSpecifiers{--}(_S_r, _T_r, _D_r, 1, pointerPromotedType{--}(t), x {->-}, sc_set)
	require $ (isObjectType{--}(pointerPromotedType{--}(t)) &&  (x /@ dom{--}(_S_r) || indexOfDesignator{--}(_S_r#(x)) /= 1))
	return $ (_S_r // listToMap [ {-raw-}(x {-:-}, {-sem-}(Designator (ell) (t) {-@-} (1))) ], --
		  _T_r, --
		  _D_r // listToMap [ {-raw-}(variableOfLinkage{--}(ell) {-:-}, Nothing) ], --
		  listToMap [ {-raw-}(variableOfLinkage{--}(ell) {-:-}, envelopeOfType{--}(pointerPromotedType{--}(t))) ], --
		  listToSet [x])

meaningOfParameterDeclaratorList {-sem-}(_S, _T, _D, sc_set, t_b {->-}, irl `IRn` ir) = do
	(_S1, _T1, _D1, _V1, x_set1) <- meaningOfParameterDeclaratorList{--}(_S,   _T,   _D,   sc_set, t_b {->-}, irl)
	(_S2, _T2, _D2, _V2, x_set2) <- meaningOfParameterDeclaratorList{--}(_S1,  _T1,  _D1,  sc_set, t_b {->-}, IR0 ir)
	return $ (_S2, _T2, _D2, _V1 `mapUnion` _V2, x_set1 `union` x_set2)

meaningOfParameterDeclaratorList {-sem-}(_S, _T, _D, sc_set, t_b {->-}, IR0 (r `InitDeclarator` i) {-,-})  = fail ("Semantic error")
\end{code}

With this final definition, we have now attained a complete formal description of the C programming language. Although arguably obtuse at times, our translation of C programs into Etude is no more difficult to absorb than any other semantic notation, while carrying the additional benefit of a fully-usable compiler implementation, whose correctness is guaranteed by its automatic derivation from the same source as that used to describe the behaviour of all C programs to the eventual users of the system.

\end{subsection}
\end{section}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%