\set{capture}{hidden}
\begin{code}

module C.Implementation (
	charType,
	intType,
	wideCharType,
	ptrDiffType,
	sizeType,
	formatOfType,
	typeOfEnumeratorList,
	layoutOfMembers,
	layoutOfVaList,
	packBitField,
	unpackBitField,
	packAtoms,
	setObject,
	iniObject,
	valueOfAtom,
) where

import Control.Monad.Fix
import Additional
import C.Syntax
import C.Variables
import C.Types
import C.Scopes
import C.Constants
import C.Expressions
import C.Declarations
import C.Statements
import C.Definitions
import Lambda

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Implementation}
\set{title}		{Implementation of C Compilers}
\begin{appendix}

In order to complete the translation semantics of C programs presented in Chapter \ref{C}, we must now provide a concrete Haskell implementation of every language parameter that was left unspecified in the earlier generic fragment of the language. On the following pages, I present one such implementation that is suitable for the MMIX architecture described in Chapter \ref{MMIX}. The design choices made below represent the typical interpretation of the C Standard common to virtually all compilers for the language that are deployed in the industry.

First and foremost, every C compiler must settle on a concrete mapping between C types and Etude formats. On MMIX, these are assigned as follows:

\BeginHaskellToken	{formatOfType}				{Open}	{«\phi»}
  \HaskellArgument							{TypeT}
  \HaskellResult							{FormatT}
\EndHaskellToken

\IndexTerm		{formatOfType}					{\phi type (format)}

\begin{code}
formatOfType {-[*]-} :: TypeT -> FormatT

formatOfType {-sem-}(Signed CharType)					= {-sem-}(ZG`FMT`8)
formatOfType {-sem-}(Signed ShortType)					= {-sem-}(ZG`FMT`16)
formatOfType {-sem-}(Signed IntType)					= {-sem-}(ZG`FMT`32)
formatOfType {-sem-}(Signed LongType)					= {-sem-}(ZG`FMT`64)
formatOfType {-sem-}(PlainCharType)					= {-sem-}(ZG`FMT`8)
formatOfType {-sem-}(PlainIntType)					= {-sem-}(ZG`FMT`32)
formatOfType {-sem-}(Unsigned CharType)					= {-sem-}(NG`FMT`8)
formatOfType {-sem-}(Unsigned ShortType)				= {-sem-}(NG`FMT`16)
formatOfType {-sem-}(Unsigned IntType)					= {-sem-}(NG`FMT`32)
formatOfType {-sem-}(Unsigned LongType)					= {-sem-}(NG`FMT`64)
formatOfType {-sem-}(FloatType)						= {-sem-}(RG`FMT`64)
formatOfType {-sem-}(DoubleType)					= {-sem-}(RG`FMT`64)
formatOfType {-sem-}(LongDoubleType)					= {-sem-}(RG`FMT`64)
formatOfType {-sem-}(FunctionType (t) (p_opt))				= {-sem-}(FG`FMT`64)
formatOfType {-sem-}(EnumType (t) (nu))					= {-sem-}(ZG`FMT`32)
formatOfType {-sem-}(Qualified (type_qualifier) (t))			= formatOfType{--}(t)
\end{code}

Observe that, since the MMIX architecture from Chapter \ref{MMIX} provides for only one representation of all floating point values, in our C compiler all three floating types “"FloatType"”, “"DoubleType"” and “"LongDoubleType"” share the common rational format “"RG`FMT`64"”.

Otherwise every signed bit field type "t" of a given width "n" is associated with the format “"ZG`FMT`n"” and every unsigned such bit field is assigned the format “"NG`FMT`n"”. Further, all pointers to an object or incomplete type "t" and all arrays of such a type are represented by the object format corresponding to "formatOfType{--}(t)", while function pointers depict the same standard function format “"FG`FMT`64"”.  Finally, the format of a structure or union type is always derived from its member list using the construction "formatOfMembers{--}(membersOfType{--}(t))" that will be described shortly. Formally:

\begin{code}
formatOfType {-sem-}(t) ---
  | isBitFieldType{--}(t) && isSignedType{--}(t)			= {-sem-}(ZG`FMT`{-sem-}(widthOfType{--}(t)))
  | isBitFieldType{--}(t) && isUnsignedType{--}(t)			= {-sem-}(NG`FMT`{-sem-}(widthOfType{--}(t)))
  | (isPointerType{--}(t) || isArrayType{--}(t)) && (isObjectType{--}(baseOfType{--}(t)) || isIncompleteType{--}(baseOfType{--}(t))) ---
  									= objectFormat{--}(formatOfType{--}(baseOfType{--}(t)))
  | isPointerType{--}(t) && isFunctionType{--}(t)			= {-sem-}(FG`FMT`64)
  | isCompoundType{--}(t)						= formatOfMembers{--}(membersOfType{--}(t))
\end{code}

In particular, all incomplete structure or union types are always assigned the same standard object format “"OG`FMT`64"”, since their empty member list cannot provide any specific information about the alignment of such objects. However, if "m_set" contains at least one member, then the format is defined as the greatest object format corresponding to the type of any "m_k @@ m_set", with all bit field members taken to have the bit field format "bitFieldFormat". Formally:

\BeginHaskellToken	{formatOfMembers}			{Open}	{«\phi»}
  \HaskellArgument							{MembersT}
  \HaskellResult							{FormatT}
\EndHaskellToken

\IndexTerm		{formatOfMembers}				{\phi members (format)}

\begin{code}
formatOfMembers {-[*]-} :: MembersT -> FormatT

formatOfMembers {-sem-}({-hfil-}{-empty-}[]	) ---
  = 	{-sem-}(OG`FMT`64)
formatOfMembers {-sem-}({-hfil-}m_set		) ---
  =	maximumOfSet{--}(	listToSet [ objectFormat{--}(formatOfType{--}(typeOfMember{--}(m_k))) --
  					| m_k <- m_set, not (isBitFieldType{--}(typeOfMember{--}(m_k))) ]
  	{--}{-hfil-}`union`	listToSet [ objectFormat{--}(bitFieldFormat) --
  					| m_k <- m_set, isBitFieldType{--}(typeOfMember{--}(m_k)) && widthOfType{--}(typeOfMember{--}(m_k)) > 0 ])
\end{code}

Further, in Chapter \ref{C}, certain properties of C programs were formalised with the help of five designated C types “"charType"”, “"wideCharType"”, “"intType"”, “"ptrDiffType"” and “"sizeType"” that were originally introduced in Section \ref{C/Types/Corresponding}. Traditionally, these types have been selected as follows:

\NewHaskellToken	{charType}				{Ord}	{\CSyntax{char\_t}}
\NewHaskellResult	{charType}					{TypeT}
\NewHaskellToken	{intType}				{Ord}	{\CSyntax{int\_t}}
\NewHaskellResult	{intType}					{TypeT}
\NewHaskellToken	{wideCharType}				{Ord}	{\CSyntax{wchar\_t}}
\NewHaskellResult	{wideCharType}					{TypeT}
\NewHaskellToken	{ptrDiffType}				{Ord}	{\CSyntax{ptrdiff\_t}}
\NewHaskellResult	{ptrDiffType}					{TypeT}
\NewHaskellToken	{sizeType}				{Ord}	{\CSyntax{size\_t}}
\NewHaskellResult	{sizeType}					{TypeT}

\IndexTerm		{charType}					{char_t (type)}
\IndexTerm		{intType}					{int_t (type)}
\IndexTerm		{wideCharType}					{wchar_t (type)}
\IndexTerm		{ptrDiffType}					{ptrdiff_t (type)}
\IndexTerm		{sizeType}					{size_t (type)}

\begin{code}
charType, wideCharType, intType, ptrDiffType, sizeType :: TypeT

charType	= {-sem-}(Signed CharType)	
wideCharType	= {-sem-}(Signed IntType)
intType		= {-sem-}(Signed IntType)
ptrDiffType	= {-sem-}(Signed LongType)
sizeType	= {-sem-}(Unsigned LongType)
\end{code}

Like most other compilers, we also choose to represent every object of an enumeration type uniformly as a plain integer, regardless of the precise set of enumerator values introduced by that type. In Section \ref{C/Declarations/EnumerationSpecifiers}, the C type corresponding to a given enumerator list "x_set" was represented by the im·ple·ment·ation-specified construct "typeOfEnumeratorList{--}(t)", which we can now settle as follows:

\BeginHaskellToken	{typeOfEnumeratorList}			{Open}	{\ZapfChancery{T}\!\!\sub{\SmallCaps{enl}}}
  \HaskellArgument							{[Integer]}
  \HaskellResult							{TypeT}
\EndHaskellToken

\IndexTerm		{typeOfEnumeratorList}				{T_ENL integers (type)}

\begin{code}
typeOfEnumeratorList {-[*]-} :: [Integer] -> TypeT

typeOfEnumeratorList {-sem-}(el) = {-sem-}(PlainIntType)
\end{code}

Next on the agenda is the member layout function "layoutOfMembers", that was utilised in Section \ref{C/Declarations/StructAndUnionSpecifiers} for completion of the structure and union type definitions. However, before introducing a concrete definition of that function, we should associate every well-formed object type with one additional property known as «alignment». Intuitively, such an alignment represents the greatest common divisor of all integer values that are suitable for representation of pointers to objects of that type. Formally, the alignment of a given type "t" is written as "alignmentOfType{--}(t)". Its almost universally accepted implementation is represented by the following simple algorithm:

\BeginHaskellToken	{alignmentOfType}			{Open}	{\ZapfChancery{A}}
  \HaskellArgument							{TypeT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{alignmentOfType}				{A type (integer)}

\begin{code}
alignmentOfType {-[*]-} :: TypeT -> Integer

alignmentOfType {-sem-}(t) ---
  | isScalarType	{--}(t)	= sizeOfType{--}(t)
  | isArrayType		{--}(t)	= alignmentOfType{--}(baseOfType{--}(t))
  | isCompoundType	{--}(t)	= lcmOfList{--}[ alignmentOfType{--}(typeOfMember{--}(m_k)) | m_k <- membersOfType{--}(t) ]
\end{code}

In other words, the alignment of all scalar types is always equal to their size, while, for arrays, it is determined by the alignment of the element type "baseOfType{--}(t)". For structure and union types, however, the alignment is defined as the least common multiple of the individual alignments of the type's members "membersOfType{--}(t)", so that, by definition, the size of every object and bit field type always represents an integral multiple of its alignment.

Using this auxiliary definition, we can now specify the precise behaviour of the member layout function "layoutOfMembers". First of all, we must ensure that the total size of the resulting member list, defined in Section \ref{C/Types/Storage} as the greatest value among the sums of each member's size "sizeOfType{--}(typeOfMember{--}(m_k))" with its offset "offsetOfMember{--}(m_k)" within the entire entity, always represents an integral multiple of its alignment, as required for a proper implementation of C arrays containing structure or union objects. The most direct way of guaranteeing this property is to extend every member list "m_set'" with an additional anonymous member located at the offset 0, whose type represents an array of "ceiling (s / a) * a" characters, recalling that the “"PlainCharType"” type always has the predetermined size of 1. The C standard explicitly allows for an implicit incorporation of such «padding» into a structure or union type, so that the following definition of "layoutOfMembers" also reflects the indented behaviour of these constructs envisioned by the authors of the C Standard:

\BeginHaskellToken	{layoutOfMembers}			{Open}	{\ZapfChancery{L}}
  \HaskellArgument							{(StructOrUnionT {->-}, MembersT)}
  \HaskellResult							{MembersT}
\EndHaskellToken

\IndexTerm		{layoutOfMembers}				{L members struct_or_union (members)}

\begin{code}
layoutOfMembers {-[*]-} :: (StructOrUnionT {->-}, MembersT) -> MembersT

layoutOfMembers {-sem-}(su {->-}, m_set) ---
		= m_set' ++ {-sem-}[ Member (PlainCharType `ArrayType` (Just {-sem-}(ceiling (s % a) * a))) ({-hide-}Nothing) {-@-} (0) ]

 where	m_set'	= layoutOfMembers'{--}(su {->-}, m_set)
 	s	= maximum	{--}[ offsetOfMember{--}(m_k) + sizeOfType{--}(typeOfMember{--}(m_k))	| m_k <- m_set' ]
  	a	= lcmOfList	{--}[ alignmentOfType	{--}(typeOfMember{--}(m_k))			| m_k <- m_set' ]
\end{code}

The actual assignment of offsets to the individual members of the list "m_set" is then performed by a separate function "layoutOfMembers'". In particular, if the supplied member list "m_set" belongs to a union type, then "layoutOfMembers'" assigns the same byte and bit offset of 0 to every member "m_k @@ m_set", as if by the "zeroMember{--}(m_k)" function described earlier in Section \ref{C/Declarations/StructAndUnionSpecifiers}, except that any bit field members of a zero or negative width are always purged from the result. However, the behaviour of "layoutOfMembers'" is more complicated if the supplied list is intended for incorporation into a structure object. In that case, "layoutOfMembers'{--}(Struct {->-}, m_set)" iterates over all members "m_k @@ m_set", assigning to each "m_k" a successive byte and bit offset within the structure:

\BeginHaskellToken	{layoutOfMembers'}			{Open}	{\ZapfChancery{L}\prime}
  \HaskellArgument							{(StructOrUnionT, Integer, Integer, Integer {->-}, MembersT)}
  \HaskellResult							{MembersT}
\EndHaskellToken

\IndexTerm		{layoutOfMembers'}				{L' members struct_or_union integer integer integer (membes)}

\begin{code}
layoutOfMembers' {-[*]-} :: (StructOrUnionT {->-}, MembersT) -> MembersT

layoutOfMembers' {-sem-}(Union   {->-}, m_set) = [ zeroMember{--}(m_k) | m_k <- m_set, not (isBitFieldType{--}(typeOfMember{--}(m_k))) || widthOfType{--}(typeOfMember{--}(m_k)) > 0 ]
layoutOfMembers' {-sem-}(Struct  {->-}, m_set) = layoutOfMembers''{--}(0, 0, undefined {->-}, m_set)
\end{code}

In particular, the construction "layoutOfMembers''{--}(sigma, omega, varsigma {->-}, m_set)" performs the actual allocation of member offsets using three integer parameters "sigma", "omega" and "varsigma", all of which are assigned the initial values of 0 at the beginning of the structure. Intuitively, "sigma" specifies the total size of all members process thus far, "omega" determines the number of bits left vacant in the bit field container preceding the current member and "varsigma" represents the offset of that partially populated container whenever "omega > 0". Using these three parameters, the precise layout of a given list "m_set" can be obtained by the following recursive algorithm:

\begin{list}

•	If "m_set" is empty, then the entire structure has been allocated and the algorithm terminates with an empty member list "emptySet".

•	Otherwise, if "sigma" does not represent an integral multiple of the alignment appropriate for the type "t" of the first member in "m_set", then the supplied member list is reprocessed with a new value of that parameter equal to "ceiling (sigma / alignmentOfType{--}(t)) * alignmentOfType{--}(t)", also setting "omega" to 0 and leaving "varsigma" undefined.

•	Once the correct alignment of "sigma" has been attained, we can use its value directly as the offset for the first member of "m_set", provided that that member has a well-formed object type "t". Any remaining members in "m_set" are then processed, after incrementing "sigma" by the size "sizeOfType{--}(t)" of the newly-allocated member and resetting "omega" to 0.

•	On the other hand, if the first member has a bit field type of a positive width "widthOfType{--}(t)" no greater than the current "omega" value, then, the member's type is adjusted with a bit offset equal to "omega - widthOfType{--}(t)", also placing the entire bit field member at the offset "varsigma". The remainder of the list is then processed with the same values of the "sigma" and "varsigma" parameters, but with "omega" decreased by "widthOfType{--}(t)".

•	If, however, the bit field is too wide for the number of bits "omega" available in the previously allocated container member, then the current container, if any, is abandoned and a new one is introduced into the structure. In particular, we set "varsigma" to the original value of "sigma", increment "sigma" itself by the size of the bit field container format "bitFieldFormat" and, finally, set "omega" to the width "widthOfFormat{--}(bitFieldFormat)" of the newly allocated container member. The bit field member is then reprocessed under these new parameter values.

•	Finally, we always reset "omega" to 0 whenever a bit field container with a zero or negative width is encountered within the structure. The member itself is always dropped from the resulting list.

\end{list}

Formally:

\BeginHaskellToken	{layoutOfMembers''}			{Open}	{\ZapfChancery{L}\pprime}
  \HaskellArgument							{(Integer, Integer, Integer {->-}, MembersT)}
  \HaskellResult							{MembersT}
\EndHaskellToken

\IndexTerm		{layoutOfMembers''}				{L'' members integer integer integer (members)}

\begin{code}
layoutOfMembers'' {-[*]-} :: (Integer, Integer, Integer {->-}, MembersT) -> MembersT

layoutOfMembers'' {-sem-}(sigma, omega, varsigma {->-}, m_set) ---
  | m_set == {-empty-}[] ---
  	= {-empty-}[]
  | sigma `mod` alignmentOfType{--}(t) /= 0 ---
  	= layoutOfMembers''{--}(ceiling (sigma % alignmentOfType{--}(t)) * alignmentOfType{--}(t), 0, undefined {->-}, m_set)
  | isObjectType{--}(t) ---
  	= {-sem-}[Member (t) (x) {-@-} (sigma)] ++ layoutOfMembers''{--}(sigma + sizeOfType{--}(t), 0, undefined {->-}, tail{--}(m_set))
  | isBitFieldType{--}(t) && inRange (0) (<) (widthOfType{--}(t)) (<=) (omega) ---
  	= {-sem-}[Member (t') (x) {-@-} (varsigma)] ++ layoutOfMembers''{--}(sigma, omega - widthOfType{--}(t'), varsigma {->-}, tail{--}(m_set))
  | isBitFieldType{--}(t) && widthOfType{--}(t) > omega ---
  	= layoutOfMembers''{--}(sigma + sizeOfFormat{-sem-}(bitFieldFormat), widthOfFormat{--}(bitFieldFormat), sigma {->-}, m_set)
  | isBitFieldType{--}(t) && widthOfType{--}(t) <= 0 ---
  	= layoutOfMembers''{--}(sigma, 0, undefined {->-}, tail{--}(m_set))

 where	 {-sem-}(Member (t) (x) {-@-} (n)) ---
 	 		= head{--}(m_set)
 	 {-raw-}(t')  	= setToList (tqOfType{--}(t)) `qualifiedType` --
 	 			{-sem-}(BitFieldType ({-sem-}(baseOfType{--}(t))) {-:-} ({-sem-}(widthOfType{--}(t))) {-@-} ({-sem-}(omega - widthOfType{--}(t))))
\end{code}

In Section \ref{C/Expressions/FunctionArguments}, a similar construction "layoutOfVaList{--}(_V)" was also employed during preparation of the argument values for calls to C functions with a variable parameter list. In particular, given a set "_V" of all temporary local variables allocated for storage of these arguments' values, "layoutOfVaList{--}(_V)" produces a mapping of each variable "nu_k @@ dom{--}(_V)" to successive offsets within a conglomerated “\CKeyword{va\_list}” object, with every argument allocated at an address aligned to the MMIX word boundary. Formally:

\BeginHaskellToken	{layoutOfVaList}			{Open}	{\ZapfChancery{L}}
  \HaskellArgument							{V}
  \HaskellResult							{(MapT VarT Integer)}
\EndHaskellToken

\IndexTerm		{layoutOfVaList}				{L V (\nu integer sss)}

\begin{code}
layoutOfVaList {-[*]-} :: V -> (MapT VarT Integer)

layoutOfVaList {-sem-}(_V) ---
  | _V   == emptyMap	= emptyMap
  | _W'  == emptyMap	= listToMap [ {-raw-}(nu {-:-}, 0) ]
  | otherwise		= _W' `mapUnion` listToMap [ {-raw-}(nu {-:-}, maximum [ {-,-} ceiling ((n_k + sizeOfEnvelope{--}(_V#(nu_k))) % 8) * 8 --
  											| {-raw-}(nu_k {-:-}, n_k) <- mapToList _W' {-,-} ]) ]
  
 where	_W'	= layoutOfVaList{--}(_V `mapSetDifference` listToSet [ nu ])
 	nu	= maximumOfSet{--}(dom{--}(_V))
\end{code}

A precise manipulation of bit field l-values also introduces a number of further im·ple·ment·ation-defined aspects into the language. In Section \ref{C/Expressions/LValues}, a pair of constructions "unpackBitField{--}(t {->-}, alpha)" and "packBitField{--}(t, alpha {->-}, alpha')" were used to, respectively, extract the value of a given bit field "t" from its container atom "alpha" and to populate that container with a new value "alpha'" for that bit field. On MMIX, the first of these constructions can be represented as follows:

\BeginHaskellToken	{unpackBitField}			{Open}	{\ZapfChancery{U}}
  \HaskellArgument							{(TypeT {->-}, AtomT VarT)}
  \HaskellResult							{AtomT VarT}
\EndHaskellToken

\IndexTerm		{unpackBitField}				{U atom type (atom)}

\begin{code}
unpackBitField {-[*]-} :: (TypeT {->-}, AtomT VarT) -> AtomT VarT

unpackBitField {-sem-}(t {->-}, alpha) ---
	= {-sem-}(OP2 ({-hv-}(OP2 (alpha) --
				(SHL) (NG`FMT`64) --
				(IMM (toRational {-sem-}(64 - widthOfType{--}(t) - offsetOfType{--}(t))) (NG`FMT`64)))) --
			(SHR) ({-sem-}(genreOfFormat{--}(formatOfType{--}(t)))`FMT`64) --
			(IMM (toRational {-sem-}(64 - widthOfType{--}(t))) (NG`FMT`64)))
\end{code}

Intuitively, this expression discards the most significant "64 - widthOfType{--}(t) - offsetOfType{--}(t)" bits of "alpha" using an appropriate “"SHL{-sub-}(NG`FMT`64)"\!” operation, and shifts the resulting bit field value right by "64 - widthOfType{--}(t)" bit positions using “"SHR{-sub-}(NG`FMT`64)"\!” or “"SHR{-sub-}(ZG`FMT`64)"”, as appropriate for the genre of "t", essentially sign-extending its value to 64 bits and placing the result at the \hbox{bit offset of 0.}

The dual operation "packBitField{--}(t, alpha {->-}, alpha')" begins by applying the supplied bit container value "alpha" to the bit mask \hbox{"2^64 - 2^(widthOfType{--}(t) + offsetOfType{--}(t)) + 2^(offsetOfType{--}(t)) - 1"}, using an appropriate bitwise AND operation, effectively discarding the existing content of the specified bit field in "alpha", then shifts "alpha'" into place, while also truncating any of its unnecessary most significant bits. It delivers a bitwise OR of the resulting two atomic values. Formally:

\BeginHaskellToken	{packBitField}				{Open}	{\ZapfChancery{P}}
  \HaskellArgument							{(TypeT, AtomT VarT {->-}, AtomT VarT)}
  \HaskellResult							{AtomT VarT}
\EndHaskellToken

\IndexTerm		{packBitField}					{P atom type atom (atom)}

\begin{code}
packBitField {-[*]-} :: (TypeT, AtomT VarT {->-}, AtomT VarT) -> AtomT VarT

packBitField {-sem-}(t, alpha {->-}, alpha') ---
	= {-sem-}(OP2 	({-hv-}(OP2 (alpha) --
				(AND) (NG`FMT`64) --
				(IMM (toRational {-sem-}(2^64 - 2^(widthOfType{--}(t) + offsetOfType{--}(t)) + 2^(offsetOfType{--}(t)) - 1)) (NG`FMT`64)))) --
			(IOR) (bitFieldFormat)
			({-hv-}(OP2 ({-hv-}(OP2 (alpha') --
						(SHL) (NG`FMT`64) --
						(IMM (toRational {-sem-}(64 - widthOfType{--}(t))) (NG`FMT`64)))) --
					(SHR) (NG`FMT`64) --
					(IMM (toRational {-sem-}(64 - widthOfType{--}(t) - offsetOfType{--}(t))) (NG`FMT`64)))))
\end{code}

In Section \ref{C/Declarations/Initialisers}, one more bit field assembly construction is required to combine the initial values of multiple bit fields that appear within a single memory-resident object, each pre-packed into the bit field container “"IMM (0) (bitFieldFormat)"\!”. Intuitively, such feat can be achieved by performing an inclusive OR of all atomic values involved in the operation. Formally, this simple construction is implemented on MMIX as follows:

\BeginHaskellToken	{packAtoms}				{Open}	{\ZapfChancery{P}}
  \HaskellArgument							{AtomsT VarT}
  \HaskellResult							{AtomT VarT}
\EndHaskellToken

\IndexTerm		{packAtoms}					{P atoms (atom)}

\begin{code}
packAtoms {-[*]-} :: AtomsT VarT -> AtomT VarT

packAtoms {-sem-}(alpha_set) ---
  | (alpha_set == {-empty-}[]) = {-sem-}(IMM (0) (NG`FMT`64))
  | (alpha_set /= {-empty-}[]) = {-sem-}(OP2 ({-sem-}(head{--}(alpha_set))) (IOR) (NG`FMT`64) ({-sem-}(packAtoms{--}(tail{--}(alpha_set)))))
\end{code}

Finally, the two memory copy operations  “"setObject"” and “"iniObject"\!” that, in Sections \ref{C/Expressions} and \ref{C/Declarations} were used to depict assignments and initialisations of structure and union objects, can be represented naïvely by a sequence of one or more character copy operations, derived from the size of the underlying object type "t" as follows:

\BeginHaskellToken	{setObject}				{Ord}	{set}
  \HaskellArgument							{(TypeT {->-}, AtomT VarT)}
  \HaskellToken		{{-to-}}				{Ord}	{to}
  \HaskellArgument							{(TypeT {->-}, AtomT VarT)}
  \HaskellResult							{TermT VarT}
\EndHaskellToken

\BeginHaskellToken	{iniObject}				{Ord}	{set\sub{\SmallCaps{i}}}
  \HaskellArgument							{(TypeT {->-}, AtomT VarT)}
  \HaskellToken		{{-to-}}				{Ord}	{to}
  \HaskellArgument							{(TypeT {->-}, AtomT VarT)}
  \HaskellResult							{TermT VarT}
\EndHaskellToken

\NewHaskellToken	{{-to-}}				{Ord}	{to}

\IndexTerm		{setObject}					{set_to atom type atom type (term)}
\IndexTerm		{iniObject}					{seti_to atom type atom type (term)}

\begin{code}
setObject {-  (	 {-[*]-}) {-to-} ({-[*]-}) -}, iniObject {- ({-[*]-}) {-to-} ({-[*]-}) -} :: (TypeT {->-}, AtomT VarT) -> (TypeT {->-}, AtomT VarT) -> TermT VarT

setObject     (	 {-sem-}(t1 {->-}, alpha1)) {-to-} ({-sem-}(t2 {->-}, alpha2)) = copyObject{-sem-}(alpha1, alpha2, sizeOfType{--}(t2))
iniObject     (	 {-sem-}(t1 {->-}, alpha1)) {-to-} ({-sem-}(t2 {->-}, alpha2)) = copyObject{-sem-}(alpha1, alpha2, sizeOfType{--}(t2))
\end{code}

In both cases, an actual Etude term that performs the required movement of data is constructed from the size of the source object "t2" by the following recursive algorithm:

\BeginHaskellToken	{copyObject}				{Ord}	{copy}
  \HaskellArgument							{(AtomT VarT, AtomT VarT, Integer)}
  \HaskellResult							{TermT VarT}
\EndHaskellToken

\IndexTerm		{copyObject}					{copy atom atom integer (term)}

\begin{code}
copyObject {-[*]-} :: (AtomT VarT, AtomT VarT, Integer) -> TermT VarT

copyObject {-sem-}(alpha1, alpha2, n) ---
  | n > 0	= {-sem-}(	LET [{-hv-}[] 		=: {-sem-}(copyObject{--}(alpha1, alpha2, n - 1)) ] $
  			  	LET [{-raw-}[TVar 1] 	=: GET (OP2 (alpha2) (ADD) (OG`FMT`64) (IMM (toRational n) (ZG`FMT`64)) {-hide-}, {-hide-}emptySet) (NG`FMT`8)] $
  			  	SET (OP2 (alpha1) (ADD) (OG`FMT`64) (IMM (toRational n) (ZG`FMT`64)) {-hide-}, {-hide-}emptySet) (NG`FMT`8) {-TO-} (VAR (TVar 1)))
  | n <= 0	= {-sem-}(	RET [])
\end{code}

Observe that, since our target architecture does not implement any memory access attributes, there is no difference between the “"setObject"” and “"iniObject"\!” variants of this operation. Further, it should be pointed out that the above definition of “"copyObject"” could be easily enhanced to take advantage of the precise address alignment information included in every Etude object format. For example, objects aligned on a word boundary could be copied using the “"GET (..) (NG`FMT`64)"” and “"SET (..) (NG`FMT`64)"” term forms instead of the above “"GET (..) (NG`FMT`8)"” and “"SET (..) (NG`FMT`8)"” variants, therefore reducing the length of the resulting term sequence by 87.5\%. However, in the interest of exposition, I leave all such improvements open for a future work on optimising program transformations with \hbox{the Etude framework.}

Last but not least, in Section \ref{C/Expressions/Constants} one additional notation "valueOfAtom(phi){--}(alpha)" was left open to interpretation by individual C compilers. In Chapter \ref{C}, this construction was used to obtain an actual numeric value corresponding to some closed Etude atom "alpha" that was derived from a constant C expression of an arithmetic type associated with the Etude format "phi".

Although the C Standard admits a broad spectrum of different implementations for "valueOfAtom(phi)", including some that disagree with certain aspects of the true meanings of normal Etude atoms, like most modern C compilers, ours will always ensure that every arithmetic C constant has precisely the same meaning as a syntactically identical expression evaluated during the actual program execution. Accordingly, in this section, the function "valueOfAtom(phi)" is defined directly in terms of the actual operational semantics of Etude atoms "evalAtom" that were described in Section \ref{Lambda/Arch/Atoms}. It differs from "evalAtom" only insofar as its support for floating point arithmetic is restricted to operations over finite numeric quantities. Further, the reader should observe that, unlike "evalAtom", "valueOfAtom(phi)" is formulated as a monadic construction rather than a partial Haskell function, in order to facilitate proper diagnostics of malformed C programs. In particular, a typical implementation of "valueOfAtom(phi)" can be derived from the evaluation function "evalAtom" of the underlying instruction set architecture as follows:

\BeginHaskellToken	{valueOfAtom}				{Open}	{\ZapfChancery{V}}
  \HaskellContext							{(MonadFix _M)}
  \HaskellModifier							{\sub}
  \HaskellArgument							{FormatT}
  \HaskellArgument							{AtomT VarT}
  \HaskellResult							{_M{--}(Rational)}
\EndHaskellToken

\IndexTerm		{valueOfAtom}					{V format atom (M rational) ((monad-fix M))}

\begin{code}
valueOfAtom {-[*]-} {-[*]-} :: (MonadFix _M, Monad _M) => FormatT -> AtomT VarT -> _M{--}(Rational)

valueOfAtom ({-sem-}(phi')) {-sem-}(IMM (x) (phi)) = do
	require $	((genreOfFormat{--}(phi   ) @@ listToSet [NG, ZG] && inRange (1) (<=) (encodingOfFormat{--}(phi   )) (<=) (64)) || phi   == {-sem-}(RG`FMT`64)) &&
			((genreOfFormat{--}(phi'  ) @@ listToSet [NG, ZG] && inRange (1) (<=) (encodingOfFormat{--}(phi'  )) (<=) (64)) || phi'  == {-sem-}(RG`FMT`64)) &&
			(phi' /= {-sem-}(RG`FMT`64) || isFiniteRational{--}(encodeAtom(phi){--}(x)))
  	return $ (decodeAtom(phi'){--}(encodeAtom(phi){--}(x)))

valueOfAtom ({-sem-}(phi'')) {-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(alpha)) = do
	require $ 	((genreOfFormat{--}(phi'  ) @@ listToSet [NG, ZG] && inRange (1) (<=) (encodingOfFormat{--}(phi'  )) (<=) (64)) || phi'  == {-sem-}(RG`FMT`64))
	x	<- valueOfAtom(phi)	{--}(alpha)
	z	<- valueOfAtom(phi'')	{--}(evalAtom{-sem-}(OP1 (CNV phi' {-!-}) (phi) {--} {-hv-}(IMM (x) (phi))))
	return $ (z)

valueOfAtom ({-sem-}(phi')) {-sem-}(OP1 (op) (phi) {--} {-hv-}(alpha)) = do
	require $ (phi /= {-sem-}(RG`FMT`64) || op /= {-sem-}(NOT))
	x	<- valueOfAtom(phi)	{--}(alpha)
	z	<- valueOfAtom(phi')	{--}(evalAtom{-sem-}(OP1 (op) (phi) {--} {-hv-}(IMM (x) (phi))))
	return $ (z)

valueOfAtom ({-sem-}(phi')) {-sem-}(OP2 (alpha1) (op) (phi) (alpha2)) = do
	require $ (phi /= {-sem-}(RG`FMT`64) || op /@ listToSet [ REM, AND, IOR, XOR, SHL, SHR ])
	x 	<- valueOfAtom(phi)	{--}(alpha1)
	y 	<- valueOfAtom(phi)	{--}(alpha2)
	z	<- valueOfAtom(phi')	{--}(evalAtom{-sem-}(OP2 (IMM (x) (phi)) (op) (phi) (IMM (y) (phi))))
	return $ (z)

valueOfAtom ({-sem-}(phi')) {-sem-}(VAR nu) = fail ("Semantic error")
\end{code}

This final definition completes the entire specification of the C programming language under the MMIX architecture. While, in theory, no portable C program should ever rely on any of the details discussed in this section, most practical software projects tend to require varying degrees of knowledge about the precise behaviour of the underlying system, which, in most cases, goes beyond the somewhat relaxed guarantees that are provided by the C Standard. Fortunately, as demonstrated by the above set of Haskell definitions, the linear correctness approach to compiler design can readily facilitate such programs, by furnishing them with multiple layers of semantic specifications, each depicting the behaviour of actual computational hardware with an ever increasing amount of detail.

\end{appendix}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%