\set{capture}{hidden}
\begin{code}

module C.Scopes (
	linkageOfDesignator,
	typeOfDesignator,
	indexOfDesignator,
	isGlobalLinkage,
	isStaticLinkage,
	isLValueLinkage,
	variableOfLinkage,
	integerOfLinkage,
	uvar,
	utag,
	completeScope,
	completeType,
	completeMembers,
	completePrototypeOpt,
) where

import Additional
import C.Variables
import C.Syntax
import C.Types
import {-# SOURCE #-} C.Implementation
import Lambda

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{C/Scopes}
\set{title}		{Name Spaces and Scopes}
\begin{section}

In C, identifiers are used to designate a number of different kinds of entities, such as «functions», «variables», «type names», «type tags», «statement labels» and finally «members» of structure and union types. When two or more entities in a C program are designated by the same identifier, they are said to «share» that identifier with each other. A single identifier may be always shared by entities of a distinct kind, with the actual entity being denoted determined by its syntactic context. Accordingly, the language is said to define a number of «name spaces» for its identifiers, one for functions, variables and type names (known collectively as «ordinary identifiers»), one for type tags, one for statement labels and one each for every structure and union type defined by the program. The present section contains a detailed discussion of the ordinary identifier, tag and member name spaces, while the 
the rather specialised label identifiers are scrutinised later in Section \ref{C/Statements} as part of a semantic analysis of C statements.

A single identifier may be also shared by multiple entities belonging to the same name space. However, unlike identifiers shared across name spaces, at most one of the entities that share such an identifier can be «visible» to any given portion of a C program. The collection of all entities visible to some program fragment is known as a «scope» and, in most cases, corresponds loosely to a set of constructs enclosed in a single compound statement “\!"CompoundStatement () (..)"\!”.

The scoping structure of C programs differs substantially from that of typical purely functional languages such as Etude. In the Etude term “"LET [nu =: tau1] (tau2)"\!”, the scope of "nu" is contained cleanly within the body "tau2", as dictated by the lexical structure of the underlying parse tree, whereas, in the C statement \hbox{“\!"CompoundStatement (Declaration (IntSpecifier) (x) {-;-}) (ExpressionStatement (s1) {-;-} ExpressionStatement (s2) {-;-} .. ExpressionStatement (s{-sub-}n) {-;-})"\!”,} the scope of "x" extends to the entire statement list “"ExpressionStatement (s1) {-;-} ExpressionStatement (s2) {-;-} .. ExpressionStatement (s{-sub-}n) {-;-}"\!” and concludes only at the end of the complete compound statement that surrounds its declaration.

This effect is even more pronounced for tag names, whose declarations may be buried deeply within a C expression, often transcending the program's lexical syntax in a highly unpredictable manner. For example, the following code fragment constitutes a perfectly valid C program:

\begin{C}
	int x 		= sizeof (struct s { int a, b, c });
	struct s y 	= { 1, 2, 3 };
	int z 		= y.a + y.b + y.c;
\end{C}

The structure type \C{s} is introduced within the initialiser of \C{x} and remains visible to the following declarations of \C{y} and \C{z}, so that its scope extends outside of the nesting structure within which \C{s} has been defined.

In light of such examples, most of the entities encountered within a C program cannot be analysed in isolation and must be generally interpreted in the context of the surrounding program in which they appear. In the following discussion, this linguistic feature is modelled by a «translation context» explicitly associated with most syntactic entities of the program. Any adjustments to this context introduced by a given syntactic entity are included as part of that entity's formal denotation and, in almost all cases, the updated context is propagated to all subsequent entities in the depth-first left-to-right traversal of the program's parse tree. For example, in the above code fragment, the structure type \C{s} is introduced into the current tag scope "_T" by its definition \hbox{“"Struct x CompoundStatement () (IntSpecifier a `Comma` b `Comma` ExpressionStatement (c) {-;-} )"\!”} and remains visible to the following two statements.

For most entities, the translation context includes at least four components "_S", "_T", "_D" and "_I", known as the «current variable» and «tag scopes», «set of item definitions» and the «scope index», respectively. Their structures are described in Haskell by the following three type synonyms:

\IndexType		{S}						{S (\tstar)}
\IndexType		{D}						{D (\tstar)}
\IndexType		{I}						{I (\tstar)}

\begin{syntax}
\InsertHaskell		{S, D, I}
\end{syntax}

In particular, the current scopes "_S" and "_T" define the semantic significance of all visible identifiers whose declarations are bound to a C variable, typedef name, function designator, structure, union or enumeration type. Further, the set of item definitions "_D" depicts the actual bindings of all Etude variables associated with any C functions and data objects defined in the course of translation. Finally, the scope index "_I" represents the nesting depth of the current scope with the program's lexical structure.

The actual binding of a given entity in its current scope associates its identifier with a «name designator» formed from a C type, an entity known as «linkage» and a scope index, as captured by the following data type definition:

\IndexType		{DesignatorT}					{designator (\tstar)}
\IndexCons		{Designator}					{@ type linkage scope (designator)}

\begin{syntax}
\InsertHaskell		{DesignatorT}
\end{syntax}

whose three components can be accessed individually with help of the following trivial Haskell functions:

\BeginHaskellToken	{linkageOfDesignator}			{Open}	{\ZapfChancery{L}}
  \HaskellArgument							{DesignatorT}
  \HaskellResult							{LinkageT}
\EndHaskellToken

\BeginHaskellToken	{typeOfDesignator}			{Open}	{\ZapfChancery{T}}
  \HaskellArgument							{DesignatorT}
  \HaskellResult							{TypeT}
\EndHaskellToken

\BeginHaskellToken	{indexOfDesignator}			{Open}	{\ZapfChancery{I}}
  \HaskellArgument							{DesignatorT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{linkageOfDesignator}				{L designator (linkage)}
\IndexTerm		{typeOfDesignator}				{T designator (type)}
\IndexTerm		{indexOfDesignator}				{I designator (integer)}

\begin{code}
linkageOfDesignator	{-[*]-} 	:: DesignatorT -> LinkageT		-- linkage
typeOfDesignator 	{-[*]-} 	:: DesignatorT -> TypeT			-- type
indexOfDesignator	{-[*]-} 	:: DesignatorT -> Integer		-- identifier

linkageOfDesignator	{-sem-}(Designator (ell) (t) {-@-} (_I) {-,-})	= ell
typeOfDesignator 	{-sem-}(Designator (ell) (t) {-@-} (_I) {-,-})	= t
indexOfDesignator	{-sem-}(Designator (ell) (t) {-@-} (_I) {-,-})	= _I
\end{code}

A designator's scope index "indexOfDesignator{--}(d)" identifies the precise scope into which the binding of the corresponding C name has been originally introduced. As we shall soon discover, this index is critical to a correct formalisation of the C declaration semantics in \hbox{Section \ref{C/Declarations}.} The linkage of a designator has the following structure:

\IndexType		{LinkageT}					{linkage (\tstar)}
\IndexCons		{ExternLinkage}					{extern var (linkage)}
\IndexCons		{InternLinkage}					{intern var (linkage)}
\IndexCons		{PrivateLinkage}				{private var (linkage)}
\IndexCons		{AutoLinkage}					{auto var (linkage)}
\IndexCons		{RegisterLinkage}				{register var (linkage)}
\IndexCons		{ConstantLinkage}				{constant integer (linkage)}
\IndexCons		{TypeLinkage}					{type (linkage)}

\begin{syntax}
\InsertHaskell		{LinkageT}
\end{syntax}

Collectively, all of the “"ExternLinkage"” and “"InternLinkage"” linkage forms are said to represent the family of «globally-linked» entities, which are identified formally by the notation “"isGlobalLinkage{--}(ell)"”. Further, the set of all “"ExternLinkage"”, “"InternLinkage"” and “"PrivateLinkage"” entities constitutes the family of «static linkage forms», identified by the Haskell predicate “"isStaticLinkage"” \hbox{as follows:}

\BeginHaskellToken	{isGlobalLinkage}			{Open}	{GL}
  \HaskellArgument							{LinkageT}
  \HaskellResult							{Bool}
\EndHaskellToken

\BeginHaskellToken	{isStaticLinkage}			{Open}	{SL}
  \HaskellArgument							{LinkageT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isGlobalLinkage}				{GL linkage (bool)}
\IndexTerm		{isStaticLinkage}				{SL linkage (bool)}

\begin{code}
isGlobalLinkage 	{-[*]-}, isStaticLinkage {-[*]-} :: LinkageT -> Bool

isGlobalLinkage		{-sem-}(ExternLinkage	(nu))		= True
isGlobalLinkage		{-sem-}(InternLinkage 	(nu))		= True
isGlobalLinkage 	{-sem-}(other)				= False

isStaticLinkage		{-sem-}(ExternLinkage 	(nu))		= True
isStaticLinkage		{-sem-}(InternLinkage 	(nu))		= True
isStaticLinkage		{-sem-}(PrivateLinkage 	(nu))		= True
isStaticLinkage 	{-sem-}(other)				= False
\end{code}

The linkage of a designator serves the dual rôle of associating certain C identifiers with Etude variables, while also providing additional information about the precise usage of these variables within a C program. In particular, an identifier declared with a linkage of the form “"ExternLinkage (nu)"”, “"InternLinkage (nu)"” or “"PrivateLinkage (nu)"” always designates a single Etude object, whose declaration is included in the set of item definitions of the eventual Etude module constructed from the entire translation unit, while identifiers associated with a linkage of the form “"AutoLinkage (nu)"” or “"RegisterLinkage (nu)"” represent temporal variables defined locally within a C function, with the later form reserved specifically for identifiers declared with the “"RegisterSpecifier"” storage class specifier as described later in Section \ref{C/Expressions}. Collectively, these five constructions are known as the «l-value linkage» forms and are singled out by the following predicate “"isLValueLinkage"”:

\BeginHaskellToken	{isLValueLinkage}			{Open}	{LV}
  \HaskellArgument							{LinkageT}
  \HaskellResult							{Bool}
\EndHaskellToken

\IndexTerm		{isLValueLinkage}				{LV linkage (bool)}

\begin{code}
isLValueLinkage 	{-[*]-} :: LinkageT -> Bool

isLValueLinkage		{-sem-}(ExternLinkage 		(nu))		= True
isLValueLinkage		{-sem-}(InternLinkage 		(nu))		= True
isLValueLinkage		{-sem-}(PrivateLinkage 		(nu))		= True
isLValueLinkage		{-sem-}(AutoLinkage 		(nu))		= True
isLValueLinkage		{-sem-}(RegisterLinkage 	(nu))		= True
isLValueLinkage 	{-sem-}(other)					= False
\end{code}

A further discussion of l-values is included later in Section \ref{C/Expressions/LValues}. Intuitively the Etude variable "nu" associated with every such linkage form denotes a mutable object designated by that identifier. In this work, this variable is represented uniformly by the notation “"variableOfLinkage{--}(ell)"”, as formalised by the following Haskell definition:

\BeginHaskellToken	{variableOfLinkage}			{Open}	{«\nu»}
  \HaskellArgument							{LinkageT}
  \HaskellResult							{VarT}
\EndHaskellToken

\IndexTerm		{variableOfLinkage}				{\nu linkage (\nu)}

\begin{code}
variableOfLinkage {-[*]-} :: LinkageT -> VarT

variableOfLinkage {-sem-}(ExternLinkage 	(nu)) 	= nu
variableOfLinkage {-sem-}(InternLinkage 	(nu)) 	= nu
variableOfLinkage {-sem-}(PrivateLinkage 	(nu)) 	= nu
variableOfLinkage {-sem-}(AutoLinkage		(nu)) 	= nu
variableOfLinkage {-sem-}(RegisterLinkage	(nu)) 	= nu
\end{code}

The actual bindings of these Etude variables are maintained in the associated set of variable definitions "_D", whose codomain will contain, by the end of an entire translation process, the precise set of items declared in the Etude module constructed from the supplied C program. Since local C variables are never associated with a static Etude item, they are represented in "_D" by an omitted entity “"Nothing"”. Their bindings are only recorded in "_D" for the sake of presentation, in order to ensure that the denotation of every logically-distinct C entity retains a unique syntax after its translation into Etude. In particular, whenever a new local object is introduced into the C program, its linkage is generally bound to a unique Etude variable of the form “"IVar n"\!”, in which the integer index "n" renders the variable distinct from all others already present in "_D". Such a unique variable can be obtained naturally by the following Haskell construction:

\BeginHaskellToken	{uvar}					{Open}	{\nu}
  \HaskellArgument							{D}
  \HaskellResult							{VarT}
\EndHaskellToken

\IndexTerm		{uvar}						{\nu D (\nu)}

\begin{code}
uvar {-[*]-} :: D -> VarT

uvar {-sem-}(_D) = succ{--}(maximumOfSet{--}(listToSet [ {-sem-}(IVar k) |{-sem-}(IVar k) <- setToList (dom{--}(_D)) ] `union` listToSet [IVar 0]))
\end{code}

which relies on the standard Haskell successor function “"succ"” to construct a new variable index greater than any others already present in "_D". Before obtaining the successor of the maximum element from the domain of "_D", the above construction ensures that this domain includes at least one internal variable of the “"IVar n"\!” form by extending this set with an otherwise-unused variable “"IVar 0"\!”. We will discuss the precise syntax and semantics of variable introduction separately in Section \ref{C/Declarations}.

A linkage of the form “"ConstantLinkage (n)"” indicates that the corresponding identifier represents an «enumeration constant» with the integer value specified by the integer "n". Since this value can be viewed as an intuitive denotation of the constant, its retrieval is depicted in the following presentation by the following notation “"integerOfLinkage{--}(d)"”:

\BeginHaskellToken	{integerOfLinkage}			{Open}	{«n»}
  \HaskellArgument							{LinkageT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{integerOfLinkage}				{n linkage (integer)}

\begin{code}
integerOfLinkage {-[*]-} :: LinkageT -> Integer

integerOfLinkage {-sem-}(ConstantLinkage (n)) = n
\end{code}

Finally, a variable name bound to an object declaration with a “"TypeLinkage"” linkage is said to represent a «typedef name». Such identifiers denote a C type rather than a variable and are introduced into the concrete syntax of the language by the “\CKeyword{typedef}” declarations scrutinised later in Section \ref{C/Declarations}. Nevertheless, since typedef names share a single name space with other ordinary identifiers used to designate C variables and functions, they are recorded in the current scope as part of its object declaration set.

The “"TypeLinkage"” linkage form is also associated with the designator of every tag name binding present in a current tag scope "_T". The type of such a designator always represents a C structure, union or enumeration type, the syntax of which includes a tag variable used to differentiate between structurally identical but logically distinct types introduced by the program. To ensure uniqueness of these entities, every new structure, union and enumeration type is always labelled with a unique tag variable upon its introduction into the program. For convenience, these tags are generated by the following function “"utag{--}(_D)"” similar to the earlier definition of “"uvar{--}(_D)"”:

\BeginHaskellToken	{utag}					{Open}	{tag}
  \HaskellArgument							{D}
  \HaskellResult							{VarT}
\EndHaskellToken

\IndexTerm		{utag}						{tag D (\nu)}

\begin{code}
utag {-[*]-} :: D -> VarT

utag {-sem-}(_D) = succ{--}(maximumOfSet{--}(listToSet [ {-sem-}(TVar k) |{-sem-}(TVar k) <- setToList (dom{--}(_D)) ] `union` listToSet [TVar 0]))
\end{code}

Besides its use in type identification, these temporary variables serve no other useful purpose in the program and are always bound in the set of its item definitions "_D" to an omitted Etude item “"Nothing"”.

When a new structure or union type is initially introduced into the program's syntax, it always begins as an incomplete type with an empty list of members, subject to a later completion by a subsequent declaration of that type within the same scope. Formally, this type completion process is depicted by the notation “"completeScope{--}(t {->-}, _S)"”, which adjusts the types of all designators found in the supplied scope "_S" as follows:

\BeginHaskellToken	{completeScope}				{Open}	{\ZapfChancery{C}}
  \HaskellArgument							{(TypeT {->-}, S)}
  \HaskellResult							{S}
\EndHaskellToken

\IndexTerm		{completeScope}					{C S type S}

\begin{code}
completeScope {-[*]-} :: (TypeT {->-}, S) -> S

completeScope {-sem-}(t {->-}, _S) ---
		= listToMap [ {-raw-}(x_k {-:-}, {-sem-}(Designator (ell_k) ({-sem-}(completeType{--}(t' {->-}, t_k))) {-@-} (_I_k))) --
				| {-raw-}(x_k {-:-}, {-sem-}(Designator (ell_k) (t_k) {-@-} (_I_k))) <- mapToList _S ]
 where	t'	= {-sem-}(CompoundType ({-sem-}(suOfType{--}(t))) ({-sem-}(tagOfType{--}(t))) ({-sem-}(completeMembers{--}(t' {->-}, membersOfType{--}(t)))))
\end{code}

assuming, of course, that "t" represents a complete structure or union type.

In particular the construction “"completeType{--}(t' {->-}, t)"” substitutes the supplied complete type "t'" for every occurrence of a compatible structure or union type in "t". Formally:

\BeginHaskellToken	{completeType}				{Open}	{\ZapfChancery{C}\sub{\SmallCaps{t}}}
  \HaskellArgument							{(TypeT {->-}, TypeT)}
  \HaskellResult							{TypeT}
\EndHaskellToken

\IndexTerm		{completeType}					{C_T type type (type)}

\begin{code}
completeType {-[*]-} :: (TypeT {->-}, TypeT) -> TypeT

completeType {-sem-}(t' {->-}, t) ---
  | unqualifiedType{--}(t) `areCompatibleTypes` t' ---
  					= tqOfType{--}(t) `requalifiedType` t'
  | isCompoundType	{--}(t)		= tqOfType{--}(t) `requalifiedType` {-sem-}(CompoundType ({-sem-}(suOfType{--}(t))) ({-sem-}(tagOfType{--}(t))) --
  											({-sem-}(completeMembers{--}(t' {->-}, membersOfType{--}(t)))))
  | isPointerType	{--}(t)		= tqOfType{--}(t) `requalifiedType` {-sem-}(PointerType ({-sem-}(completeType{--}(t' {->-}, baseOfType{--}(t)))) --
  											{-*-})
  | isFunctionType	{--}(t)		= tqOfType{--}(t) `requalifiedType` {-sem-}(FunctionType ({-sem-}(completeType{--}(t' {->-}, baseOfType{--}(t)))) --
  											({-sem-}(completePrototypeOpt{--}(t' {->-}, prototypeOfType{--}(t)))))
  | isArrayType		{--}(t)		= tqOfType{--}(t) `requalifiedType` {-sem-}(ArrayType ({-sem-}(completeType{--}(t' {->-}, baseOfType{--}(t)))) --
  											({-sem-}(lengthOfType{--}(t))))
  | otherwise				= t
\end{code}

in which "completeMembers{--}(t {->-}, m_set)" projects the same algorithm onto the type of every member found in the specified list "m_set":

\BeginHaskellToken	{completeMembers}			{Open}	{\ZapfChancery{C}\sub{\SmallCaps{m}}}
  \HaskellArgument							{(TypeT {->-}, MembersT)}
  \HaskellResult							{MembersT}
\EndHaskellToken

\IndexTerm		{completeMembers}				{C_M members type (members)}

\begin{code}
completeMembers {-[*]-} :: (TypeT {->-}, MembersT) -> MembersT

completeMembers	{-sem-}(t {->-}, m_set) ---
	= [ {-sem-}(Member ({-sem-}(completeType{--}(t {->-}, t_k))) (x_k) {-@-} (n_k)) | {-sem-}(Member (t_k) (x_k) {-@-} (n_k)) <- m_set ]
\end{code}

and, similarly, "completePrototypeOpt{--}(t {->-}, p_opt)" completes every type found in a given optional function prototype "p_opt" as follows:

\BeginHaskellToken	{completePrototypeOpt}			{Open}	{\ZapfChancery{C}\sub{\SmallCaps{p}}}
  \HaskellArgument							{(TypeT {->-}, Maybe PrototypeT)}
  \HaskellResult							{Maybe PrototypeT}
\EndHaskellToken

\IndexTerm		{completePrototypeOpt}				{C_P prototype_opt type (prototype_opt)}

\begin{code}
completePrototypeOpt {-[*]-} :: (TypeT {->-}, Maybe PrototypeT) -> Maybe PrototypeT

completePrototypeOpt {-sem-}(t {->-}, Nothing				) ---
	= {-sem-}(Nothing)
completePrototypeOpt {-sem-}(t {->-}, Just (Types (t_set)) {-,-}	) ---
	= {-sem-}(Just (Types ({-sem-}([ completeType{--}(t {->-}, t_k) | t_k <- t_set {-,-} ]))))
completePrototypeOpt {-sem-}(t {->-}, Just (Ellipsis (t_set) {-...-})) ---
	= {-sem-}(Just (Ellipsis ({-sem-}([ completeType{--}(t {->-}, t_k) | t_k <- t_set {-,-} ])) {-...-}))
\end{code}

A careful reader will observe that scope completion may, in fact, produce an infinite Haskell object, whenever the structure or union being completed is itself referenced in the syntax of a pointer entity embedded within the supplied type, member or function prototype operand. Fortunately, with its non-strict evaluation semantics, Haskell is well-equipped to deal with such constructions, provided that we refrain from excessive examination of the resulting member types. For example, in Section \ref{C/Types/Compatibility}, the type compatibility algorithm for structures and unions was defined solely on the basic of equality between their “"struct_or_union"” and tag components, avoiding any reference to the associated member lists.

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%