\set{capture}{hidden}
\begin{code}

module Lambda.Modules (
	termOfModule,
	exportsOfModule,
	itemsOfModule,
	fvOfExports,
	fvOfItems,
	fvOfItem,
	substItems,
	substItem,
	renameData,
) where

import Additional
import Lambda.Syntax
import Lambda.Atoms
import Lambda.Terms
import {-# SOURCE #-} Lambda.Arch

infixr 9 `substItems`, `substItem`, `renameData`

renameData {-[*]-} {-/-} {-[*]-} :: (Ord upsilon, Ord nu) => DataT upsilon -> (MapT (upsilon) (nu)) -> DataT nu

renameData {-sem-}(delta_set) {-/-} {-sem-}(_S) ---
	= listToSet [ {-sem-}(delta_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, {-sem-}(alpha_k `renameAtom` _S)) ---
			| {-sem-}(delta_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k) <- setToList delta_set ]
\end{code}

\BeginHaskellToken	{renameData}				{Void}	{}
  \HaskellContext							{(Ord upsilon, Ord nu)}
  \HaskellArgument							{DataT upsilon}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{(MapT (upsilon) (nu))}
  \HaskellResult							{DataT nu}
\EndHaskellToken

\NewHaskellToken	{`renameData`}				{Op}	{\slash}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Modules}
\set{title}{Modules and Programs}
\begin{section}

Usually, Etude programs are not constructed as a single holistic entity, but rather assembled from one or more discrete «modules» that are designed and compiled as autonomous work units. Any realistic translation of a language such as C must recognise this common software engineering practice and, accordingly, Etude provides the following Haskell structures intended to capture the essential idea of module separation:

\IndexType		{ModuleT}					{module \nu (\tstar)}
\IndexCons		{MODULE}					{MODULE_EXPORT_WHERE term exports items (module)}

\IndexType		{ExportsT}					{exports \nu (\tstar)}
\IndexType		{ItemsT}					{items \nu (\tstar)}

\IndexType		{ItemsT}					{items \nu (\tstar)}

\IndexType		{ItemT}						{item \nu (\tstar)}
\IndexCons		{FUNI}						{ function (item)}
\IndexCons		{OBJI}						{OBJ_WITH envelope data (item)}
\IndexCons		{IMPI}						{IMP string (item)}

\IndexType		{DataT}						{data \nu (\tstar)}

\begin{syntax}
\InsertHaskell		{ModuleT, \par, ExportsT, \par, ItemsT, \par, ItemT, \par, DataT}
\end{syntax}

A single module consists of a term known as «module initialiser» and two sets known as «module exports» and «module definitions», respectively. The initialiser represents an expression evaluated at the beginning of every execution of the program and it is these terms, irrespective of any environments current upon their termination, that dictate all algebraic properties of all software written in Etude. Module exports specify those variables defined within the module that are to be made visible to all other entities within the program. Any such variables are annotated in the module's syntax with their globally-unique names depicted by Haskell string values. Finally, every module includes a set of zero or more bindings of Etude variables to «module items», which, intuitively, describe the initial evaluation environment current prior to every invocation of the program. An individual item can represent an Etude function, an «object specification» or a reference to a named declaration imported from some other module of the entire program. Every object specification entity consists, in turn, of an envelope associated with a «data specification set» whose structure resembles another envelope, in which every «datum» has been annotated with a concrete atomic value, intended to represent the initial content of the corresponding address space region within the object. Intuitively, the collection of all object specifications included in an Etude program represents a concise depiction of its initial object environment defined statically prior to the execution of any initialiser terms and corresponds directly to the notion of static variables in the \hbox{C programming language.}

For convenience, the three components of an Etude module may be accessed individually by the following Haskell functions:

\BeginHaskellToken	{termOfModule}				{Open}	{«\tau»}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{ModuleT nu}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\BeginHaskellToken	{exportsOfModule}			{Open}	{«\superimpose{\raise.5ex\hbox{\macron{}}}{\xi}»}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{ModuleT nu}
  \HaskellResult							{ExportsT nu}
\EndHaskellToken

\BeginHaskellToken	{itemsOfModule}				{Open}	{«\superimpose{\hbox{\!\macron{}}}{\iota}»}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{ModuleT nu}
  \HaskellResult							{ItemsT nu}
\EndHaskellToken

\IndexTerm		{termOfModule}					{\tau module (term)}
\IndexTerm		{exportsOfModule}				{\xi- module (exports)}
\IndexTerm		{itemsOfModule}					{\iota- module (items)}

\begin{code}
termOfModule 		{-[*]-}	:: {- (Ord nu) => -} ModuleT nu -> TermT nu
exportsOfModule		{-[*]-}	:: {- (Ord nu) => -} ModuleT nu -> ExportsT nu
itemsOfModule		{-[*]-} :: {- (Ord nu) => -} ModuleT nu -> ItemsT nu

termOfModule		{-sem-}(MODULE (tau) {-EXPORT-} (xi_set) {-WHERE-} (iota_set) {-,-}) 	= tau
exportsOfModule		{-sem-}(MODULE (tau) {-EXPORT-} (xi_set) {-WHERE-} (iota_set) {-,-}) 	= xi_set
itemsOfModule		{-sem-}(MODULE (tau) {-EXPORT-} (xi_set) {-WHERE-} (iota_set) {-,-}) 	= iota_set
\end{code}

Although a module represents an irreducible entity that, in isolation, does not possess any specific semantic meaning, we can still refer to the free variable sets of its export and item lists using the following predictable Haskell constructions:

\BeginHaskellToken	{fvOfExports}				{Open}	{FV}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{ExportsT nu}
  \HaskellResult							{SetT nu}
\EndHaskellToken

\IndexTerm		{fvOfExports}					{FV exports (\nu s)}

\begin{code}
fvOfExports {-[*]-} :: (Ord nu) => ExportsT nu -> SetT nu

fvOfExports {-sem-}(xi_set {-,,-}) = listToSet [ nu_k | {-sem-}(x_k {-bindTerm-}, nu_k) <- mapToList xi_set {-,,-} ]
\end{code}

\BeginHaskellToken	{fvOfItems}				{Open}	{FV}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{ItemsT nu}
  \HaskellResult							{SetT nu}
\EndHaskellToken

\IndexTerm		{fvOfItems}					{FV items (\nu s)}

\begin{code}
fvOfItems {-[*]-} :: (Ord nu) => ItemsT nu -> SetT nu

fvOfItems {-sem-}(iota_set {-,-}) = unions [ fvOfItem{--}(iota_k) | {-sem-}(nu_k {-bindTerm-}, iota_k) <- mapToList iota_set {-,-} ]
\end{code}

\BeginHaskellToken	{fvOfItem}				{Open}	{FV}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{ItemT nu}
  \HaskellResult							{SetT nu}
\EndHaskellToken

\IndexTerm		{fvOfItem}					{FV item (\nu s)}

\begin{code}
fvOfItem {-[*]-} :: (Ord nu) => ItemT nu -> SetT nu

fvOfItem {-sem-}(FUNI (FUN (nu_set) {-DOT-} (tau))) ---
  = fvOfTerm{--}(tau) `difference` listToSet nu_set
fvOfItem {-sem-}(OBJI (delta_set) {-OF-} (xi_set) {-,,-}) ---
  = unions [ fvOfAtom{--}(alpha_k) | {-sem-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k) <- setToList delta_set {-,,-} ]
fvOfItem {-sem-}(IMPI (x)) ---
  = emptySet
\end{code}

Further, we can also formulate the usual substitution function over the list of module items as follows:

\BeginHaskellToken	{substItems}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{ItemsT nu}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{(MapT (nu) (AtomT nu))}
  \HaskellResult							{ItemsT nu}
\EndHaskellToken

\NewHaskellToken	{`substItems`}				{Op}	{\slash}

\IndexTerm		{substItems}					{\slash items \nu s_atoms (items)}

\begin{code}
substItems {-[*]-} {-/-} {-[*]-} :: (Ord nu) => ItemsT nu -> (MapT (nu) (AtomT nu)) -> ItemsT nu

substItems {-sem-}(iota_set) {-/-} {-sem-}(_S) ---
	= listToMap [ {-sem-}(nu_k {-bindTerm-}, {-sem-}(iota_k `substItem` _S)) | {-sem-}(nu_k {-bindTerm-}, iota_k) <- mapToList iota_set ]
\end{code}

\BeginHaskellToken	{substItem}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{ItemT nu}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{(MapT (nu) (AtomT nu))}
  \HaskellResult							{ItemsT nu}
\EndHaskellToken

\NewHaskellToken	{`substItem`}				{Op}	{\slash}

\IndexTerm		{substItem}					{\slash item \nu s_atoms (item)}

\begin{code}
substItem {-[*]-} {-/-} {-[*]-} :: (Ord nu) => ItemT nu -> (MapT (nu) (AtomT nu)) -> ItemT nu
		  
substItem {-sem-}(FUNI (FUN (nu_set) {-DOT-} (tau))) ---
  {-/-} {-sem-}(_S) = {-sem-}(FUNI (FUN (nu_set) {-DOT-} ({-sem-}(tau `substTerm` (_S `mapSetDifference` listToSet nu_set)))))
substItem {-sem-}(OBJI (delta_set) {-OF-} (xi_set) {-,,-}) ---
  {-/-} {-sem-}(_S) = {-sem-}(OBJI ({-sem-}(listToSet [ {-sem-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, {-sem-}(alpha_k `substAtom` _S)) --
	  			| {-sem-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k) <- setToList delta_set {-,,-} ])) {-OF-} (xi_set))
substItem {-sem-}(IMPI (x)) ---
  {-/-} {-sem-}(_S) = {-sem-}(IMPI (x))
\end{code}

Although all practical Etude implementations, including the one from Chapter \ref{MMIX}, are typically defined to perform translations of only a single module at a time, conceptually, the well-formedness and equivalence properties can be only asserted over entire Etude programs. Formally, such programs are formed by annotating every one of its modules "m_k" with a suitable substitution function "_S_k", so that the set of modules that constitutes a complete description of an entire self-contained Etude program can be represented by the following Haskell data structure:

\IndexType		{ModulesT}					{modules \nu (\tstar)}

\begin{syntax}
\InsertHaskell		{ModulesT}
\end{syntax}

In general, the required substitution functions are constructed by an unspecified «linking process» that is defined individually for each operating system targeted by our compiler and therefore remains outside of the present work's scope. However, on every implementation of the Etude language, the resulting set of modules "m_set" must always satisfy a number of simple constraints described in the remainder of this section.

\NewHaskellToken	{wfModuleJ}				{Ord}	{\TKeyword{wf\sub{m}}}
\NewHaskellDescription	{wfModuleJ}					{Validity of Etude modules}
\IndexTheorem		{wfModuleJ}					{WF_M Validity of Etude modules}

First of all, for every module "m_k" in "dom{--}(m_set)", the set of all free variables appearing in the module's initialiser term "termOfModule{--}(m_k)", exports "exportsOfModule{--}(m_k)" and item definitions "itemsOfModule{--}(m_k)" must represent a subset of the variables mapped in "itemsOfModule{--}(m_k)", which, in turn, must be identical to the domain of the corresponding substitution function "m_set#(m_k)":

\begin{haskell}
wfModuleJ1 :: forall m_set, m_k =>
	WfJ{--}(m_set) ->
	{-sem-}(m_k @@ dom{--}(m_set)) ->
	{-sem-}((fvOfTerm{--}(termOfModule{--}(m_k)) `union` fvOfExports{--}(exportsOfModule{--}(m_k)) `union` fvOfItems{--}(itemsOfModule{--}(m_k))) --
		`subseteq` dom{--}(itemsOfModule{--}(m_k)) == dom{--}(m_set#(m_k)))
\end{haskell}

Further, for every variable "nu_j" bound in this set to a definition of an Etude function item “"nu_j =: FUNI (FUN (nu_set_j) {-DOT-} (tau_j))"\!”, the corresponding substitution function "m_set#(m_k)" must map "nu_j" to a well-formed atom of the form “"IMM ({-sem-}(n_j)) (FG`FMT`standardEncoding)"\!” for some integer value "n_j":

\begin{haskell}
wfModuleJ2 :: forall m_set, m_k, nu_j, nu_set_j, tau_j =>
	WfJ{--}(m_set) ->
	{-sem-}(m_k @@ dom{--}(m_set) && itemsOfModule{--}(m_k)#(nu_j) == {-sem-}(FUNI (FUN (nu_set_j) {-DOT-} (tau_j)))) ->
	WfJ{-sem-}(IMM ({-sem-}(atomToFunction{--}(m_set#(m_k)#(nu_j)))) (FG`FMT`standardEncoding))
\end{haskell}

in which "atomToFunction{--}(m_set#(m_k)#(nu_j))" abstracts over the concrete value of "n_j" using the following implementation-defined Haskell construction:

\ReindexTerm		{atomToFunction}

\begin{haskell}
atomToFunction {-[*]-} :: (Ord nu) => AtomT nu -> Integer
\end{haskell}

More so, every variable "nu_j" that is bound in one of the supplied modules "m_k" to a function definition of the form “"nu_j =: FUNI (FUN (nu_set_j) {-DOT-} (tau_j))"\!” must be renamed by "m_set#(m_k)" to an integer value "n_j" unique across the entire module collection "m_set":

\begin{haskell}
wfModuleJ3 :: forall m_set, n_set =>
	WfJ{--}(m_set) ->
	{-sem-}(n_set == [ atomToFunction{--}(m_set#(m_k)#(nu_j)) | m_k <- dom{--}(m_set), {-sem-}(nu_j =: FUNI (FUN (nu_set_j) {-DOT-} (tau_j))) <- itemsOfModule{--}(m_k) ]) -> 
	{-sem-}(lengthOfList{--}(n_set) == cardinality (listToSet n_set))
\end{haskell}

Similarly, for every variable "nu_j" bound in this set to a definition of an Etude object specification “"nu_j =: OBJI (delta_set_j) {-OF-} (xi_set_j)"”, the corresponding substitution function "m_set#(m_k)" must map "nu_j" to a well-formed atom of the form “"IMM ({-sem-}(n_j)) (OG`FMT`standardEncoding)"\!” for some integer value "n_j":

\begin{haskell}
wfModuleJ4 :: forall m_set, m_k, nu_j, delta_set_j =>
	WfJ{--}(m_set) ->
	{-sem-}(m_k @@ dom{--}(m_set) && itemsOfModule{--}(m_k)#(nu_j) == {-sem-}(OBJI (delta_set_j) {-OF-} (xi_set_j))) ->
	WfJ{-sem-}(IMM ({-sem-}(atomToObject{--}(m_set#(m_k)#(nu_j)))) (OG`FMT`standardEncoding))
\end{haskell}

in which, once again, "atomToObject{--}(m_set#(m_k)#(nu_j))" abstracts over the concrete value of "n_j" using the following implementation-defined Haskell construction:

\ReindexTerm		{atomToObject}

\begin{haskell}
atomToObject {-[*]-} :: (Ord nu) => AtomT nu -> Integer
\end{haskell}

As for function definitions, the list of all such Etude variables must constitute a set of integer values unique across the entire module collection:

\begin{haskell}
wfModuleJ5 :: forall m_set => 
	WfJ{--}(m_set) ->
	{-sem-}(n_set == [ atomToObject{--}(m_set#(m_k)#(nu_j)) | m_k <- dom{--}(m_set), {-sem-}(nu_j =: OBJI (delta_set_j) {-OF-} (xi_set_j)) <- itemsOfModule{--}(m_k) ]) ->
	{-sem-}(lengthOfList{--}(n_set) == cardinality (listToSet n_set))
\end{haskell}

More so, for every integer "n_j" associated with an object item “"nu_j =: OBJI (delta_set_j) {-OF-} (xi_set_j)"\,”, the precise location of every envelope datum "{-hv-}(n_i {-hv-}, phi_i {-hv-}, mu_set_i {-hv-}, alpha_i) @@ delta_set_j" must be representable as an atom of the form “"OP2 ({-hv-}(OP1 (CNV (objectFormat{--}(phi_i))) (OG`FMT`standardEncoding) {--}{-hv-}(IMM ({-sem-}(n_j)) (OG`FMT`standardEncoding)))) (ADD) (objectFormat{--}(phi_i)) (IMM ({-sem-}(n_i)) (ZG`FMT`standardEncoding))"\!”:

\begin{haskell}
wfModuleJ6 :: forall m_set, m_k, nu_j, delta_set_j, n_i, phi_i, mu_set_i, alpha_i =>
	WfJ{--}(m_set) ->
	{-sem-}(m_k @@ dom{--}(m_set) && --
		 itemsOfModule{--}(m_k)#(nu_j) == {-sem-}(OBJI (delta_set_j) {-OF-} (xi_set_j)) && --
		 {-hv-}(n_i {-hv-}, phi_i {-hv-}, mu_set_i {-hv-}, alpha_i) @@ delta_set_j) ->
	WfJ{-sem-}(OP2 ({-hv-}(OP1 (CNV {-sem-}(objectFormat{--}(phi_i))) (OG`FMT`standardEncoding) --
			{--}{-hv-}(IMM ({-sem-}(atomToObject{--}(m_set#(m_k)#(nu_j)))) (OG`FMT`standardEncoding)))) --
				(ADD) (objectFormat{--}(phi_i)) (IMM ({-sem-}(n_i)) (ZG`FMT`standardEncoding)))
\end{haskell}

Finally, across the entire module collection, every variable "nu_j" bound to an imported item of the form “"IMPI (x_j)"” must be renamed to the same value as that associated with the corresponding export definition found in some other module in the program. Formally:

\begin{haskell}
wfModuleJ7 :: forall m_set, m_k, nu_j, x_j =>
	WfJ{--}(m_set) ->
	{-sem-}(m_k @@ dom{--}(m_set) && itemsOfModule{--}(m_k)#(nu_j) == {-sem-}(IMPI (x_j))) ->
	{-sem-}(m_set#(m_k)#(nu_j) == exportsOfModules{--}(m_set)#(x_j))
\end{haskell}

where the notation “"exportsOfModules{--}(m_set)"” constructs a finite map from exported definition names to their respective atomic values as follows:

\BeginHaskellToken	{exportsOfModules}			{Open}	{«\superimpose{\raise.5ex\hbox{\macron{}}}{\xi}»}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{ModulesT nu}
  \HaskellResult							{(MapT (String) (AtomT nu))}
\EndHaskellToken

\IndexTerm		{exportsOfModules}				{\xi- modules (strings)}

\begin{code}
exportsOfModules {-[*]-} 		:: (Ord nu) => (MapT (ModuleT nu) ((MapT (nu) (AtomT nu)))) -> (MapT (String) (AtomT nu))

exportsOfModules {-sem-}(m_set) 	= listToMap [ {-raw-}(x_k =: {-sem-}(_S_k#(nu_k))) --
							| {-raw-}(m_k {-:-}, _S_k) <- mapToList m_set, {-raw-}(x_k {-=-}, nu_k) <- mapToList (exportsOfModule{--}(m_k)) ]
\end{code}

Intuitively, for every module "m_k" that is associated in "m_set" with the substitution function "_S_k" and for every variable "nu_k" exported from that module as string "x_k", the above "exportsOfModule{--}(m_set)" construction includes a binding of the string "x_k" to the atom associated with "nu_k" by the substitution function "_S_k".

Last but not least, every well-formed collection of Etude modules "m_set" must represent a valid Etude program:

\begin{haskell}
wfModuleJ8 :: forall m_set => WfJ{--}(m_set) -> WfJ{--}(linkModules{--}(m_set))
\end{haskell}

derived from the included module list by the following simple process:

\BeginHaskellToken	{linkModules}				{Open}	{\ZapfChancery{L}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{(MapT (ModuleT nu) ((MapT (nu) (AtomT nu))))}
  \HaskellResult							{({-,,,-}FEnvT nu, OEnvT {->-}, TermT nu)}
\EndHaskellToken

\IndexTerm		{linkModules}					{L modules (f_env o_env term)}

\begin{haskell}
linkModules {-[*]-} :: (Ord nu) => (MapT (ModuleT nu) ((MapT (nu) (AtomT nu)))) -> ({-,,,-}FEnvT nu, OEnvT {->-}, TermT nu)

linkModules {-sem-}(m_set) ---
			= {-sem-}(_Lambda0, _Delta0 {->-}, LET (beta_set0) (RET []))
 where	_Lambda0	= listToMap [ {-sem-}({-sem-}(atomToObject{--}(_S_k#(nu_j))) =: FUN (nu_set_j) {-DOT-} ({-sem-}(tau_j `substTerm` _S_k)))
			{-quad-}{-quad-}	| {-raw-}(m_k {-:-}, _S_k) <- mapToList m_set, --
						  {-sem-}(nu_j {-=-}, FUNI (FUN (nu_set_j) {-DOT-} (tau_j))) <- mapToList (itemsOfModule{--}(m_k)) ]
 	_Delta0		= linkOEnv (listToMap [ {-raw-}(atomToFunction{--}(_S_k#(nu_j)) {-:-}, {-sem-}(OBJI (delta_set_j) {-OF-} (xi_set_j)) `substItem` _S_k)
			{-quad-}{-quad-}	| {-raw-}(m_k {-:-}, _S_k) <- mapToList m_set, --
	 					  {-sem-}(nu_j {-=-}, OBJI (delta_set_j) {-OF-} (xi_set_j)) <- mapToList (itemsOfModule{--}(m_k)) ])
 	beta_set0	= [ {-sem-}({-hv-}[] =: {-sem-}(termOfModule{--}(m_k) `substTerm` _S_k)) --
				| {-raw-}(m_k {-:-}, _S_k) <- mapToList m_set ]
\end{haskell}

In other words, the program constructed by the linking process evaluates, in an unspecified order, the initialiser terms of all the modules in the collection. Every one of these initialiser terms "termOfModule{--}(m_k)" must represent a monadic construct that delivers an empty set of Etude atoms “"RET []"”, so that the entire program constitutes a simple term of the form “"LET (beta_set0) (RET [])"”. Its initial function environment "_Lambda0" is constructed from all the function definitions found anywhere across the entire collection, while the initial object environment "_Delta0" is obtained from the set of suitably-renamed versions of all object specifications in the individual item lists "itemsOfModule{--}(m_k)" using the following unspecified construction:

\BeginHaskellToken	{linkOEnv}				{Open}	{\ZapfChancery{L\sub{\Delta}}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{(MapT (Integer) (ItemT nu))}
  \HaskellResult							{OEnvT}
\EndHaskellToken

\IndexTerm		{linkOEnv}					{L_\Delta integers_items (o_env)}

\begin{haskell}
linkOEnv {-[*]-} :: (Ord nu) => (MapT (Integer) (ItemT nu)) -> OEnvT
\end{haskell}

such that, for every "{-hv-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k)" and "{-hv-}(n_k {-hv-}, phi_k {-hv-}, mu_set_k {-hv-}, alpha_k)" included in the specification of an object mapped in "iota_set" at some address "k", the initial object environment "linkOEnv{--}(iota_set)" includes a binding of "(k + n_k, phi_k)" to the atom "alpha_k" under the set of access attributes "mu_set_k". Formally:

\NewHaskellToken	{linkOEnvJ}				{Ord}	{\TKeyword{link}}
\NewHaskellDescription	{linkOEnvJ}					{Validity of linked Etude programs}
\IndexTheorem		{linkOEnvJ}					{LINK Validity of linked Etude programs}

\begin{haskell}
linkOEnvJ1 :: forall iota_set, k, n_j, phi_j, mu_set_j, alpha_j =>
	WfJ{--}(linkOEnv{--}(iota_set)) ->
	{-sem-}(iota_set#(k) == {-sem-}(OBJI (delta_set_k) {-OF-} (xi_set_k))) ->
	{-sem-}({-hv-}(n_j {-hv-}, phi_j {-hv-}, mu_set_j) @@ xi_set_k || {-hv-}(n_j {-hv-}, phi_j {-hv-}, mu_set_j {-hv-}, alpha_j) @@ delta_set_k) ->
	{-sem-}({-hv-}(k + n_j, phi_j, mu_set_j) `inEnvelope` envelopeOfOEnv{--}(linkOEnv{--}(iota_set)))

linkOEnvJ2 :: forall iota_set, k, n_j, phi_j, mu_set_j, alpha_j => 
	WfJ{--}(linkOEnv{--}(iota_set)) ->
	{-sem-}(iota_set#(k) == {-sem-}(OBJI (delta_set_k) {-OF-} (xi_set_k))) ->
	{-sem-}({-hv-}(n_j {-hv-}, phi_j {-hv-}, mu_set_j {-hv-}, alpha_j) @@ delta_set_k) ->
	{-sem-}(atomOfOEnv{--}(linkOEnv{--}(iota_set) {->-}, k + n_j, phi_j)) `EqvJ` {-sem-}(alpha_j))
\end{haskell}

The reader should observe that the structure of Etude data specification sets allows programs to place multiple data elements at overlapping address space regions within a single memory-resident object, with an understanding that the meaning of such constructions is well-defined only if the binary representations of the respective atomic values assigned to any such elements are in agreement as to the object's initial \hbox{memory image.}

% With this definition, we have now completed the generic treatment of the Etude language and are well-prepared to tackle the actual formal specification of Standard C. 

For a representative sample of typical Etude programs, the reader is referred to the number of examples featured later in Sections \ref{C/Overview}, \ref{C/Example} and \ref{MMIX/Example}.

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
