\set{capture}{hidden}
\begin{code}

module Lambda.State (
	fvOfFunction,
	substFunction,
	sizeOfEnvelope,
	shiftEnvelope,
	inEnvelope,
	purgeEnvelope,
) where

import Additional
import Lambda.Syntax
import Lambda.Atoms
import Lambda.Terms
import {-# SOURCE #-} Lambda.Arch

infixl 7 `purgeEnvelope`
infixl 6 `shiftEnvelope`
infix  4 `inEnvelope`

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/State}
\set{title}{State of the Matter}
\begin{section}

Atoms are not the only kind of primitive entities manipulate by Etude. For better or worse, most modern computational hardware is explicitly geared towards stateful operations on mutable objects, whose individual meanings may, contrary to all sensibilities of the pure lambda calculus, change gradually throughout execution of a program.

The simplest and most popular way of modelling the dynamic semantics of such rogue primitives is through the notion of an «execution state», an abstract object that is implicitly received, modified and returned by every function in the program. In Etude, the execution state consists of two distinct components known individually as the «function» and «object environments» which, for conciseness, are generally represented by the Greek letters "_Lambda" and "_Delta", or collectively as a tuple of the form “"(_Lambda, _Delta)"”.

Intuitively, a function environment "_Lambda" represents an oracle that encodes a potentially unbounded set of Etude functions using a finite number of distinct atomic values, by mapping every abstraction "lambda" that actually appears within a program's definition to a predetermined atom "_Lambda#(lambda)", so that every every first-class entity in the Etude language is always identified with a suitable atomic value, which greatly simplifies definition of a bijective mapping between Etude programs and executable machine instruction sequences exemplified later in Chapter \ref{MMIX}. Formally, the function environment constitutes a mapping of atoms to syntactic representations of lambda abstractions as described by the following Haskell data types:

\IndexType		{FEnvT}						{f_env \nu (\tstar)}

\IndexType		{FunctionT}					{function \nu (\tstar)}
\IndexCons		{FUN}						{\lambda . parameters term (function)}

\IndexType		{ParametersT}					{parameters \nu (\tstar)}

\begin{syntax}
\InsertHaskell{FEnvT, \par, FunctionT, \par, ParametersT}
\end{syntax}

in which the notation “"MapT _A _B"\,” represents a polymorphic Haskell type that describes an extendible partial function from "_A" to "_B", or a mapping from a finite set of Haskell values of type "_A" to values of type "_B". It is implemented separately in Appendix \ref{Additional} with the help of the existing standard Haskell libraries. In particular, the above specification of \hbox{"FEnvT nu"} defines the function encoding oracle as a mapping of integers to Etude abstractions of the form “"FUN [x1, x2 .. x_n] {-DOT-} (tau)"\,”, where the structure of the abstraction body "tau" is depicted by the data type “"TermT nu"” scrutinised later in Section \ref{Lambda/Terms}. In general, the "_Lambda" oracle is constructed by the system's linker in an unspecified manner by enumerating all functions defined within the individual program modules, although we could easily extend Etude to accommodate more elaborate instruction set architectures which permit dynamic introduction of new functions during the actual evaluation of a program.

In a well-formed function environment "_Lambda", every integer "n @@ dom{--}(_Lambda)" is mapped to a well-formed closed Etude function, with 0 generally excluded from "_Lambda"'s domain. Formally, these requirements can be captured concisely by the following property type:

\BeginHaskellToken	{WfFEnvJ}				{Open}	{\TKeyword{wf}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{FEnvT nu}
  \HaskellResult							{Prop}
\EndHaskellToken

\NewHaskellToken	{WfFEnvC}				{Ord}	{\TKeyword{wf}\sub{\Lambda}}
\NewHaskellDescription	{WfFEnvC}					{Validity of function environments}

\IndexProp		{WfFEnvJ}					{WF f_env (\pstar)}
\IndexAxiom		{WfFEnvC}					{WF_\Lambda Validity of Etude function environments}

\begin{haskell}
data WfJ {-[*]-} :: forall nu => FEnvT nu -> Prop

 where	WfFEnvC :: forall _Lambda =>	{-sem-}(0 /@ dom{--}(_Lambda)) ->
					(forall n -> {-sem-}(n @@ dom{--}(_Lambda)) -> WfJ{-sem-}(_Lambda#(n))) ->
					(forall n -> {-sem-}(n @@ dom{--}(_Lambda)) -> {-sem-}(fvOfFunction{--}(_Lambda#(n)) == emptySet)) ->
					WfJ{--}(_Lambda)
\end{haskell}

where the well-formedness of individual functions is determined inductively from well-formedness of their parameter lists as follows:

\BeginHaskellToken	{WfFunctionJ}				{Open}	{\TKeyword{wf}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{FunctionT nu}
  \HaskellResult							{Prop}
\EndHaskellToken

\NewHaskellToken	{WfFunctionC}				{Ord}	{\TKeyword{wf}\sub{\lambda}}
\NewHaskellDescription	{WfFunctionC}					{Validity of Etude functions}

\IndexProp		{WfFunctionJ}					{WF function (\pstar)}
\IndexAxiom		{WfFunctionC}					{WF_\lambda function}

\begin{haskell}
data WfFunctionJ {-[*]-} :: forall nu => FunctionT nu -> Prop

 where	WfFunctionC :: forall nu_set, tau => WfJ{--}(nu_set) -> WfJ{-sem-}(FUN (nu_set) {-DOT-} (tau) {-,-})
\end{haskell}

observing that the actual body term "tau" need not always be well-formed in such entities. A list of variables represents a well-formed parameter list if and only if it actually constitutes a well-formed set, i.e., if its length is equal to the cardinality of a set constructed from these variable names. Formally:

\BeginHaskellToken	{WfParametersJ}				{Open}	{\TKeyword{wf}}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{ParametersT nu}
  \HaskellResult							{Prop}
\EndHaskellToken

\NewHaskellToken	{WfParametersC}				{Ord}	{\TKeyword{wf}\sub{\nu}}
\NewHaskellDescription	{WfParametersC}					{Validity of Etude parameter lists}

\IndexProp		{WfParametersJ}					{WF parameters}
\IndexAxiom		{WfParametersC}					{WF_\nu Validity of Etude parameter lists}

\begin{haskell}
data WfParametersJ {-[*]-} :: forall nu => ParametersT nu -> Prop

 where	WfParametersC :: forall nu_set => {-sem-}(lengthOfList{--}(nu_set) == cardinality(listToSet nu_set)) -> WfJ{-sem-}(nu_set {-,-})
\end{haskell}

Finally, the set of variables appearing free within an Etude function includes every free variable from its body term, excluding any identifiers found directly in the function's parameter list. In Haskell, this is captured by the following natural definition:

\BeginHaskellToken	{fvOfFunction}				{Open}	{FV}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{FunctionT nu}
  \HaskellResult							{SetT nu}
\EndHaskellToken

\IndexTerm		{fvOfFunction}					{FV function (\nu s)}

\begin{code}
fvOfFunction {-[*]-} :: (Ord nu) => FunctionT nu -> SetT nu

fvOfFunction {-sem-}(FUN (nu_set) {-DOT-} (tau)) = fvOfTerm{--}(tau) `difference` listToSet nu_set
\end{code}

The structure and algebraic properties of Etude terms appearing in function bodies is discussed later in Section \ref{Lambda/Terms}. For now, we will only characterise them by the simple substitution construct defined on all well-formed Etude functions as follows:

\BeginHaskellToken	{substFunction}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{FunctionT nu}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{(MapT (nu) (AtomT nu))}
  \HaskellResult							{FunctionT nu}
\EndHaskellToken

\NewHaskellToken	{`substFunction`}			{Op}	{\slash}
\NewHaskellToken	{{-/-}}					{Op}	{\slash}

\IndexTerm		{substFunction}					{\slash function (function)}

\begin{code}
substFunction {-[*]-} {-/-} {-[*]-} :: (Ord nu) => FunctionT nu -> (MapT (nu) (AtomT nu)) -> FunctionT nu

substFunction {-sem-}(FUN (nu_set) {-DOT-} (tau)) {-/-} {-sem-}(_S) = {-sem-}(FUN (nu_set) {-DOT-} ({-sem-}(tau `substTerm` (_S `mapSetDifference` listToSet nu_set))))
\end{code}

Unlike the function environment described above, the structure and properties of the «object environment» "_Delta" threaded throughout the program's execution as part of its evaluation state are left mostly-unspecified in the generic fragment of Etude's algebraic semantics, since their complete implementation requires a great deal of knowledge about the precise operational behaviour of all memory access operations supported by the underlying instruction set architecture. Intuitively, an object environment describes the address space layout of an Etude program under execution and, further, depicts the current value of every memory-resident «mutable object» manipulated by monadic Etude terms in accordance with the semantic rules defined later in Section \ref{Lambda/Terms}. 

Regardless of any particular implementation-specific memory organisation regime, a generic abstract model of an object environment "_Delta" can be described by a set "envelopeOfOEnv{--}(_Delta)" known as the environment's «envelope», a list "stackOfOEnv{--}(_Delta)" known as its «data stack» and a partial function "atomOfOEnv{--}(_Delta)" that depicts bindings of the individual memory-resident objects to their current contents. Formally, every envelope "xi_set" consists of a set of tuples "{-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k)", in which the integer "sigma_k" represents some concrete memory location, the format "phi_k" describes the desired interpretation of data stored at that address and "mu_set_k" depicts a set of zero or more «memory access attributes» that further constrain the range of operations permitted on that address space region. In Haskell, the type of every Etude envelope can be defined as follows:

\IndexType		{EnvelopeT}					{envelope (\tstar)}
\IndexType		{EnvelopeElementT}				{envelope_element (\tstar)}

\IndexType		{AttrT}						{attr (\tstar)}
\IndexCons		{CM}						{C (attr)}
\IndexCons		{VM}						{V (attr)}

\begin{syntax}
\InsertHaskell		{EnvelopeT, \par, EnvelopeElementT, \par, AttrT}
\end{syntax}

Intuitively, each envelope element "{-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k)" characterises all operational properties of the address space region "[sigma_k .. sigma_k + sizeOfFormat{--}(phi_k) - 1]", observing that entries found in distinct envelopes may, on occasions, refer to overlapping regions in the program's \hbox{memory image.}

The set of access attributes "mu_set" associated with each envelope element provides additional information about the precise semantics of the corresponding memory location. In particular, portable Etude programs are universally prohibited from modifying values of objects tagged with the «constant attribute» “"CM"” and, further, may never assume any particular behaviour for those object which, in the program's address space, are tagged with the «volatile attribute» “"VM"”, since any such addresses may correspond to «shared memory regions» associated with memory-mapped registers and buffers of various hardware devices, whose precise semantics remain outside of the program's explicit control.

In every well-formed envelope, each element "{-hv-}(sigma {-hv-}, phi {-hv-}, mu_set)" must entail the validity of the format "phi", the set of attributes "mu_set" and the object atom “"IMM (sigma) ({-sem-}(objectFormat{--}(phi)))"\!”, so that "sigma" must, in effect, represent a well-formed pointer to some "phi"-formatted memory-resident object. Formally:

\BeginHaskellToken	{WfEnvelopeElementJ}			{Open}	{\TKeyword{wf}}
  \HaskellArgument							{EnvelopeElementT}
  \HaskellResult							{Prop}
\EndHaskellToken

\IndexProp		{WfEnvelopeElementJ}				{WF envelope_element (\pstar)}

\NewHaskellToken	{WfEnvelopeElementC}			{Ord}	{\TKeyword{wf}}
\NewHaskellDescription	{WfEnvelopeElementC}				{Validity of envelope elements}
\IndexAxiom		{WfEnvelopeElementC}				{WF Validity of envelope elements}

\begin{haskell}
data WfEnvelopeElementJ {-[*]-} :: EnvelopeElementT -> Prop

 where	WfEnvelopeElementC 	:: forall sigma, phi, mu_set => --
					WfJ{--}(phi) -> --
					WfJ{--}(mu_set) -> --
					WfJ{-sem-}(IMM (sigma) ({-sem-}(objectFormat{--}(phi)))) -> --
					WfEnvelopeElementJ{-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set))
\end{haskell}

in which the notation “"objectFormat{--}(phi)"” provides a mapping of each Etude format "phi" to a designated object format suitable for depiction of address values for "phi"-formatted objects:

\ReindexTerm		{objectFormat}

\begin{haskell}
objectFormat {-[*]-} :: FormatT -> FormatT
\end{haskell}

The precise size "sizeOfEnvelope{--}(xi_set)" of an entire memory region described by a given envelope "xi_set" is defined as a difference between the greatest and smallest address value corresponding to any of its individual envelope elements, or zero if "xi_set" represents an empty set. Formally:

\BeginHaskellToken	{sizeOfEnvelope}			{Open}	{\ZapfChancery{S}}
  \HaskellArgument							{EnvelopeT}
  \HaskellResult							{Integer}
\EndHaskellToken

\IndexTerm		{sizeOfEnvelope}				{S envelope (integer)}

\begin{code}
sizeOfEnvelope {-[*]-} :: EnvelopeT -> Integer

sizeOfEnvelope {-sem-}(xi_set) ---
  | (xi_set == emptySet)	= 0
  | (xi_set /= emptySet)	= maximum [ sigma_k + sizeOfFormat{--}(phi_k) | {-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) <- setToList xi_set {-,,-} ] --
					- minimum [ sigma_k | {-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) <- setToList xi_set {-,,-} ]
\end{code}

A given envelope "xi_set" may be also «shifted» into a particular memory location "sigma" by adding "n - sigma0" to the the offset "sigma_k" of every envelope element "{-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) @@ xi_set", where "sigma0" represents the least address of any element in "xi_set". In Haskell, this simple construction can be executed as follows:

\BeginHaskellToken	{shiftEnvelope}				{Void}	{}
  \HaskellArgument							{EnvelopeT}
  \HaskellToken		{{-(+)-}}				{Bin}	{\oplus}
  \HaskellArgument							{Integer}
  \HaskellResult							{EnvelopeT}
\EndHaskellToken

\NewHaskellToken	{{-(+)-}}				{Bin}	{\oplus}
\NewHaskellToken	{`shiftEnvelope`}			{Bin}	{\oplus}

\IndexTerm		{shiftEnvelope}					{\oplus envelope integer (envelope)}

\begin{code}
shiftEnvelope {-[*]-} {-(+)-} {-[*]-} :: EnvelopeT -> Integer -> EnvelopeT

shiftEnvelope {-sem-}(xi_set) {-(+)-} {-sem-}(n) ---
		= listToSet [ {-hv-}(sigma_k + n - sigma0 {-hv-}, phi_k {-hv-}, mu_set_k) | {-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) <- setToList xi_set {-,,-} ]
 where	sigma0	= minimum [ sigma_k | {-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) <- setToList xi_set {-,,-} ]

\end{code}

Finally, an envelope "xi_set" may be purged of any elements that overlap a given memory region "[sigma_i .. sigma_f - 1]" as follows:

\BeginHaskellToken	{purgeEnvelope}				{Void}	{}
  \HaskellArgument							{EnvelopeT}
  \HaskellToken		{{-\\-}}				{Bin}	{\backslash}
  \HaskellArgument							{(Integer, Integer)}
  \HaskellResult							{EnvelopeT}
\EndHaskellToken

\NewHaskellToken	{{-\\-}}				{Op}	{\backslash}
\NewHaskellToken	{`purgeEnvelope`}			{Op}	{\backslash}

\IndexTerm		{purgeEnvelope}					{\backslash envelope integer integer (envelope)}

\begin{code}
purgeEnvelope {-[*]-} {-\-} {-[*]-} :: EnvelopeT -> (Integer, Integer) -> EnvelopeT

purgeEnvelope {-sem-}(xi_set) {-\-} {-sem-}(sigma_i, sigma_f) ---
	= listToSet [ {-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) --
			| {-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) <- setToList xi_set, --
			  sigma_k + sizeOfFormat{--}(phi_k) <= sigma_i || sigma_k >= sigma_f ]
\end{code}

The use of access attribute tags in individual envelope elements introduces a number of challenges that are best met by formalising the notion of an «element accessibility» as the binary relation “"xi_k `inEnvelope` xi_set"\,”. In particular, a given element "{-hv-}(sigma {-hv-}, phi {-hv-}, mu_set)" is said to be «accessible» in "xi_set" if and only if that envelope contains at least one element of the form "{-hv-}(sigma {-hv-}, phi {-hv-}, mu_set')" such that "mu_set' `subseteq` mu_set". Intuitively, this definition ensures that every memory access operation performed by an Etude program must always specify at least the complete attribute set associated with the targeted object in the program's address space. Formally, this operator is defined in Haskell as follows:

\BeginHaskellToken	{inEnvelope}				{Void}	{}
  \HaskellArgument							{EnvelopeElementT}
  \HaskellToken		{{-inEnvelope-}}			{Bin}	{\InEnvelopeSymbol}
  \HaskellArgument							{EnvelopeT}
  \HaskellResult							{Bool}
\EndHaskellToken

\NewMacro		{InEnvelopeSymbol}				{\in\sub{\SmallCaps{a}}}
\NewHaskellToken	{{-inEnvelope-}}			{Bin}	{\InEnvelopeSymbol}
\NewHaskellToken	{`inEnvelope`}				{Bin}	{\InEnvelopeSymbol}

\IndexTerm		{inEnvelope}					{\in_A envelope_element envelope (bool)}

\begin{code}
inEnvelope {-[*]-} {-inEnvelope-} {-[*]-} :: EnvelopeElementT -> EnvelopeT -> Bool

inEnvelope {-sem-}(sigma, phi, mu_set) {-inEnvelope-} {-sem-}(xi_set) ---
	= or [ sigma_k == sigma && phi_k == phi && mu_set_k `subseteq` mu_set | {-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) <- setToList xi_set {-,,-} ]
\end{code}

Besides the address space envelope "envelopeOfOEnv{--}(_Delta)" mentioned earlier, every well-formed object environment also includes a second envelope "initOfOEnv{--}(_Delta)" whose elements describe all currently uninitialised memory objects introduced by the program. Formally, both notations are given a formal mandate by the following pair of implementation-defined Haskell functions:

\ReindexTerm		{envelopeOfOEnv}
\ReindexTerm		{initOfOEnv}

\begin{haskell}
envelopeOfOEnv {-[*]-}, initOfOEnv {-[*]-} :: OEnvT -> EnvelopeT
\end{haskell}

Further, in order to facilitate establishment of the linear correctness property for our translation of Etude into the MMIX architecture in Chapter \ref{MMIX}, it is necessary to restrict well-formed Etude programs to certain predictable patterns of memory allocations. To this end, our abstract model of object environments includes an additional «stack» component "stackOfOEnv{--}(_Delta)" that, intuitively, contains the complete history of all individual envelopes introduced into "envelopeOfOEnv{--}(_Delta)" by the program as a list of «stack frames», every one of which is represented simply by an envelope tagged with its allotted location within "envelopeOfOEnv{--}(_Delta)". Formally, the structure of such a stack can be depicted by the following pair of Haskell data types:

\IndexType		{StackT}					{stack (\tstar)}
\IndexType		{StackFrameT}					{stack_frame (\tstar)}

\begin{syntax}
\InsertHaskell		{StackT, \par, StackFrameT}
\end{syntax}

so that the notation “"stackOfOEnv{--}(_Delta)"” can be formalised by some implementation-defined construction with the following type signature:

\ReindexTerm		{stackOfOEnv}

\begin{haskell}
stackOfOEnv {-,-} {-[*]-} :: OEnvT -> StackT
\end{haskell}

Finally, the actual content of all initialised address space regions within a given object environment "_Delta" can be depicted by a partial function "atomOfOEnv{--}(_Delta {->-}, sigma, phi)" that maps every valid pair of the form “"(sigma, phi)"” to its concrete atomic value as follows:

\ReindexTerm		{atomOfOEnv}

\begin{haskell}
atomOfOEnv {-[*]-} :: {- (Ord nu) => -} (OEnvT {->-}, Integer, FormatT) -> AtomT nu
\end{haskell}

The precise behaviour of this function will be, for most part, left unspecified in the generic fragment of the Etude language. Intuitively, a given memory object "(sigma, phi)" is said to be «populated» in "_Delta" if and only if "atomOfOEnv{--}(_Delta {->-}, sigma, phi)" represents a well-formed Etude atom, in which case that construct also determines the object's current content in "_Delta".

Although, in general, we leave formulation of the precise validity criteria for object environments at the mercy of individual Etude implementations, every well-formed "_Delta" must always specify a valid envelope "envelopeOfOEnv{--}(_Delta)" that includes at least all of the envelope elements from "initOfOEnv{--}(_Delta)", together with any envelopes that are mentioned by the individual stack frames of "stackOfOEnv{--}(_Delta)". Formally:

\ReindexTheorem		{envelopeOfOEnvJ}
\ReindexTheorem		{initOfOEnvJ}
\ReindexTheorem		{stackOfOEnvJ}

\begin{haskell}
envelopeOfOEnvJ 	:: forall _Delta => WfJ{--}(_Delta) -> WfJ{-sem-}(envelopeOfOEnv{--}(_Delta))
initOfOEnvJ 		:: forall _Delta => WfJ{--}(_Delta) -> {-sem-}(initOfOEnv{--}(_Delta) `subseteq` envelopeOfOEnv{--}(_Delta))
stackOfOEnvJ 		:: forall _Delta => WfJ{--}(_Delta) -> {-sem-}(unions [ xi_set_k `shiftEnvelope` sigma_k | (sigma_k {-hv-}, xi_set_k) <- stackOfOEnv{--}(_Delta) ] --
									`subseteq` envelopeOfOEnv{--}(_Delta))
\end{haskell}

The actual introduction of a new envelope "xi_set" into an existing object environment "_Delta" is modelled by an «environment extension operation» of the form “"_Delta `extendOEnv` xi_set"\,”, while the dual «environment contraction» “"_Delta `contractOEnv` xi_set"\,” can be used to purge such an envelope from "_Delta". In the algebraic semantics of Etude, the precise behaviour of these operations is left open to further specification by individual Etude implementations, which should always assign concrete definition to the following pair of Haskell functions:

\ReindexTerm		{extendOEnv}
\ReindexTerm		{contractOEnv}

\begin{haskell}
extendOEnv 	{-[*]-} {-/-}  {-[*]-} :: OEnvT -> EnvelopeT -> OEnvT
contractOEnv 	{-[*]-} {-\-}  {-[*]-} :: OEnvT -> EnvelopeT -> OEnvT
\end{haskell}

In all cases, however, a well-formed extension "_Delta'" of some existing environment "_Delta" by an envelope "xi_set" introduces a new stack frame "{-hv-}(sigma_c {-hv-}, xi_set)" into "stackOfOEnv{--}(_Delta')", whereby it is inserted at the front of the existing stack frame list "stackOfOEnv{--}(_Delta)". Further, the new envelope is included in both "envelopeOfOEnv{--}(_Delta')" and "initOfOEnv{--}(_Delta')", after shifting it into place by its concrete location "sigma_c" chosen in some unspecified manner by the underlying instruction set architecture. Formally, these constraints on the behaviours of the environment extension operator are captured by the following three theorems:

\ReindexTheorem		{extendOEnvStackJ}
\ReindexTheorem		{extendOEnvEnvelopeJ}
\ReindexTheorem		{extendOEnvInitJ}

\begin{haskell}
extendOEnvStackJ	:: forall _Delta, xi_set => --
	WfJ{--}(_Delta) -> WfJ{--}(_Delta `extendOEnv` xi_set) -> --
	{-sem-}(stackOfOEnv{--}(_Delta `extendOEnv` xi_set) == {-raw-}[{-hv-}(newInOEnv{--}(_Delta {->-}, xi_set) {-hv-}, xi_set)] ++ stackOfOEnv{--}(_Delta))
extendOEnvEnvelopeJ	:: forall _Delta, xi_set => --
	WfJ{--}(_Delta) -> WfJ{--}(_Delta `extendOEnv` xi_set) -> --
	{-sem-}(envelopeOfOEnv{--}(_Delta `extendOEnv` xi_set) == envelopeOfOEnv{--}(_Delta) `union` (xi_set `shiftEnvelope` newInOEnv{--}(_Delta {->-}, xi_set)))
extendOEnvInitJ		:: forall _Delta, xi_set => --
	WfJ{--}(_Delta) -> WfJ{--}(_Delta `extendOEnv` xi_set) -> --
	{-sem-}(initOfOEnv{--}(_Delta `extendOEnv` xi_set) == initOfOEnv{--}(_Delta) `union` (xi_set `shiftEnvelope` newInOEnv{--}(_Delta {->-}, xi_set)))
\end{haskell}

in which the actual location of "xi_set" within the resulting address space of "_Delta `extendOEnv` xi_set" is represented by the notation “"newInOEnv{--}(_Delta {->-}, xi_set)"”, mandated by the following implementation-defined Haskell function:

\ReindexTerm		{newInOEnv}

\begin{haskell}
newInOEnv {-[*]-} :: (OEnvT {->-}, EnvelopeT) -> Integer
\end{haskell}

In every other respects, the environment extension preserves all other formal properties of the original environment "_Delta". In particular, all atomic bindings from "_Delta" that do not overlap the newly-introduced envelope "xi_set" always remain equivalent to their corresponding meanings in the extended environment "_Delta `extendOEnv` xi_set", provided that both "_Delta" and "_Delta `extendOEnv` xi_set" are deemed well-formed according to the implementation-defined rules outlined earlier:

\ReindexTheorem		{extendOEnvAtomsJ}

\begin{haskell}
extendOEnvAtomsJ 	:: forall _Delta, xi_set, sigma_k, phi_k, mu_set_k =>
	WfJ{--}(_Delta) -> --
	WfJ{--}(_Delta `extendOEnv` xi_set) -> --
	{-sem-}({-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) @@ envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(atomOfOEnv{--}(_Delta `extendOEnv` xi_set {->-}, sigma_k, phi_k)) `EqvJ` {-sem-}(atomOfOEnv{--}(_Delta {->-}, sigma_k, phi_k))
\end{haskell}

Conversely, the dual operation "_Delta `contractOEnv` xi_set" purges the most recently allocated envelope "xi_set" from the address space of "_Delta". In fact, such operations are always guaranteed to be well-formed if the specified envelope actually appears at the top of the existing stack "stackOfOEnv{--}(_Delta)", a property that can be captured in Haskell by the following theorem:

\ReindexTheorem		{wfContractOEnvJ}

\begin{haskell}
wfContractOEnvJ		:: forall _Delta, sigma_c, xi_set => --
	WfJ{--}(_Delta) -> {-sem-}(head{--}(stackOfOEnv{--}(_Delta) == {-hv-}(sigma_c {-hv-}, xi_set))) -> --
	WfJ{-sem-}(_Delta `contractOEnv` xi_set)
\end{haskell}

After every such well-formed contraction operation, the specified stack frame is popped or removed from the top of "stackOfOEnv{--}(_Delta)" and the associated envelope is purged from the environment's two envelopes "envelopeOfOEnv{--}(_Delta)" and "initOfOEnv{--}(_Delta)":

\ReindexTheorem		{contractOEnvStackJ}
\ReindexTheorem		{contractOEnvEnvelopeJ}
\ReindexTheorem		{contractOEnvInitJ}

\begin{haskell}	
contractOEnvStackJ	:: forall _Delta, sigma_c, xi_set => --
	WfJ{--}(_Delta) -> {-sem-}(head{--}(stackOfOEnv{--}(_Delta) == {-hv-}(sigma_c {-hv-}, xi_set))) -> --
	{-sem-}(stackOfOEnv{--}(_Delta `contractOEnv` xi_set) == tail{--}(stackOfOEnv{--}(_Delta)))
contractOEnvEnvelopeJ	:: forall _Delta, sigma_c, xi_set => --
	WfJ{--}(_Delta) -> {-sem-}(head{--}(stackOfOEnv{--}(_Delta) == {-hv-}(sigma_c {-hv-}, xi_set))) -> --
	{-sem-}(envelopeOfOEnv{--}(_Delta `contractOEnv` xi_set) == envelopeOfOEnv{--}(_Delta) `difference` (xi_set `shiftEnvelope` sigma_c))
contractOEnvInitJ	:: forall _Delta, sigma_c, xi_set => --
	WfJ{--}(_Delta) -> {-sem-}(head{--}(stackOfOEnv{--}(_Delta) == {-hv-}(sigma_c {-hv-}, xi_set))) -> --
	{-sem-}(initOfOEnv{--}(_Delta `contractOEnv` xi_set) == initOfOEnv{--}(_Delta) `difference` (xi_set `shiftEnvelope` sigma_c))
\end{haskell}

Most readers will readily recognise a correspondence between the above theorem and the actual data stack regime imposed on the computational models of most modern instruction set architectures. Such detailed formalisation of this model proves critical to a successful treatment of certain rogue machine instruction sequences in a definition of their semantics through a mapping of assembly programs onto Etude functions, as mandated by the linear correctness principle from Chapter \ref{Design}. Nevertheless, the reader should observe that, in most cases, neither the envelope "envelopeOfOEnv{--}(_Delta)" nor the stack "stackOfOEnv{--}(_Delta)" feature in the actual hardware implementation of the program's address space, since, strictly speaking, these components are not required for a successful evaluation of well-formed Etude programs. Nevertheless, they are included in the above format model, in order to restrain meaningful Etude programs from adopting certain patterns of memory allocation that would render verification of the linear correctness principle all but impossible in practice.

A well-formed contraction of the form “"_Delta `contractOEnv` xi_set"\,” is also guaranteed to preserve the content of every memory-resident object that, in "_Delta", is stored in a previously-allocated region "{-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) @@ envelopeOfOEnv{--}(_Delta)" of the program's address space, provided that this region does not overlap any of the elements in the envelope "xi_set" that is being exterminated \hbox{by the contraction operation:}

\ReindexTheorem		{contractOEnvAtomsJ}

\begin{haskell}
contractOEnvAtomsJ	:: forall _Delta, sigma_c, xi_set, sigma_k, phi_k, mu_set_k =>
	WfJ{--}(_Delta) ->
	{-sem-}(head{--}(stackOfOEnv{--}(_Delta) == {-hv-}(sigma_c {-hv-}, xi_set))) ->
	{-sem-}({-hv-}(sigma_k {-hv-}, phi_k {-hv-}, mu_set_k) @@ envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(sigma_k + sizeOfFormat{--}(phi_k) <= sigma_c || sigma_k >= sigma_c + sizeOfEnvelope{--}(xi_set)) ->
	{-sem-}(atomOfOEnv{--}(_Delta `contractOEnv` xi_set {->-}, sigma_k, phi_k)) `EqvJ` {-sem-}(atomOfOEnv{--}(_Delta {->-}, sigma_k, phi_k))
\end{haskell}

Finally, an Etude program may populate the individual objects in its address space with concrete atomic values using an «environment update» operation of the general form \hbox{“"_Delta `setInOEnv` (sigma, phi, alpha)"”}, as represented by the following implementation-defined function:

\ReindexTerm		{setInOEnv}

\begin{haskell}
setInOEnv {-[*]-} {-/-} {-[*]-} :: (Ord nu) => OEnvT -> (Integer, FormatT, AtomT nu) -> OEnvT
\end{haskell}

In plain terms, "_Delta `setInOEnv` (sigma, phi, alpha)" constructs a new environment "_Delta'" in which the given object "(sigma, phi)" is populated with the specified atom "alpha" and the associated envelope element is removed from "initOfOEnv{--}(_Delta')", provided that the resulting object environment is valid and that "envelopeOfOEnv{--}(_Delta)" includes some element of the form “"{-hv-}(sigma {-hv-}, phi {-hv-}, mu_set)"”. Formally:

\ReindexTheorem		{setInOEnvAtomJ}
\ReindexTheorem		{setInOEnvInitJ}

\begin{haskell}
setInOEnvAtomJ 		:: forall _Delta, sigma, phi, alpha, mu_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(atomOfOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha) {->-}, sigma, phi)) `EqvJ` {-sem-}(alpha)

setInOEnvInitJ 		:: forall _Delta, sigma, phi, alpha, mu_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(initOfOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha) {->-}, sigma, phi) == initOfOEnv{--}(_Delta) `purgeEnvelope` (sigma, sigma + sizeOfFormat{--}(phi)))
\end{haskell}

Such well-formed object update operations will never affect the contents of any non-over·lapping memory-resident objects currently allocated in the same address space:

\ReindexTheorem		{setInOEnvAtomsJ}

\begin{haskell}
setInOEnvAtomsJ 	:: forall _Delta, sigma, phi, alpha, xi_set, sigma_k, phi_k, mu_set_k =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) ->
	{-sem-}({-hv-}(sigma {-hv-},    phi {-hv-},    mu_set)    `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}({-hv-}(sigma_k {-hv-},  phi_k {-hv-},  mu_set_k)  `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(sigma_k + sizeOfFormat{--}(phi_k) <= sigma || sigma_k >= sigma + sizeOfFormat{--}(phi)) ->
	{-sem-}(atomOfOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha) {->-}, sigma_k, phi_k)) `EqvJ` {-sem-}(atomOfOEnv{--}(_Delta {->-}, sigma_k, phi_k))
\end{haskell}

and, further, they always preserve all other properties of the original environment \hbox{as follows:}

\ReindexTheorem		{setInOEnvEnvelopeJ}
\ReindexTheorem		{setInOEnvStackJ}
\ReindexTheorem		{setInOEnvExtensionJ}
\ReindexTheorem		{setInOEnvNewJ}

\begin{haskell}
setInOEnvEnvelopeJ 	:: forall _Delta, sigma, phi, alpha, mu_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(envelopeOfOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha)) == envelopeOfOEnv{--}(_Delta))

setInOEnvStackJ 	:: forall _Delta, sigma, phi, alpha, mu_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(stackOfOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha)) == stackOfOEnv{--}(_Delta))

setInOEnvExtensionJ 	:: forall _Delta, sigma, phi, alpha, mu_set, xi_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) -> --
	WfJ{-sem-}(_Delta `extendOEnv` xi_set) ->
	WfJ{-sem-}((_Delta `setInOEnv` (sigma, phi, alpha)) `extendOEnv` xi_set)

setInOEnvNewJ 		:: forall _Delta, sigma, phi, alpha, mu_set, xi_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) -> --
	WfJ{-sem-}(_Delta `extendOEnv` xi_set) ->
	{-sem-}(newInOEnv{--}(_Delta `setInOEnv` (sigma, phi, alpha) {->-}, xi_set) == newInOEnv{--}(_Delta {->-}, xi_set))
\end{haskell}

However, the resulting object environment is guaranteed to be well-formed only if "alpha" represents a meaningful constant akin to “"IMM (x) (phi)"\!” for some rational number "x", and under assumption that an appropriate element "{-hv-}(sigma {-hv-}, phi {-hv-}, mu_set)" has been already allocated in the address space of "_Delta":

\ReindexTheorem		{wfSetInOEnvJ}

\begin{haskell}
wfSetInOEnvJ 		:: forall _Delta, sigma, phi, x, mu_set =>
	WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (phi)) -> {-sem-}({-hv-}(sigma {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	WfJ{-sem-}(_Delta `setInOEnv` (sigma, phi, {-sem-}(IMM (x) (phi))))
\end{haskell}

Finally, all updates of the same memory-resident object "(sigma, phi)" with equivalent atomic forms always result in identical object environments, whether the operation itself is well-formed or not:

\ReindexTheorem		{eqvSetInOEnvJ}

\begin{haskell}
eqvSetInOEnvJ 		:: forall _Delta, sigma, phi, alpha1, alpha2 => --
	{-sem-}(alpha1) `EqvJ` {-sem-}(alpha2) -> --
	{-sem-}(_Delta `setInOEnv` (sigma, phi, alpha1) = _Delta `setInOEnv` (sigma, phi, alpha2))
\end{haskell}

The reader should observe that the above definition of environment update operations does not scrutinise any access attributes associated with the targeted object. However, these attributes will soon become relevant to the semantics of the actual Etude term forms that empower programs with a direct access to the environment data during \hbox{their execution.}

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
