\set{capture}{hidden}
\begin{code}

module Lambda.Terms (
	bvOfBindings,
	fvOfTerm,
	fvOfBindings,
	substTerm,
	substBindings,
	composeTerms,
	completeTerm,
) where

import Additional
import Lambda.Syntax
import Lambda.Atoms

infixl 9 `composeTerms`, `completeTerm`
infixr 9 `substTerm`, `substBindings`

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}{Lambda/Terms}
\set{title}{Terms of the Game}
\begin{section}

Armed with the notions of atoms and the evaluation state, we are now ready to describe the actual computational framework of the language, depicted by a family of syntactic entities known as «terms». Besides delivering their desired result values, Etude terms may be also used to manipulate the program's evaluation state and interact with other aspects of its surrounding environment, such as the plethora of input and output devices supported by modern computers. Although a number of different approaches towards modelling of such interactive programs in a purely-functional language have been proposed in literature, the most satisfactory one depicts interactive computations by structures known as «monads» [Moggi89, Moggi91, Wadler92, PeytonJones93]. Without dwelling into the somewhat arcane and, in our case, mostly irrelevant category-theoretic foundations of these constructions, it will suffice to observe that monads in Etude follow the same principles as their use in Haskell, with an additional provision made for the absence of first-class functions in our intermediate program representation. In short, the technique is based on a realisation that, in principle, it should be possible to detach the semantic model of an interactive program from that of the devices being interacted with, in the same way as atoms provide for a separation between the meanings of Etude terms and the arithmetic objects that these terms manipulate. To this end, an interactive Etude program is described as a «monadic sequence» of abstract entities depicting invocations of the individual operating system primitives, every one of which represents a single incident in the program's life, generally without any attempt to model the perceivable effects of these incidents on the surrounding computational environment. In this, the monadic approach abstracts over the precise behaviour of individual input and output devices, focusing instead on the manner in which these devices are utilised by the program. In particular, given their interactive nature, the meanings of Etude terms are most naturally expressed as state transformation functions instead of mere scalar values. For example, the denotation of the C “printf\,\,” function is captured not by its rather uninteresting return value, or by the collection of side effects resulting from its evaluation such as the sequence of characters printed on the user's terminal, but rather by a function that applies a sequence of character-producing primitive operations to an abstract object that represents the world in which the function is being evaluated. 

Formally, the abstract syntax of all recognised Etude term forms is depicted by the following set of Haskell data type definitions:

\IndexType		{TermT}						{term \nu (\tstar)}
\IndexCons		{RET}						{RET() atoms (term)}
\IndexCons		{APP}						{() atom atoms (term)}
\IndexCons		{SYS}						{() prim atoms (term)}
\IndexCons		{IFF}						{IF_THEN_ELSE atom term term (term)}
\IndexCons		{NEW}						{NEW() envelope (term)}
\IndexCons		{DEL}						{DEL() envelope (term)}
\IndexCons		{GET}						{GET[,] atom mode (term)}
\IndexCons		{SET}						{SET_TO[,] atom mode format atom (term)}
\IndexCons		{INI}						{SETI_TO[,] atom mode format atom (term)}
\IndexCons		{LET}						{LET; bindings term (term)}

\IndexType		{BindingsT}					{bindings \nu (\tstar)}
\IndexType		{BindingT}					{binding \nu (\tstar)}

\begin{syntax}
\InsertHaskell{TermT, \par, BindingsT, \par, BindingT}
\end{syntax}

The algebraic semantics of terms are modelled by a set of Haskell functions and properties analogous to the well-formedness and equivalence of Etude atoms described earlier in Section \ref{Lambda/Atoms}. In particular, every Etude term "tau" is associated with a set of free variables "fvOfTerm{--}(tau)" derived from its lexical syntax as follows:

\BeginHaskellToken	{fvOfTerm}				{Open}	{FV}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{TermT nu}
  \HaskellResult							{SetT nu}
\EndHaskellToken

\IndexTerm		{fvOfTerm}					{FV term (\nu s)}

\begin{code}
fvOfTerm {-[*]-} :: (Ord nu) => TermT nu -> SetT nu

fvOfTerm {-sem-}(RET (alpha_set))					= fvOfAtoms{--}(alpha_set)
fvOfTerm {-sem-}(APP (alpha)  (alpha_set))				= fvOfAtom{--}(alpha) `union` fvOfAtoms{--}(alpha_set)
fvOfTerm {-sem-}(SYS (pi)     (alpha_set))				= fvOfAtoms{--}(alpha_set)
fvOfTerm {-sem-}(IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2))		= fvOfAtom{--}(alpha) `union` fvOfTerm{--}(tau1) `union` fvOfTerm{--}(tau2)
fvOfTerm {-sem-}(NEW  (xi_set))						= emptySet
fvOfTerm {-sem-}(DEL  (xi_set))						= emptySet
fvOfTerm {-sem-}(GET  (alpha {-hv-},   mu_set) (phi))			= fvOfAtom{--}(alpha)
fvOfTerm {-sem-}(SET  (alpha1 {-hv-},  mu_set) (phi) {-TO-} (alpha2))	= fvOfAtom{--}(alpha1) `union` fvOfAtom{--}(alpha2)
fvOfTerm {-sem-}(INI  (alpha1 {-hv-},  mu_set) (phi) {-TO-} (alpha2))	= fvOfAtom{--}(alpha1) `union` fvOfAtom{--}(alpha2)
fvOfTerm {-sem-}(LET  (beta_set) (tau))					= fvOfBindings{--}(beta_set) `union` (fvOfTerm{--}(tau) `difference` bvOfBindings{--}(beta_set))
\end{code}

and, for the actual list of bindings found in every “"LET"” construct:

\BeginHaskellToken	{fvOfBindings}				{Open}	{FV}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{BindingsT nu}
  \HaskellResult							{SetT nu}
\EndHaskellToken

\IndexTerm		{fvOfBindings}					{FV bindings (\nu s)}

\begin{code}
fvOfBindings {-[*]-} :: (Ord nu) => BindingsT nu -> SetT nu

fvOfBindings {-sem-}(beta_set {-,,-}) = unions [ fvOfTerm{--}(tau_k) | {-sem-}(nu_k {-bindTerm-}, tau_k) <- beta_set {-,,-} ]
\end{code}

In general, the above construction collects all variable names appearing anywhere within the term's structure, except that any variables bound by the definition list "beta_set" in a term of the form “"LET (beta_set) (tau)"\,” are always excluded from the set of variables appearing free in such term's body "tau". Formally, the set of variables bound by a list of Etude definitions is depicted by the notation “"bvOfBindings{--}(beta_set {-,-})"” and constructed from the union of all variables appearing anywhere to the left of the “"=:"” sign in the list as follows:

\BeginHaskellToken	{bvOfBindings}				{Open}	{BV}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{BindingsT nu}
  \HaskellResult							{SetT nu}
\EndHaskellToken

\IndexTerm		{bvOfBindings}					{BV bindings (\nu s)}

\begin{code}
bvOfBindings {-[*]-} :: (Ord nu) => BindingsT nu -> SetT nu

bvOfBindings {-sem-}(beta_set {-,,-}) = unions [ listToSet nu_k | {-sem-}(nu_k {-bindTerm-}, tau_k) <- beta_set {-,,-} ]
\end{code}

Further, we can also subject terms to the usual substitution function “"tau `substTerm` _S"\,” where "_S" represents a mapping of variable names to atoms intended as a replacement for these variables within the term "tau". Assuming that all atoms appearing in the finite map "_S" are closed, the definition of “\!"`substTerm`"” is simple, provided that we take care to purge from "_S" any variables bound by every “"LET"” term before applying it to that term's body:

\BeginHaskellToken	{substTerm}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{TermT nu}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{(MapT (nu) (AtomT nu))}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\NewHaskellToken	{{-/-}}					{Op}	{\slash}
\NewHaskellToken	{`substTerm`}				{Op}	{\slash}

\IndexTerm		{substTerm}					{\slash term (term)}

\begin{code}
substTerm {-[*]-} {-/-} {-[*]-} :: (Ord nu) => TermT nu -> (MapT (nu) (AtomT nu)) -> TermT nu

substTerm {-sem-}(RET (alpha_set))					{-/-} {-sem-}(_S) = {-sem-}(RET ({-sem-}(alpha_set `substAtoms` _S)))
substTerm {-sem-}(APP (alpha)  (alpha_set))				{-/-} {-sem-}(_S) = {-sem-}(APP ({-sem-}(alpha `substAtom` _S)) ({-sem-}(alpha_set `substAtoms` _S)))
substTerm {-sem-}(SYS (pi)     (alpha_set))				{-/-} {-sem-}(_S) = {-sem-}(SYS (pi) ({-sem-}(alpha_set `substAtoms` _S)))
substTerm {-sem-}(IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2))		{-/-} {-sem-}(_S) --
  = {-sem-}(IFF ({-sem-}(alpha `substAtom` _S)) {-THEN-} ({-sem-}(tau1 `substTerm` _S)) {-ELSE-} ({-sem-}(tau2 `substTerm` _S)))
substTerm {-sem-}(NEW  (xi_set))					{-/-} {-sem-}(_S) = {-sem-}(NEW  (xi_set))
substTerm {-sem-}(DEL  (xi_set))					{-/-} {-sem-}(_S) = {-sem-}(DEL  (xi_set))
substTerm {-sem-}(GET  (alpha {-hv-},   mu_set) (phi))			{-/-} {-sem-}(_S) = {-sem-}(GET  ({-sem-}(alpha `substAtom` _S {-hv-}, mu_set)) (phi))
substTerm {-sem-}(SET  (alpha1 {-hv-},  mu_set) (phi) {-TO-} (alpha2))	{-/-} {-sem-}(_S) = {-sem-}(SET  ({-sem-}(alpha1 `substAtom` _S {-hv-}, mu_set)) (phi) --
													{-TO-} ({-sem-}(alpha2 `substAtom` _S)))
substTerm {-sem-}(INI  (alpha1 {-hv-},  mu_set) (phi) {-TO-} (alpha2))	{-/-} {-sem-}(_S) = {-sem-}(INI  ({-sem-}(alpha1 `substAtom` _S {-hv-}, mu_set)) (phi) --
													{-TO-} ({-sem-}(alpha2 `substAtom` _S)))
substTerm {-sem-}(LET  (beta_set) (tau))				{-/-} {-sem-}(_S) = {-sem-}(LET  ({-sem-}(beta_set `substBindings` _S)) --
													(tau `substTerm` (_S `mapSetDifference` bvOfBindings{--}(beta_set))))
\end{code}

and, for “"LET"” bindings:

\BeginHaskellToken	{substBindings}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{BindingsT nu}
  \HaskellToken		{{-/-}}					{Bin}	{\slash}
  \HaskellArgument							{(MapT (nu) (AtomT nu))}
  \HaskellResult							{BindingsT nu}
\EndHaskellToken

\NewHaskellToken	{{-/-}}					{Op}	{\slash}
\NewHaskellToken	{`substBindings`}			{Op}	{\slash}

\IndexTerm		{substBindings}					{\slash bindings (bindings)}

\begin{code}
substBindings {-[*]-} {-/-} {-[*]-} :: (Ord nu) => BindingsT nu -> (MapT (nu) (AtomT nu)) -> BindingsT nu

substBindings {-sem-}(beta_set {-,,-} ) {-/-} {-sem-}(_S) --
  = [ {-sem-}(nu_set_k {-=-}, {-sem-}(tau_k `substTerm` _S)) | {-sem-}(nu_set_k {-=-}, tau_k) <- beta_set {-,,-} ]
\end{code}

Like atoms, terms are generally considered to be meaningful only if they are closed, i.e., if all of their free variables have been substituted for some well-formed atomic constants. However, in order to formalise the impact of monadic Etude terms on their evaluation state, every such term "tau" must be always interpreted in the context of an appropriate function environment "_Lambda" and object environment "_Delta", so that its well-formedness is asserted by a property of the form “"WfJ{-sem-}(_Lambda, _Delta {->-}, tau)"” rather than a mere “"WfJ{-sem-}(tau)"” as done previously for Etude atoms. Similarly, the equivalence of two terms "tau1" and "tau2", each taken in the context of its respective evaluation state, is always depicted as a binary property of the form “"{-sem-}(_Lambda1, _Delta1 {->-}, tau1) `EqvJ` {-sem-}(_Lambda2, _Delta2 {->-}, tau2)"”.

In particular, a monadic unit term of the form “"RET (alpha_set {-,-})"” permits Etude programs to «lift» a list of atoms into the term monad. For conciseness, we will usually omit the parentheses surrounding the atom list "alpha_set" whenever that list consists of precisely one atom. Conceptually, such lifted terms depict the «unit» action of the term monad, which always delivers a predetermined set of results described precisely by the specified list of atoms without affecting the program's environment in any other way. Accordingly, well-formedness of lifted terms is determined solely by the well-formedness of its constituents, as captured by the following simple theorem:

\ReindexTheorem		{wfRetTermJ}

\begin{haskell}
wfRetTermJ :: forall _Lambda, _Delta, alpha_set => WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{--}(alpha_set) -> WfJ{-sem-}(_Lambda, _Delta {->-}, RET (alpha_set))
\end{haskell}

and a pair of such terms should be always considered equivalent whenever it is formed from pairwise-equivalent atomic lists:

\ReindexTheorem		{eqvRetTermJ}

\begin{haskell}
eqvRetTermJ :: forall _Lambda, _Delta, alpha_set1, alpha_set2 => --
	(alpha_set1 `EqvJ` alpha_set2) -> --
	{-sem-}(_Lambda, _Delta {->-}, RET (alpha_set1)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, RET (alpha_set2))
\end{haskell}

In Etude, an application of a given function atom "alpha" to a list of zero or more arguments "alpha_set" is depicted by a term of the form “"APP (alpha) (alpha_set)"”. Both the function itself and its arguments are represented by atomic values. While Etude places no specific requirements on the argument list "alpha_set" beyond its general validity, the function value "alpha" must always represent an equivalent of a well-formed atomic form “"IMM (x) (phi)"\!”, in which "phi" is a valid format from the function genre and the integer "x" is mapped in the associated function environment "_Lambda" to a well-formed Etude function of the form “"FUN (nu_set) {-DOT-} (tau)"\,”. Further, for such an application term to be considered well-formed, the function's parameter list "nu_set" must have the same length as the argument list "alpha_set" and the body term "tau" must be well-formed after substitution of all parameter variables from "nu_set" for the corresponding atomic values from "alpha_set". Formally:

\ReindexTheorem		{wfAppTermJ}

\begin{haskell}
wfAppTermJ :: forall _Lambda, _Delta, x, alpha_set, nu_set, tau =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (FG`FMT`standardEncoding)) -> WfJ{--}(alpha_set) ->
	{-sem-}(_Lambda#(x) == {-sem-}(FUN (nu_set) {-DOT-} (tau)) && lengthOfList{--}(nu_set) == lengthOfList{--}(alpha_set)) ->
	WfJ{--}(_Lambda, _Delta {->-}, tau `substTerm` (nu_set `zipList` alpha_set)) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, APP (IMM (x) (FG`FMT`standardEncoding)) (alpha_set))
\end{haskell}

Every such application term is equivalent precisely to a term formed from the substitution of "alpha_set" for the variable list "nu_set" in the corresponding function's body "tau":

\ReindexTheorem		{eqvAppBetaTermJ}

\begin{haskell}
eqvAppBetaTermJ :: forall _Lambda, _Delta, x, alpha_set, nu_set, tau =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (FG`FMT`standardEncoding)) -> WfJ{--}(alpha_set) ->
	{-sem-}(_Lambda#(x) == {-sem-}(FUN (nu_set) {-DOT-} (tau)) && lengthOfList{--}(nu_set) == lengthOfList{--}(alpha_set)) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, tau `substTerm` (nu_set `zipList` alpha_set)) ->
	{-sem-}(_Lambda, _Delta {->-}, APP (IMM (x) (FG`FMT`standardEncoding)) (alpha_set)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, tau `substTerm` (nu_set `zipList` alpha_set))
\end{haskell}

Readers will observe that the above theorem provides a direct Etude analogue of the beta reduction rule from Section \ref{Lambda/Pure}. To aid reasoning about unreduced function applications, we also insist on the following structural equivalence between pairs of such terms in which all corresponding components represent pairwise-equivalent \hbox{atomic values:}

\ReindexTheorem		{eqvAppTermJ}

\begin{haskell}
eqvAppTermJ :: forall _Lambda, _Delta, alpha1, alpha_set1, alpha2, alpha_set2 =>
	(alpha1 `EqvJ` alpha2) -> --
	(alpha_set1 `EqvJ` alpha_set2) ->
	{-sem-}(_Lambda, _Delta {->-}, APP (alpha1) (alpha_set1)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, APP (alpha2) (alpha_set2))
\end{haskell}

Although superficially-similar, terms of the form “"SYS (pi) (alpha_set)"”, in which "pi" represents a «primitive operation» taken from an implementation-defined set of values selected individually by every system architecture have a very different set of formal properties. For one, the generic fragment of Etude's semantics remains silent on any well-formedness criteria applicable to these terms, since, in general, the range of concrete values acceptable for a given primitive "pi" is determined solely by the murky details of the underlying operating system. Nevertheless, if the same primitive is applied to a pair of equivalent atomic lists, then we can always be sure about the equivalence of the resulting terms:

\ReindexTheorem		{eqvSysTermJ}

\begin{haskell}
eqvSysTermJ :: forall _Lambda, _Delta, pi, alpha_set1, alpha_set2 => --
	(alpha_set1 `EqvJ` alpha_set2) -> --
	{-sem-}(_Lambda, _Delta {->-}, SYS (pi) (alpha_set1)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, SYS (pi) (alpha_set2))
\end{haskell}

Theorem "eqvSysTermJ" makes it possible to detach the notion of term equivalence from the semantic specifics of the underlying operating system. To complete this detachment, conditional terms of the form “"IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2)"” allow for embedding of arbitrary, even malformed operands within well-formed Etude programs. In such constructs, only one of the two branches "tau1" and "tau2" needs to represent a valid computation:

\ReindexTheorem		{wfIffTrueTermJ}
\ReindexTheorem		{wfIffFalseTermJ}

\begin{haskell}
wfIffTrueTermJ   :: forall _Lambda, _Delta, x, tau1, tau2 =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (ZG`FMT`standardEncoding)) -> WfJ{--}(tau1) -> {-sem-}(x /= 0) ->
	WfJ{-sem-}(IFF (IMM (x) (ZG`FMT`standardEncoding)) {-THEN-} (tau1) {-ELSE-} (tau2))
	
wfIffFalseTermJ   :: forall _Lambda, _Delta, x, tau1, tau2 =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (ZG`FMT`standardEncoding)) -> WfJ{--}(tau2) -> {-sem-}(x == 0) ->
	WfJ{-sem-}(IFF (IMM (x) (ZG`FMT`standardEncoding)) {-THEN-} (tau1) {-ELSE-} (tau2))
\end{haskell}

In particular, if "alpha" represents any natural number other than zero, then the Etude term “"IFF (alpha) {-THEN-} (tau1) {-ELSE-} (tau2)"\!” is always equivalent to "tau1":

\ReindexTheorem		{eqvIffTrueTermJ}

\begin{haskell}
eqvIffTrueTermJ   :: forall _Lambda, _Delta, x, tau1, tau2 =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (ZG`FMT`standardEncoding)) -> WfJ{--}(tau1) -> {-sem-}(x /= 0) ->
	{-sem-}(IFF (IMM (x) (ZG`FMT`standardEncoding)) {-THEN-} (tau1) {-ELSE-} (tau2)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, tau1)
\end{haskell}

Conversely, if "alpha" has a zero value, then the conditional term serves as a mere synonym for its later branch "tau2". Formally:

\ReindexTheorem		{eqvIffFalseTermJ}

\begin{haskell}
eqvIffFalseTermJ  :: forall _Lambda, _Delta, x, tau1, tau2 =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) -> WfJ{-sem-}(IMM (x) (ZG`FMT`standardEncoding)) -> WfJ{--}(tau2) -> {-sem-}(x == 0) ->
	{-sem-}(IFF (IMM (x) (ZG`FMT`standardEncoding)) {-THEN-} (tau1) {-ELSE-} (tau2)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, tau2)
\end{haskell}

As for the earlier term form, two such terms are also equivalent when assembled from pairwise-equivalent components:

\ReindexTheorem		{eqvIffTermJ}

\begin{haskell}
eqvIffTermJ :: forall _Lambda1, _Delta1, alpha1, tau11, tau21, _Lambda2, _Delta2, alpha2, tau12, tau22 =>
	(alpha1 `EqvJ` alpha2) ->
	(_Lambda1, _Delta1 {->-}, tau11) `EqvJ` (_Lambda2, _Delta2 {->-}, tau12) -> --
	(_Lambda1, _Delta1 {->-}, tau12) `EqvJ` (_Lambda2, _Delta2 {->-}, tau22) ->
	{-sem-}(_Lambda1, _Delta1 {->-}, IFF (alpha1) {-THEN-} (tau11) {-ELSE-} (tau21)) `EqvJ` {-sem-}(_Lambda2, _Delta2 {->-}, IFF (alpha2) {-THEN-} (tau12) {-ELSE-} (tau22))
\end{haskell}

Terms of the form “"NEW (xi_set)"” and “"DEL (xi_set)"” provide Etude programs with explicit facilities for extending and contracting the address space of their environment by the envelope "xi_set". Formally, such terms are considered to be well-formed only in the context of an object environment "_Delta" to which the corresponding extension and contraction operations "_Delta `extendOEnv` xi_set" and  "_Delta `contractOEnv` xi_set" can be applied within the constraints of the underlying language implementation outlined earlier in Section \ref{Lambda/State}. Specifically:

\ReindexTheorem		{wfNewTermJ}
\ReindexTheorem		{wfDelTermJ}

\begin{haskell}
wfNewTermJ  :: forall _Lambda, _Delta, xi_set => WfJ{--}(_Lambda) -> WfJ{-sem-}(_Delta `extendOEnv`    xi_set) -> WfJ{-sem-}(_Lambda, _Delta {->-}, NEW  (xi_set))
wfDelTermJ  :: forall _Lambda, _Delta, xi_set => WfJ{--}(_Lambda) -> WfJ{-sem-}(_Delta `contractOEnv`  xi_set) -> WfJ{-sem-}(_Lambda, _Delta {->-}, DEL  (xi_set))
\end{haskell}

Under these conditions, “"NEW (xi_set)"” is always equivalent to a lifted term of the form “"RET [IMM (x) (OG`FMT`standardEncoding)]"” in which the integer "x" has the value of "newInOEnv{--}(_Delta {->-}, xi_set)", while “"DEL (xi_set)"” stands for an empty list of atoms “"RET []"” in the context of an object environment "_Delta" contracted by the supplied envelope "xi_set". Specifically:

\ReindexTheorem		{eqvNewTermJ}
\ReindexTheorem		{eqvDelTermJ}

\begin{haskell}
eqvNewTermJ  :: forall _Lambda, _Delta, xi_set =>
	WfJ{--}(_Lambda) -> WfJ{-sem-}(_Delta `extendOEnv` xi_set) ->
	{-sem-}(_Lambda, _Delta {->-}, NEW  (xi_set)) `EqvJ` {-sem-}(_Lambda, _Delta `extendOEnv` xi_set {->-}, --
		RET [IMM ({-sem-}(newInOEnv{--}(_Delta {->-}, xi_set))) (OG`FMT`standardEncoding)])

eqvDelTermJ  :: forall _Lambda, _Delta, xi_set =>
	WfJ{--}(_Lambda) -> WfJ{-sem-}(_Delta `contractOEnv` xi_set) ->
	{-sem-}(_Lambda, _Delta {->-}, DEL  (xi_set)) `EqvJ` {-sem-}(_Lambda, _Delta `contractOEnv` xi_set {->-}, RET [])
\end{haskell}

Further, each valid construction of the form “"NEW (xi_set)"” guarantees well-formedness of every object atom “"OP2 (alpha) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)"\!”, in which "alpha" is equivalent to an appropriately converted value of “"IMM ({-sem-}(newInOEnv{--}(_Delta {->-}, xi_set))) (OG`FMT`standardEncoding)"\!” and "n" represents an offset into one of the envelope elements in "xi_set", as captured by the following Haskell definition:

\ReindexTheorem		{wfNewAtomsJ}

\begin{haskell}
wfNewAtomsJ :: forall _Lambda, _Delta, xi_set =>
	WfJ{-sem-}(_Lambda, _Delta {->-}, NEW (xi_set)) ->
	(forall n, phi, mu_set => --
		{-sem-}({-hv-}(n {-hv-}, phi {-hv-}, mu_set) @@ xi_set) -> --
		WfJ{-sem-}(OP2 ({-hv-}(OP1 (CNV {-sem-}(objectFormat{--}(phi))) (OG`FMT`standardEncoding) --
						{--}{-hv-}(IMM ({-sem-}(newInOEnv{--}(_Delta {->-}, xi_set))) (OG`FMT`standardEncoding)))) --
				(ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))))
\end{haskell}

Similarly, the “"GET (alpha {-hv-}, mu_set) (phi)"\!” term form makes it possible for Etude programs to access individual atomic bindings from the associated object environment "_Delta". As is to be expected, the "alpha" argument of “"GET"” should be equivalent to some well-formed \hbox{object} atom “"OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))"\!”, such that "atomOfOEnv{--}(_Delta {->-}, x + n, phi)" represents a valid atomic value whenever "{-hv-}(x + n {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)". Formally:

\ReindexTheorem		{wfGetTermJ}

\begin{haskell}
wfGetTermJ  :: forall _Lambda, _Delta, x, n, phi, mu_set =>
	WfJ{--}(_Lambda) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) -> --
	WfJ{-sem-}(atomOfOEnv{--}(_Delta {->-}, x + n, phi)) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, --
			GET (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi))
\end{haskell}

In all cases, a pair of “"GET"” terms that access equivalent address atoms under identical access modes and formats are always considered to be equivalent:

\ReindexTheorem		{eqvGetTermJ}

\begin{haskell}
eqvGetTermJ :: forall _Lambda, _Delta, alpha1, alpha2, phi, mu_set =>
	(alpha1 `EqvJ` alpha2) ->
	{-sem-}(_Lambda, _Delta {->-}, GET (alpha1 {-hv-}, mu_set) (phi)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, GET (alpha2 {-hv-}, mu_set) (phi))
\end{haskell}

Further, if the address "x + n" is actually mapped in the term's evaluation state to an atom "alpha'" without the volatile access attribute “"VM"”, then “"GET (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi)"\!” is also equivalent to a simple lifted term of the form “"RET [alpha]"”. Formally:

\ReindexTheorem		{eqvGetEnvTermJ}

\begin{haskell}
eqvGetEnvTermJ :: forall _Lambda, _Delta, x, n, phi, mu_set =>
	WfJ{--}(_Lambda) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) -> --
	WfJ{-sem-}(atomOfOEnv{--}(_Delta {->-}, x + n, phi)) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, mu_set `difference` listToSet [VM]) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(_Lambda, _Delta {->-}, --
			GET (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi)) --
		`EqvJ` {-sem-}(_Lambda, _Delta {->-}, RET {-sem-}[atomOfOEnv{--}(_Delta {->-}, x + n, phi)])
\end{haskell}

Conversely, both the “"SET (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi) {-TO-} (alpha)"” term form and its “"INI"” variant perform the environment update operation "_Delta `setInOEnv` (x + n, phi, alpha)". In other words, their well-formedness is mandated by the following pair of theorems:

\ReindexTheorem		{wfSetTermJ}
\ReindexTheorem		{wfIniTermJ}

\begin{haskell}
wfSetTermJ :: forall _Lambda, _Delta, x, n, phi, mu_set, alpha =>
	WfJ{--}(_Lambda) -> --
	WfJ{--}(_Delta `setInOEnv` (x + n, phi, alpha)) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, mu_set) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, --
		SET (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi) {-TO-} (alpha))

wfIniTermJ :: forall _Lambda, _Delta, x, n, phi, mu_set, alpha =>
	WfJ{--}(_Lambda) -> --
	WfJ{--}(_Delta `setInOEnv` (x + n, phi, alpha)) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, mu_set) `inEnvelope` initOfOEnv{--}(_Delta)) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, --
		INI (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi) {-TO-} (alpha))
\end{haskell}

Intuitively, the “"SET"” variant of an environment update preserves the conventional semantics of the constant memory access attribute “"CM"”, while “"INI"” ignores that attribute for objects located within the uninitialised region of the address space "initOfOEnv{--}(_Delta)", in order to facilitate a once-off initialisation of otherwise-immutable memory-resident objects within a well-formed Etude program. In particular, both forms of the term “"SET (alpha1 {-hv-}, mu_set) (phi) {-TO-} (alpha2)"\!” are reducible into the trivial monadic sequence “"RET []"”, taken in the context of an object environment updated with the binding of the address "alpha1" to the value of "alpha2", provided that such an update is deemed well-formed by the above \hbox{validity criteria:}

\ReindexTheorem		{eqvSetEnvTermJ}
\ReindexTheorem		{eqvIniEnvTermJ}

\begin{haskell}
eqvSetEnvTermJ :: forall _Lambda, _Delta, x, n, mu_set, phi, alpha =>
	WfJ{--}(_Lambda) -> --
	WfJ{--}(_Delta `setInOEnv` (x + n, phi, alpha)) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, emptySet) `inEnvelope` envelopeOfOEnv{--}(_Delta)) ->
	{-sem-}(_Lambda, _Delta {->-}, ---
		SET (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi) {-TO-} (alpha)) --
			`EqvJ` {-sem-}(_Lambda, _Delta `setInOEnv` (x + n, phi, alpha) {->-}, RET []) {-!-}{-!-}{-!-}{-!-}{-!-}

eqvIniEnvTermJ :: forall _Lambda, _Delta, x, n, mu_set, phi, alpha =>
	WfJ{--}(_Lambda) -> --
	WfJ{--}(_Delta `setInOEnv` (x + n, phi, alpha)) -> --
	WfJ{-sem-}(OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding))) ->
	{-sem-}({-hv-}(x + n {-hv-}, phi {-hv-}, listToSet [CM]) `inEnvelope` initOfOEnv{--}(_Delta)) ->
	{-sem-}(_Lambda, _Delta {->-}, ---
		INI (OP2 (IMM (x) ({-sem-}(objectFormat{--}(phi)))) (ADD) ({-sem-}(objectFormat{--}(phi))) (IMM (n) (ZG`FMT`standardEncoding)) {-hv-}, mu_set) (phi) {-TO-} (alpha)) --
			`EqvJ` {-sem-}(_Lambda, _Delta `setInOEnv` (x + n, phi, alpha) {->-}, RET []) {-!-}{-!-}{-!-}{-!-}{-!-}
\end{haskell}

Finally, both “"SET"” and “"INI"” are subject to the usual structural equivalence rules as follows:

\ReindexTheorem		{eqvSetTermJ}
\ReindexTheorem		{eqvIniTermJ}

\begin{haskell}
eqvSetTermJ :: forall _Lambda, _Delta, alpha11, alpha21, alpha12, alpha22, mu_set, phi =>
	(alpha11 `EqvJ` alpha12) -> --
	(alpha21 `EqvJ` alpha22) ->
	{-sem-}(_Lambda, _Delta {->-}, SET (alpha11 {-hv-}, mu_set) (phi) {-TO-} (alpha21)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, SET (alpha12 {-hv-}, mu_set) (phi) {-TO-} (alpha22))

eqvIniTermJ :: forall _Lambda, _Delta, alpha11, alpha21, alpha12, alpha22, mu_set, phi =>
	(alpha11 `EqvJ` alpha12) -> --
	(alpha21 `EqvJ` alpha22) ->
	{-sem-}(_Lambda, _Delta {->-}, INI (alpha11 {-hv-}, mu_set) (phi) {-TO-} (alpha21)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, SET (alpha12 {-hv-}, mu_set) (phi) {-TO-} (alpha22))
\end{haskell}

Last but not least, a term of the form “"LET [nu_set1 =: tau1 {-hv-}, nu_set2 =: tau2 .. nu_set{-sub-}n =: tau_n] (tau)"\,” permits Etude programs to capture the results delivered by the monadic terms "tau1 .. tau_n" and makes them available within the body term "tau" as the corresponding variables from the parameter lists "nu_set1 .. nu_set{-sub-}n". Accordingly, a degenerate form of the “"LET (beta_set) (tau)"\,” term that specifies an empty binding list "beta_set" is always equivalent to its body expression "tau" and well-formed whenever "tau" happens to be valid. Formally:

\ReindexTheorem		{eqvLetNullTermJ}

\begin{haskell}
eqvLetNullTermJ  :: forall _Lambda, _Delta, tau => {-sem-}(_Lambda, _Delta {->-}, LET [] (tau)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, tau)
\end{haskell}

Further, a term of the form “"LET [nu_set =: RET (alpha_set)] (tau)"\,” constitutes a simple syntactic equivalent of the beta redex “"{-hv-}(PureLambda (nu_set) {-dot-} (tau)){--}(alpha_set)"” from the pure lambda calculus described in Section \ref{Lambda/Pure}. Accordingly, such terms are deemed valid only if "nu_set" and "alpha_set" represent well-formed lists of equal lengths. Their generic properties are determined in a manner similar to that described earlier for function applications:

\ReindexTheorem		{eqvLetRetTermJ}

\begin{haskell}
eqvLetRetTermJ :: forall _Lambda, _Delta, nu_set, alpha_set =>
	WfJ{--}(nu_set) -> WfJ{--}(alpha_set) ->
	{-sem-}(lengthOfList{--}(nu_set) == lengthOfList{--}(alpha_set)) ->
	{-sem-}(_Lambda, _Delta {->-}, LET [nu_set =: RET (alpha_set)] (tau)) `EqvJ` (_Lambda, _Delta {->-}, tau `substTerm` (nu_set `zipList` alpha_set))
\end{haskell}

For conciseness, in the remainder of this work I will usually drop the “"RET"” keyword in such trivial “"LET"” expressions, writing the above term simply as “"LET [nu_set =: {-hide-}RET {--}(alpha_set)] (tau)"\,” for readability. This style of programming, in which all subexpressions are systematically bound to variable names and all beta redexes appear as explicit “"LET"” bindings is well known in literature under the title of the «monadic normal form» used to study the operational semantics of call-by-value lambda calculi [Moggi91, Hatcliff94, Danvy03]. It is also known as the «administrative normal form» to some authors, since it eliminates the so-called «administrative redexes» introduced by translation of a program into the «continuation passing style» [Sabry92, Flanagan93].

The semantics of “"LET"” bindings get interesting when two such terms are nested into a binding of the form “"LET [nu_set =: {-hv-}(LET (beta_set) (tau1))] (tau2)"\!”. The next theorem states that all such constructions can be always flattened into “"LET (beta_set) {-hv-}(LET [nu_set =: tau1] (tau2))"\!”, provided that none of the variables bound in "beta_set", other than those found in the associated parameter list "nu_set", appear free in "tau2". Formally:

\ReindexTheorem		{eqvLetLetTermJ}

\begin{haskell}
eqvLetLetTermJ :: forall _Lambda, _Delta, nu_set, beta_set, tau1, tau2 =>
	{-sem-}(bvOfBindings{--}(beta_set) `intersection` (fvOfTerm{--}(tau2) `difference` nu_set) == emptySet) ->
	{-sem-}(_Lambda, _Delta {->-}, LET ({-hv-}[nu_set =: LET (beta_set) (tau1)]) (tau2)) --
		`EqvJ` {-sem-}(_Lambda, _Delta {->-}, LET (beta_set) {-hv-}(LET [nu_set =: tau1] (tau2)))
\end{haskell}

The reader should observe that the free variable constraint in the above theorem conveniently circumvents the variable capture problem inherent to the pure calculus formulation of the substitution function from Section \ref{Lambda/Pure}.

Unfortunately, little else can be said about validity of any other singular term form “"LET [nu_set =: tau1] (tau2)"\!” without dwelling into the underlying implementation details of Etude's operational model, since the potential presence of primitive applications in "tau1" makes it impossible to narrow down the set of evaluation states that may be encountered by the body term "tau2". However, a rather relaxed set of requirements for these remaining constructs may be captured eloquently without divulging any details of the underlying operating system implementation with the help of the following theorem:

\ReindexTheorem		{eqvLetSysTermJ}

\begin{haskell}
eqvLetSysTermJ :: forall _Lambda, _Delta, nu_set1, tau11, tau21, nu_set2, tau12, tau22 =>
	{-sem-}(_Lambda, _Delta {->-}, tau11) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, tau12) ->
	{-sem-}(lengthOfList{--}(nu_set1) == lengthOfList{--}(nu_set2)) ->
	(	forall _Lambda', _Delta', alpha_set' =>
	 	{-quad-}	WfJ{--}(alpha_set') ->
	 	{-quad-}	{-sem-}(lengthOfList{--}(alpha_set') == lengthOfList{--}(nu_set1)) ->
		{-quad-}	{-sem-}(_Lambda', _Delta' {->-}, tau21 `substTerm` (nu_set1 `zipList` alpha_set')) --
					`EqvJ` {-sem-}(_Lambda', _Delta' {->-}, tau22 `substTerm` (nu_set2 `zipList` alpha_set'))) ->
	{-sem-}(_Lambda, _Delta {->-}, LET [nu_set1 =: tau11] (tau21)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, LET [nu_set2 =: tau12] (tau22))
\end{haskell}

Intuitively, theorem "eqvLetSysTermJ" states that we can always assert equivalence between a pair of terms of the form “"LET [nu_set1 =: tau11] (tau21)"\!” and “"LET [nu_set2 =: tau12] (tau22)"\!”, provided that "tau11" is equivalent to "tau12", "nu_set1" and "nu_set2" represent variable lists of equal lengths and, no matter what well-formed atomic values are substituted for these variables into the body terms "tau21" and "tau22", these bodies remain equivalent under every conceivable evaluation state.

A reader may observe that, so far, we have not mentioned anything about “"LET"” terms that specify multiple bindings in the list "beta_set". Intuitively, such constructs are intended to capture the unspecified aspects of certain expression forms in the C language, whose precise evaluation order is left open to individual interpretation by every implementation of a C compiler. In principle, such terms are essentially equivalent to a sequential application of each binding from "beta_set":

\ReindexTheorem		{eqvLetListTermJ}

\begin{haskell}
eqvLetListTermJ :: forall _Lambda, _Delta, nu_set, beta_set, tau =>
	{-sem-}(lengthOfList{--}(beta_set) > 1) ->
	{-sem-}(_Lambda, _Delta {->-}, LET [beta_set] (tau)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, LET [{-sem-}(head{--}(beta_set))] (LET ({-sem-}(tail{--}(beta_set))) (tau)))
\end{haskell}

However, they are only considered to be well-formed if all permutations of "beta_set" are equivalent to each other and, further, if none of the variables found to the left of one of the “"=:"” signs appears free in any other binding from the list. Using the standard mathematical definition of the factorial function "factorial (x) {-!-}", together with "permutation{--}(k, beta_set)", which returns the canonical "k"-th permutation of the list "beta_set", this constraint can be formalised in Haskell as the following theorem:

\ReindexTheorem		{wfLetListTermJ}

\begin{haskell}
wfLetListTermJ :: forall _Lambda, _Delta, beta_set, tau =>
	WfJ{--}(_Lambda) -> WfJ{--}(_Delta) ->
	WfJ{-sem-}(_Lambda, _Delta {->-}, LET [{-sem-}(head{--}(beta_set))] (LET ({-sem-}(tail{--}(beta_set))) (tau))) ->
	{-sem-}(union [fvOfTerm{--}(tau_k) `difference` nu_set{-sub-}k | {-sem-}(nu_set{-sub-}k =: tau_k) <- beta_set {-,,-} ] `intersection` bvOfBindings{--}(beta_set) == emptySet) ->
	(	forall k => --
			{-sem-}(0 <= k < factorial (lengthOfList{--}(beta_set)) {-!-}) -> --
			{-sem-}(_Lambda, _Delta {->-}, LET (beta_set) (tau)) `EqvJ` {-sem-}(_Lambda, _Delta {->-}, LET ({-sem-}(permutation{--}(k, beta_set))) (tau))) -> --
				{-!-}{-!-}{-!-}{-!-}{-!-}{-!-}
	WfJ{-sem-}(_Lambda, _Delta {->-}, LET (beta_set) (tau))
\end{haskell}

To wrap up our algebraic treatment of Etude terms, we must specify four more administrative theorems. The first of these states that, if one term in a given equivalence class happens to represent a meaningful computation, then all other terms equivalent to it are likewise well-formed:

\ReindexTheorem		{wfEqvTermJ}

\begin{haskell}
wfEqvTermJ :: forall _Lambda1, _Delta1, tau1, _Lambda2, _Delta2, tau2 =>
	WfJ{-sem-}(_Lambda1, _Delta1 {->-}, tau1) ->
	{-sem-}(_Lambda1, _Delta1 {->-}, tau1) `EqvJ` {-sem-}(_Lambda2, _Delta2 {->-}, tau2) ->
	WfJ{-sem-}(_Lambda2, _Delta2 {->-}, tau2)
\end{haskell}

Finally, the “"`EqvJ`"” property of terms must always represent a true equivalence relation, in that the following reflexivity, symmetry and transitivity laws must be satisfied:

\ReindexTheorem		{eqvReflTermJ}
\ReindexTheorem		{eqvSymmTermJ}
\ReindexTheorem		{eqvTrnsTermJ}

\begin{haskell}
eqvReflTermJ	:: forall _Lambda, _Delta, tau =>
	{-sem-}(_Lambda,   _Delta   {->-}, tau   ) `EqvJ` {-sem-}(_Lambda,   _Delta   {->-}, tau  )

eqvSymmTermJ	:: forall _Lambda1, _Delta1, tau1, _Lambda2, _Delta2, tau2 =>
	{-sem-}(_Lambda1,  _Delta1  {->-}, tau1  ) `EqvJ` {-sem-}(_Lambda2,  _Delta2  {->-}, tau2  ) ->
	{-sem-}(_Lambda2,  _Delta2  {->-}, tau2  ) `EqvJ` {-sem-}(_Lambda1,  _Delta1  {->-}, tau1  )

eqvTrnsTermJ	:: forall _Lambda1, _Delta1, tau1, _Lambda2, _Delta2, tau2, _Lambda3, _Delta3, tau3 =>
	{-sem-}(_Lambda1,  _Delta1  {->-}, tau1  ) `EqvJ` {-sem-}(_Lambda2,  _Delta2  {->-}, tau2  ) ->
	{-sem-}(_Lambda2,  _Delta2  {->-}, tau2  ) `EqvJ` {-sem-}(_Lambda3,  _Delta3  {->-}, tau3  ) ->
	{-sem-}(_Lambda1,  _Delta1  {->-}, tau1  ) `EqvJ` {-sem-}(_Lambda3,  _Delta3  {->-}, tau3  )
\end{haskell}

In the following chapters, incomplete expression of the form “"LET (beta_set)"” that depict Haskell functions of the type “"TermT nu -> TermT nu"\!” are often composed together and eventually applied to some tail "tau" in order to construct a proper Etude term entity. In Haskell, such structures would be normally written as “"(tau_fun1 . tau_fun2 . .. . tau_fun{-sub-}k){--}tau"\,”, but their semantic meaning is more obvious when the composition operator “"."” is replaced with the “"`composeTerms`"” symbol, resulting in the expected term sequence notation “"tau_fun1 `composeTerms` tau_fun2 `composeTerms` .. tau_fun{-sub-}k `completeTerm` tau"\,”. To give this notation a formal mandate, it is useful to introduce the following two Haskell \hbox{combinators:}

\BeginHaskellToken	{composeTerms}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{(TermT nu -> TermT nu)}
  \HaskellToken		{{-semicolon-}}				{Close}	{\LKeyword{;}}
  \HaskellArgument							{(TermT nu -> TermT nu)}
  \HaskellResult							{(TermT nu -> TermT nu)}
\EndHaskellToken

\BeginHaskellToken	{completeTerm}				{Void}	{}
  \HaskellContext							{(Ord nu)}
  \HaskellArgument							{TermT nu -> TermT nu}
  \HaskellToken		{{-semicolon-}}				{Close}	{\LKeyword{;}}
  \HaskellArgument							{TermT nu}
  \HaskellResult							{TermT nu}
\EndHaskellToken

\NewHaskellToken	{{-semicolon-}}				{Close}	{\LKeyword{;}}
\NewHaskellToken	{`composeTerms`}			{Close}	{\LKeyword{;}}
\NewHaskellToken	{`completeTerm`}			{Close}	{\LKeyword{;}}

\IndexTerm		{composeTerms}					{; term term}
\IndexTerm		{completeTerm}					{; term term}

\begin{code}
composeTerms {-[*]-} {-semicolon-} {-[*]-} :: (TermT nu -> TermT nu) -> (TermT nu -> TermT nu) -> (TermT nu -> TermT nu)

composeTerms {-sem-}(tau_fun1) {-semicolon-} {-sem-}(tau_fun2) 	= tau_fun1 . tau_fun2
{--}
completeTerm {-[*]-} {-semicolon-} {-[*]-} :: (TermT nu -> TermT nu) -> TermT nu -> TermT nu

completeTerm {-sem-}(tau_fun) {-semicolon-} {-sem-}(tau) 	= tau_fun tau
\end{code}

In other words, in this work the “"`composeTerms`"” symbol may stand for either the function composition operator “"."” or a function application, as dictated by the expression's context. Either way, the Etude syntax and the associated informal discussion should always make the intended meaning clear, so that, in practice, this overloading of the “"`composeTerms`"” operator should never impair clarity of the following presentation.

An astute reader will rightly observe that the above set of theorems is, by itself, insufficient to establish confluence of the rewrite system implied by the stated well-formedness and term equivalence laws. However, in Chapter \ref{MMIX}, confluence of the MMIX Etude variant is proven constructively by the virtue of its «evaluation function», which performs a single-step functional reduction of a monadic term into its appropriate normal form. In the meantime, however, the above Spartan system of algebraic relations will have to suffice for all reasoning about generic Etude constructs and the portable C programs from which they have been derived.

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%