\set{capture}{hidden}
\begin{code}

module MMIX.Semantics (
	meaningOfMMixModule,
	pregs,
	regs,
	shiftMMixVar,
	meaningOfMMixInstr,
) where

import Additional
import Lambda
import MMIX.Syntax

infix 6 `shiftMMixVar`

\end{code}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\set{label}		{MMIX/Semantics}
\set{title}		{Semantics of MMIX}
\begin{section}

In this work, the meanings of all MMIX programs are described through their translation into Etude, as defined by a Haskell function "meaningOfMMixModule". In particular, the semantic significance of a symbolic module \hbox{“"MMixLET (_S) {-TEXT-} (_T) {-DATA-} (_D) {-IN-} (sigma0)"\!”} is represented by the Etude construct “\,"MODULE (tau) {-EXPORT-} (xi_set) {-WHERE-} (iota_set)"\,\,”, in which the initialiser term "tau" represents an application of the Etude function derived from a jump to the start symbol "sigma0", the export list "xi_set" is extracted from the MMIX symbol table "_S" and the set of item definitions "iota_set" is derived from the module's data, symbol and text segments as follows:

\BeginHaskellToken	{meaningOfMMixModule}			{Open}	{\ZapfChancery{D}\sub{\MKeyword{m}}}
  \HaskellArgument							{MMixModuleT}
  \HaskellResult							{ModuleT MMixVarT}
\EndHaskellToken

\IndexTerm		{meaningOfMMixModule}				{D_M MMIX_module (module)}

\begin{code}
meaningOfMMixModule {-[*]-} :: MMixModuleT -> ModuleT MMixVarT

meaningOfMMixModule {-sem-}(MMixLET (_S) {-TEXT-} (_T) {-DATA-} (_D) {-IN-} (sigma0))
 {--}			= 		{-sem-}(MODULE (tau) {-EXPORT-} (xi_set) {-WHERE-} (iota_set))
 where	tau		= 		{-sem-}(APP (VAR (sigma0 `SR` 0)) (regs))
 	xi_set		= 		listToMap [ (_S#(sigma_k) {-:-}, sigma_k `SR` 0) --
 							| sigma_k <- setToList (dom{--}(_S)), sigma_k @@ (dom{--}(_T) `union` dom{--}(_D)) ]
 	iota_set	= 		listToMap [ (sigma_k `SR` 0 {-:-}, IMPI {-sem-}(_S#(sigma_k))) --
 			     				| sigma_k <- setToList (dom{--}(_S)), sigma_k /@ (dom{--}(_T) `union` dom{--}(_D)) ]
			`mapUnion` 	listToMap [ (sigma_k `SR` 0 {-:-}, _D#(sigma_k)) --
 							| sigma_k <- setToList (dom{--}(_D)) ]
 		 	`mapUnion` 	listToMap [ (sigma_k `SR` 0 {-:-}, FUNI (meaningOfMMixInstr{--}(sigma_k `SR` n_k {->-}, iota_k))) --
 							| sigma_k <- setToList (dom{--}(_T)), (iota_k {-:-}, n_k) <- _T#(sigma_k) `zipList` [ 0, 4 .. ] ]
\end{code}

In particular, the resulting export list "xi_set" includes a subset of the module's symbol table "_S", corresponding to those symbols from the domain of "_S" whose bindings are provided locally by the associated data or text segment. On the other hand, every undeclared symbol "sigma_k @@ dom{--}(_S)" is introduced into the constructed module's set of item definitions "iota_set" with a binding to an imported Etude item “"IMPI x"”, in which "x" represents the symbol's external name as specified in "_S". Further, "iota_set" also includes the bindings of all object items defined in the supplied data segment "_D". Finally, it is also extended with a set of function items derived from the text segment "_T". In particular, each instruction found at some byte offset "n" into one the text sections "sigma_k" in the entire source module materialises in "iota_set" as a binding of the MMIX variable “"sigma_k `SR` 4{--}n"” to an Etude function derived from the instruction's syntax by an algorithm "meaningOfMMixInstr" defined throughout the remainder \hbox{of the present section.}

In the resulting program representation, all Etude variables are depicted universally by values of the following Haskell type:

\IndexType		{MMixVarT}					{MMIX_var (\tstar)}
\IndexCons		{R}						{\$ integer (MMIX_var)}
\IndexCons		{RR}						{rR (MMIX_var)}
\IndexCons		{SR}						{. MMIX_symbol integer (MMIX_var)}

\begin{syntax}
\InsertHaskell		{MMixVarT}
\end{syntax}

Intuitively, variables of the form “"R _N"” are used to denote the 256 general purpose registers provided by every MMIX processor, while the “"sigma `SR` n"” constructs depict symbolic memory locations, or names assigned to various forms of Etude item definitions. Every such variable consists of an MMIX symbol "sigma" that must be associated with some relocatable segment of the surrounding module, together with an integer "n" that represents a byte offset into that segment's memory-resident image. Finally, a designated symbol “"RR"” represents the architecture's «remainder register» mentioned earlier in Section \ref{MMIX/Syntax}.

As implied by the above definition of "meaningOfMMixModule", every MMIX instruction denotes a separate Etude function that is bound to a symbolic variable “"sigma `SR` 4{--}n"”, derived from the instruction's location within the program's eventual address space. The body of such a function represents a monadic term that captures the instruction's operational behaviour. Strictly speaking, only the “\,"MMixTRAP"” instruction, which is mapped to an Etude system call operation, as well as all of the memory access operations such as “\,"MMixLDOUi"\!\!” and “"MMixSTOUi"\!\!”, must be represented by monadic term forms, since all other instructions always denote terms that are reducible to pure atomic expressions lifted into the term monad using a trivial “"RET"” construct. Nevertheless, for the sake of exposition, we will treat all MMIX instructions equally in the following translation.

Further, in the interest of consistency, the denotation of every MMIX instruction accepts the same list of parameters, which includes every conceivable general purpose register "R _N", as well as the remainder register "RR". For conciseness, in the following presentation this parameter list will be always represented collectively by the notation “"pregs"”, which can be presented with a formal mandate by the following Haskell construction:

\NewHaskellToken	{pregs}					{Ord}	{«\pi»}
\NewHaskellToken	{regs}					{Ord}	{«\pi»}

\NewHaskellResult	{pregs}						{ParametersT MMixVarT}
\NewHaskellResult	{regs}						{ParametersT MMixVarT}

\IndexTerm		{pregs}						{\pi (MMIX_vars)}

\begin{code}
pregs :: ParametersT MMixVarT

pregs = [ R 0 .. R 255 ] ++ [ RR ]
\end{code}

\set{capture}		{hidden}
\begin{code}
regs :: AtomsT MMixVarT

regs = coerceList VAR pregs
\end{code}

Given the above definition, a jump to a text segment address denoted by some Etude variable "nu" can be depicted simply as an Etude tail call of the form “"APP (nu) (regs)"”. In a purely-sequential MMIX program, such jumps are performed implicitly at the end of every machine instruction. Specifically, if a given instruction is located at some memory address “"sigma `SR` n"”, then the following instruction is generally represented by the Etude variable “"sigma `SR` {-sem-}(n+4)"”, so that the denotations of most MMIX instructions end in an Etude application term of the form “"APP ({-sem-}(sigma `SR` n `shiftMMixVar` 4)) (regs)"”, in which the notation “"nu `shiftMMixVar` n"” adjusts the existing offset value of the symbolic variable "nu" by "n" bytes as follows:

\BeginHaskellToken	{shiftMMixVar}				{Void}	{}
  \HaskellArgument							{MMixVarT}
  \HaskellToken		{{-(+)-}}				{Bin}	{\oplus}
  \HaskellArgument							{Integer}
  \HaskellResult							{MMixVarT}
\EndHaskellToken  

\NewHaskellToken	{`shiftMMixVar`}			{Bin}	{\oplus}
\IndexTerm		{shiftMMixVar}					{\oplus MMIX_var integer (MMIX_var)}

\begin{code}
shiftMMixVar {-[*]-}	{-(+)-}	{-[*]-} :: MMixVarT -> Integer -> MMixVarT

shiftMMixVar {-sem-}(sigma `SR` n)	{-(+)-} {-sem-}(m)	= {-sem-}(sigma `SR` {-sem-}(n + m))
\end{code}

In particular, the precise semantics of a given MMIX instruction "iota" that is located at some variable address "nu" within the program's address space is represented by the notation “"meaningOfMMixInstr{--}(nu {->-}, iota)"”, which maps every such instruction to an appropriate Etude function with an equivalent operational behaviour. Formally, the algorithm "meaningOfMMixInstr" is defined by a Haskell construction of the following type:

\advance\HaskellLeftSkip by -10pt

\BeginHaskellToken	{meaningOfMMixInstr}			{Open}	{\ZapfChancery{D}\sub{\MKeyword{i}}}
  \HaskellArgument							{(MMixVarT {->-}, MMixInstrT)}
  \HaskellResult							{FunctionT MMixVarT}
\EndHaskellToken

\IndexTerm		{meaningOfMMixInstr}				{D_I MMIX_instr (function)}

\begin{code}
meaningOfMMixInstr {-[*]-} :: (MMixVarT {->-}, MMixInstrT) -> FunctionT MMixVarT
\end{code}

Most simple MMIX operations such as “"MMixInstr (MMixADDU) (R _X) {-,-} (R _Y) {-,-} (R _Z)"\,” denote Etude functions of the form “"FUN (pregs) {-DOT-} $ LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) (op) (phi) (VAR (R _Z))]] $ APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs)"”, in which “"op{-sub-}phi"\!” represents an appropriate binary Etude operator and format. In all cases, "phi" will be set to one of the three standard arithmetic formats “"NG`FMT`64"”, “"ZG`FMT`64"” or “"RG`FMT`64"”, as appropriate for the instruction's interpretation of its operand and result values. In particular, this simple semantic translation is adopted directly by the following thirteen MMIX instruction forms:

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixFADD)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}ADD  ) (RG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixFSUB)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}SUB  ) (RG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixFMUL)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}MUL  ) (RG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixFDIV)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}DIV  ) (RG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixADDU)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}ADD  ) (NG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixSUBU)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}SUB  ) (NG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixMULU)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}MUL  ) (NG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixAND)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}AND  ) (NG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixXOR)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}XOR  ) (NG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixOR)		(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}IOR  ) (NG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixSLU)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}SHL  ) (NG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixSRU)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}SHR  ) (NG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixSR)		(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) ({-hfil-}SHR  ) (ZG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
\end{code}

The reader should observe that, by convention, opcodes whose mnemonics begin with “\,\MKeyword{f}\,” always represent rational or floating point operations, while all remaining instruction forms interpret their arguments as integer values. A few MMIX mnemonics are also supplied in pairs of the form “\MKeyword{«xxx»}\,” and “\MKeyword{«xxx»u}\,”, in which case the earlier variant performs the required operation under the integer format “"ZG`FMT`64"”, while the later assumes the default unsigned integer model of a modulo-"2^64" arithmetic.

On MMIX, the integer division instructions “\,"MMixDIVU"\,” and “\,"MMixDIV"\,” compute simultaneously both the quotient and the remainder from the corresponding arithmetic operation. The later is always stored in the remainder register "RR", so that the formal meanings of these two instruction forms can be captured concisely by the following translations:

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixDIVU)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) (DIV) (NG`FMT`64)  (VAR (R _Z))] {-hv-}, --
						     {-raw-}[RR] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) (REM) (NG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixDIV)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) (DIV) (ZG`FMT`64)  (VAR (R _Z))] {-hv-}, --
		      				     {-raw-}[RR] =: {-hide-}RET {-raw-}[OP2 (VAR (R _Y)) (REM) (ZG`FMT`64)  (VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
\end{code}

Further, the “\,"MMixFCMP"”, “"MMixCMPU"” and “"MMixCMP"” instructions perform comparison of two numeric quantities, returning "negate 1", "0" or "1" whenever their first operand is less than, equal to or greater than the second. Mathematically, this can be expressed as a subtraction operation of the form “"(R _Z < R _Y) - (R _Y < R _Z)"”, assuming the standard encoding of relational operations as the numeric constants 1 and 0 for true and false boolean values, respectively. In Etude, the resulting term structures are represented as follows:

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixFCMP)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 ({-hv-}(OP2 (VAR (R _Z)) (CLT) (RG`FMT`64) (VAR (R _Y)))) --
										(SUB) (ZG`FMT`64) ({-hv-}(OP2 (VAR (R _Y)) (CLT) (RG`FMT`64)  (VAR (R _Z))))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs)) {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixCMPU)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 ({-hv-}(OP2 (VAR (R _Z)) (CLT) (NG`FMT`64) (VAR (R _Y)))) --
										(SUB) (ZG`FMT`64) ({-hv-}(OP2 (VAR (R _Y)) (CLT) (NG`FMT`64)  (VAR (R _Z))))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs)) {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixCMP)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 ({-hv-}(OP2 (VAR (R _Z)) (CLT) (ZG`FMT`64) (VAR (R _Y)))) --
										(SUB) (ZG`FMT`64) ({-hv-}(OP2 (VAR (R _Y)) (CLT) (ZG`FMT`64)  (VAR (R _Z))))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs)) {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-} {-!-}
\end{code}

In order to facilitate more general comparison operations, the architecture also provides a set of six instructions with the mnemonic forms “"MMixZSNi"\!\!”, “"MMixZSZi"\!\!”, “"MMixZSPi"\!\!”, “"MMixZSNNi"\!\!”, “"MMixZSNZi"\!\!” and “"MMixZSNPi"\!\!”. Intuitively, each of these instructions sets the register "R _X" to the integer constant "_Z" if "R _Y" has, respectively, a negative, zero, positive, non-negative, non-zero or non-positive value and, in all other cases, continues evaluation with "R _X" set to zero. Mathematically, this behaviour can be modelled by expressions of the form "(R _Y {-,,,-} op {-,,,-} {-,,-} 0) * _Z", so that the meanings of these six instruction forms are captured in this work as follows:

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixZSNi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) --
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 ({-hv-}(OP2 (VAR (R _Y)) (CLT) (ZG`FMT`64) (IMM (0) (NG`FMT`64)))) --
												(MUL) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixZSZi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 ({-hv-}(OP2 (VAR (R _Y)) (CEQ) (ZG`FMT`64) (IMM (0) (NG`FMT`64)))) --
												(MUL) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixZSPi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 ({-hv-}(OP2 (VAR (R _Y)) (CGT) (ZG`FMT`64) (IMM (0) (NG`FMT`64)))) --
												(MUL) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))

meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixZSNNi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 ({-hv-}(OP2 (VAR (R _Y)) (CGE) (ZG`FMT`64) (IMM (0) (NG`FMT`64)))) --
												(MUL) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixZSNZi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 ({-hv-}(OP2 (VAR (R _Y)) (CNE) (ZG`FMT`64) (IMM (0) (NG`FMT`64)))) --
												(MUL) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixZSNPi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 ({-hv-}(OP2 (VAR (R _Y)) (CLE) (ZG`FMT`64) (IMM (0) (NG`FMT`64)))) --
												(MUL) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
\end{code}

On MMIX, conversions between integers and floating point numbers are facilitated by a set of four opcode mnemonics “\,"MMixFIX"”, “\,"MMixFIXU"\,”,  “\,"MMixFLOT"\,” and “\,"MMixFLOTU"\,”. In all four of the associated instruction forms, the immediate operand "_Y" is used to specify a rounding mode for the operation, but, in this chapter, only those variants of these instructions whose rounding modes correspond precisely to the behaviour expected of the corresponding C operation are formalised, so that their complete translation into Etude is specified by the following six definitions:

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixFIXU)	(_X) {-,-} (1) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP1 (CNV (NG`FMT`64))  (RG`FMT`64)  {--} {-hv-}(VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixFIX)	(_X) {-,-} (1) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP1 (CNV (ZG`FMT`64))  (RG`FMT`64)  {--} {-hv-}(VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixFLOTU)	(_X) {-,-} (0) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP1 (CNV (RG`FMT`64))  (NG`FMT`64)  {--} {-hv-}(VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixFLOT)	(_X) {-,-} (0) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP1 (CNV (RG`FMT`64))  (ZG`FMT`64)  {--} {-hv-}(VAR (R _Z))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
\end{code}

Much of the opcode space in the MMIX instruction set is occupied by operations dedicated to manipulation of memory-resident data. In particular, instructions with opcodes of the form “\,\MKeyword{st«x»u\sub{i}}\!” update the memory location \hbox{"OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64))"} with the least-significant 8, 16, 32 or 64 bits of the specified register "R _X":

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixSTBUi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-hv-}[] =: SET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet) (NG`FMT`8) ---
									{-TO-} (VAR (R _X))] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixSTWUi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-hv-}[] =: SET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet) (NG`FMT`16) ---
									{-TO-} (VAR (R _X))] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixSTTUi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-hv-}[] =: SET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet) (NG`FMT`32) ---
									{-TO-} (VAR (R _X))] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixSTOUi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-hv-}[] =: SET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet) (NG`FMT`64) ---
									{-TO-} (VAR (R _X))] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
\end{code}

Conversely, the eight “\MKeyword{ld«x»u\sub{i}}\!” and “\MKeyword{ld«x»\sub{i}}\!” operations retrieve a 1, 2, 4 or 8-byte object located at the address specified by the sum of the general purpose register "R _Y" and the immediate operand "IMM (toRational _Z) (NG`FMT`64)", interpreting the object's bit pattern as a signed or unsigned integer value that is written directly into the specified register "R _X". Formally:

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixLDBUi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: --
							GET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet) (NG`FMT`8)]  	$ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixLDWUi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: --
							GET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet) (NG`FMT`16)] 	$ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixLDTUi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: --
							GET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet) (NG`FMT`32)] 	$ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixLDOUi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: --
							GET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet) (NG`FMT`64)]  	$ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))

meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixLDBi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: --
							GET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet)  (ZG`FMT`8)] 	$ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixLDWi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: --
							GET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet) (ZG`FMT`16)]  	$ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixLDTi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: --
							GET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet) (ZG`FMT`32)]  	$ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixLDOi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: --
							GET (OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)) {-hide-}, {-hide-}emptySet) (ZG`FMT`64)]  	$ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
\end{code}

The “"MMixGET"\,” and “\,"MMixPUT"\,” instructions provide access to the 32 special purpose registers of MMIX. In this chapter, however, we are only ever interested in the sixth such register "RR", so that only the following two translation rules are required for a complete implementation of our verified C compiler:

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixGET)	({-hfill-}_X  ) {-,-} (0) {-,-} ({-hfill-}6   ) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] 	 =: {-hide-}RET {-raw-}[VAR (RR)]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixPUT)	({-hfill-}6   ) {-,-} (0) {-,-} ({-hfill-}_Z  ) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[RR] 	 =: {-hide-}RET {-raw-}[VAR (R _Z)]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
\end{code}

The “"MMixGOi"\!\!” instruction performs an unconditional jump to a memory location determined dynamically by the sum of its two operands "R _Y" and "IMM (_Z) (NG`FMT`64)". In all cases, the address of the following instruction "nu `shiftMMixVar` 4" is also stored in the specified general purpose register "R _X", for use as the continuation term once the targeted MMIX procedure has completed its job:

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixGOi)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[VAR {-sem-}(nu `shiftMMixVar` 4)]] $ --
						APP ({-hv-}(OP2 (VAR (R _Y)) (ADD) (NG`FMT`64) (IMM (toRational _Z) (NG`FMT`64)))) (regs))
\end{code}

On the other hand, a related «branch instruction» with the mnemonic “\,"MMixBZ"\,” selects one of two possible addresses for its continuation term, based on a value specified in the operand "R _X". In particular, if that register contains the value of zero, then the instruction performs a jump to an address determined from the sum of its own location "nu" and the 16-bit offset "4#*#{--}(2^8#*#{--}_Y + _Z)" as specified by its immediate operands "_Y" and "_Z". Otherwise, execution will continue normally at the following address "nu `shiftMMixVar` 4". In Etude, this construction can be represented as follows:

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixBZ)		(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing))
	= {-sem-}(FUN (pregs) {-DOT-} $		IFF (OP2 (VAR (R _X)) (CEQ) (NG`FMT`64) (IMM (0) (NG`FMT`64))) {-THEN-} --
						(APP (VAR {-sem-}(nu `shiftMMixVar` {-raw-}(4#*#{--}(2^8#*#{--}_Y + _Z)))) (regs)) --
						{-ELSE-} --
						(APP (VAR {-sem-}(nu `shiftMMixVar` 4)) (regs))) {-!-}{-!-}{-!-}{-!-}{-!-}
\end{code}

The actual location of the following instruction can be also obtained using the MMIX operation “"MMixGETA"”, which stores the numeric value of its address directly in the specified general purpose register "R _X":

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixGETA)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[VAR {-sem-}(nu `shiftMMixVar` {-raw-}(4#*#{--}(2^8#*#{--}_Y + _Z)))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
\end{code}

The reader will observe that all of the MMIX instruction forms described so far have operated solely on variable operands and small non-negative constants representable as a single unsigned byte value. To facilitate an efficient synthesis of larger numeric quantities, MMIX provides a block of four instructions with the mnemonics of “"MMixSETL"”, “"MMixINCML"”, “"MMixINCML"”, “"MMixINCMH"\,” and “"MMixINCH"\,”. All of these instructions accept a pair of immediate operands "_Y" and "_Z", interpreted as a single unsigned integer "_YZ" with a value of the form "2^k {--} (2^8 {--} _Y + _Z)", where "k" is equal to 0, 16, 32 or 48 for the “\,\MKeyword{l}”, “\,\MKeyword{ml}”, “\,\MKeyword{mh}\,” and “\,\MKeyword{h}\,” variants of these instructions, respectively. In particular, the “"MMixSETL"” operation sets "R _X" to the value of this constant directly, while the three “\MKeyword{incxx}\,” instruction forms update "R _X" with the sum of its existing value and the constant "2^k {--} (2^8 {--} _Y + _Z)":

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixSETL)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[IMM (toRational {-sem-}(2^8 #*#_Y + _Z)) (NG`FMT`64)]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixINCML)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _X)) (ADD) (NG`FMT`64) --
							(IMM (toRational {-sem-}(2^16 #*# {--} (2^8 #*#_Y + _Z))) (NG`FMT`64))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixINCMH)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _X)) (ADD) (NG`FMT`64) --
							(IMM (toRational {-sem-}(2^32 #*# {--} (2^8 #*#_Y + _Z))) (NG`FMT`64))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixINCH)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _X)) (ADD) (NG`FMT`64) --
							(IMM (toRational {-sem-}(2^48 #*# {--} (2^8 #*#_Y + _Z))) (NG`FMT`64))]] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
\end{code}

The above four instruction forms also accept an optional fourth symbolic operand, permitting MMIX programs to perform explicit arithmetic operations on the concrete values of any symbolic variables introduced into its modules by a later linking stage of compilation. In particular, if one of these instructions is annotated with a given symbol "sigma", then the register "R _X" is updated with an appropriate 16-bit subset of a value assigned by the linker to the MMIX variable "sigma `SR` n", in which the offset "n" is equal to the constant "2^k {--} (2^8 {--} _Y + _Z)", determined by the instruction's immediate operands "_Y" and "_Z" in a manner appropriate for its opcode. Formally:

\begin{code}
meaningOfMMixInstr  {-sem-}(nu {->-}, MMixInstr (MMixSETL)	(_X) {-,-} (_Y) {-,-} (_Z) {-@-} (Just sigma))
	= 	  {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (sigma `SR` {-sem-}(2^8 #*#_Y + _Z))) --
								(AND) (NG`FMT`64) (IMM (toRational {-sem-}(2^16 - 2^0)) (NG`FMT`64))]] $ --
							APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr  {-sem-}(nu {->-}, MMixInstr (MMixINCML)	(_X) {-,-} (_Y) {-,-} (_Z) {-@-} (Just sigma))
	= 	  {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _X)) (ADD) (NG`FMT`64) --
									({-hv-}(OP2 (VAR (sigma `SR` {-sem-}(2^16 #*# {--} (2^8 #*#_Y + _Z)))) --
								(AND) (NG`FMT`64) (IMM (toRational {-sem-}(2^32 - 2^16)) (NG`FMT`64))))]] $ --
							APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr  {-sem-}(nu {->-}, MMixInstr (MMixINCMH)	(_X) {-,-} (_Y) {-,-} (_Z) {-@-} (Just sigma))
	= 	  {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _X)) (ADD) (NG`FMT`64) --
									({-hv-}(OP2 (VAR (sigma `SR` {-sem-}(2^32 #*# {--} (2^8 #*#_Y + _Z)))) --
								(AND) (NG`FMT`64) (IMM (toRational {-sem-}(2^48 - 2^32)) (NG`FMT`64))))]] $ --
							APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
meaningOfMMixInstr  {-sem-}(nu {->-}, MMixInstr (MMixINCH)	(_X) {-,-} (_Y) {-,-} (_Z) {-@-} (Just sigma))
	= 	  {-sem-}(FUN (pregs) {-DOT-} $		LET [{-raw-}[R _X] =: {-hide-}RET {-raw-}[OP2 (VAR (R _X)) (ADD) (NG`FMT`64) --
									({-hv-}(OP2 (VAR (sigma `SR` {-sem-}(2^32 #*# {--} (2^8 #*#_Y + _Z)))) --
								(AND) (NG`FMT`64) (IMM (toRational {-sem-}(2^64 - 2^48)) (NG`FMT`64))))]] $ --
							APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))						
\end{code}

Last but not least, an MMIX operation of the form \hbox{“"MMixInstr (MMixTRAP) (_X) {-,-} (_Y) {-,-} (_Z)"”} denotes an invocation of the operating system facility identified by a 24-bit integer encoded in the instruction's immediate operands "_X", "_Y" and "_Z" as follows:

\begin{code}
meaningOfMMixInstr {-sem-}(nu {->-}, MMixInstr (MMixTRAP)	(_X) {-,-} (_Y) {-,-} (_Z) {-hide-}(Nothing)) ---
	= {-sem-}(FUN (pregs) {-DOT-} $		LET [pregs =: SYS ({-sem-}(2^16#*#{--}_X + 2^8#*#{--}_Y + _Z)) (regs)] $ --
						APP (VAR ({-sem-}(nu `shiftMMixVar` 4))) (regs))
\end{code}

The reader should observe that, on MMIX, all such system calls are always assumed to return a new value for every available MMIX register. In the remainder of this chapter, “\,"MMixTRAP"” instructions will receive only a superficial treatment, since their satisfactory formalisation would inherently involve a detailed specification of the entire operating system leveraged by the program under semantic scrutiny.

None of the remaining 206 MMIX opcodes are required for the simple translation of Etude programs into their MMIX counterparts presented later in Section \ref{MMIX/Lambda}, although many of these omitted instruction forms could be easily utilised by a smarter version of our compiler in order to improve, sometimes dramatically, an operational efficiency of the resulting programs. For example, most of the above instruction forms are also accompanied by analogous operations on immediate operand values, alleviating the need for a continuous synthesis of many simple numeric constants using the “"MMixSETL"” instruction family. Nevertheless, in this work, I avoid a formal specification and utilisation of all such instructions, in order to focus the present discussion on the actual challenge of compiler verification.

\end{section}

%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	%%%%%%%	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%